/*
 *      ircengine.h
 *
 *      Copyright 2010 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef IRCENGINE_H
#define IRCENGINE_H

#ifndef DXSOCKET_H
#include "dxsocket.h"
#endif
#include "defs.h"
#include "config.h"
#include "dxutils.h"

class IrcEngine: public FXObject
{
    FXDECLARE(IrcEngine)
    friend class IrcTabItem;
    public:
        IrcEngine(FXApp*, FXObject*, FXString, FXString);
        virtual ~IrcEngine();

        void startConnection();
        void disconnect();
        void disconnect(const FXString &reason);
        void reconnect();
        void closeConnection(FXbool disableReconnect=FALSE);
        void appendTarget(FXObject*);
        void removeTarget(FXObject*);
        FXbool findTarget(FXObject*);
        FXbool clearTarget();
        void clearAttempts() { m_attempts = 0; }
        void setServerName(const FXString &name) { m_serverName = name; }
        FXString getServerName() const { return m_serverName; }
        FXString getNetworkName() const { return m_networkName; }
        void setServerPort(FXint port) { m_serverPort = port; }
        FXint getServerPort() const { return m_serverPort; }
        void setServerPassword(const FXString &pass) { m_serverPassword = pass; }
        void setNickName(const FXString &nick) { m_nickName = nick; }
        FXString getNickName() const { return m_nickName; }
        void setUserName(const FXString &user);
        void setRealName(const FXString &realname) { m_realName = realname; }
        void setStartChannels(const FXString &channels) { m_startChannels = dxutils::removeSpaces(channels); }
        void setStartCommands(const FXString &commands) { m_startCommands = commands; }
        void setUseSsl(const FXbool &usessl) { m_useSsl = usessl; }
        FXbool getConnected() const { return m_connected; }
        FXbool getConnecting() const { return m_connecting; }
        FXbool getUseSsl() const { return m_useSsl; }
        FXint getNickLen() const { return m_nickLen; }
        FXint getTopicLen() const { return m_topicLen; }
        FXint getKickLen() const { return m_kickLen; }
        FXint getAwayLen() const { return m_awayLen; }
        FXchar getAdminPrefix() const { return m_adminPrefix; }
        FXchar getOwnerPrefix() const { return m_ownerPrefix; }
        FXchar getOpPrefix() const { return m_opPrefix; }
        FXchar getVoicePrefix() const { return m_voicePrefix; }
        FXchar getHalfopPrefix() const { return m_halfopPrefix; }
        FXString getChanTypes() const { return m_chanTypes; }
        FXString getLocalIP() { return m_socket->getLocalIP(); }
        FXString getLocalIPBinary();
        FXString getHostname(const FXString &address);
        FXString getRemoteIP() { return m_socket->getRemoteIP(); }
        void addIgnoreWho(const FXString &who);
        void addNick(const FXString &nick, const FXString &user, const FXString &real, const FXString &host, const FXbool &away);
        void addNick(const FXString &nick, const FXString &user, const FXString &real, const FXString &host);
        void removeNick(const FXString &nick);
        NickInfo getNickInfo(const FXString &nick) const;
        FXString getBannedNick(const FXString &banmask) const;
        dxStringArray getChannelsList() const;
        void clearChannelsList();
        FXbool sendAdmin(const FXString &params);
        FXbool sendAway(const FXString &params);
        FXbool sendBanlist(const FXString &channel);
        FXbool sendCtcp(const FXString &to, const FXString &params);
        FXbool sendCtcpNotice(const FXString &to, const FXString &params);
        FXbool sendMode(const FXString &params);
        FXbool sendInvite(const FXString &to, const FXString &params);
        FXbool sendJoin(const FXString &chan);
        FXbool sendKick(const FXString &chan, const FXString &nick, const FXString &reason);
        FXbool sendKill(const FXString &nick, const FXString &reason);
        FXbool sendList(const FXString &params);
        FXbool sendMe(const FXString &to, const FXString &message);
        FXbool sendMsg(const FXString &to, const FXString &message);
        /** Send names request
         *
         * @param channel   name of channel
         * @param list      true if command answer could be show
         * @return          true is command was succesfull
         */
        FXbool sendNames(const FXString &channel, FXbool list);
        FXbool sendNick(const FXString &nick);
        FXbool sendNotice(const FXString &to, const FXString &message);
        FXbool sendOper(const FXString &login, const FXString &password);
        FXbool sendPart(const FXString &chan);
        FXbool sendPart(const FXString &chan, const FXString &reason);
        FXbool sendQuote(const FXString &text);
        FXbool sendStats(const FXString &type);
        FXbool sendTopic(const FXString &chan, const FXString &topic);
        FXbool sendTopic(const FXString &chan);
        FXbool sendVersion(const FXString &to);
        FXbool sendWallops(const FXString &msg);
        FXbool sendWho(const FXString &mask);
        FXbool sendWhoami();
        FXbool sendWhois(const FXString &params);
        FXbool sendWhowas(const FXString &params);
        FXbool isUserIgnored(FXString nick, FXString user, FXString host, FXString on);
        FXbool isUserIgnored(FXString nick, FXString on);
        FXbool isAway(const FXString &nick);
        FXString getMyUserHost() const { return m_myUserHost; }

        long onSocketCanRead(FXObject*, FXSelector, void*);
        long onSocketConnected(FXObject*, FXSelector, void*);
        long onSocketDisconnected(FXObject*, FXSelector, void*);
        long onSocketError(FXObject*, FXSelector, void*);
        long onReconnectTimeout(FXObject*, FXSelector, void*);
        long onTryAgainTimeout(FXObject*, FXSelector, void*);
        long onPingTimeout(FXObject*, FXSelector, void*);
    private:
        IrcEngine():m_application(NULL),m_connected(FALSE),m_useSsl(FALSE),m_connecting(FALSE)
        ,m_endmotd(FALSE),m_ignoreUserHost(FALSE),m_serverPort(0),m_attempts(0),m_nickLen(0)
        ,m_topicLen(0),m_kickLen(0),m_awayLen(0),m_adminPrefix(' '),m_ownerPrefix(' '),m_opPrefix(' '),m_voicePrefix(' ')
        ,m_halfopPrefix(' '),m_socket(NULL),m_ping(0)
        {}

        FXApp *m_application;
        FXbool m_connected, m_useSsl, m_connecting, m_endmotd;
        FXbool m_ignoreUserHost, m_storeChannelList;
        FXint m_serverPort, m_attempts;
        FXint m_nickLen, m_topicLen, m_kickLen, m_awayLen;
        FXString m_serverName, m_networkName, m_serverPassword, m_nickName;
        FXString m_realName, m_userName, m_startChannels, m_startCommands;
        FXString m_receiveRest;
        FXString m_chanTypes; //channel prefixes
        FXString m_myUserHost; //usefull for dcc
        FXString m_ignoreWho;
        FXchar m_adminPrefix, m_ownerPrefix, m_opPrefix, m_voicePrefix, m_halfopPrefix; //prefix for nick modes
        dxTargetsArray m_targets;
        dxStringArray m_ignoreWhoQueue, m_namesQueue, m_channelsList;
        dxNickInfoArray m_nicks;
        dxSocket *m_socket;
        FXTime m_ping;

        void readData();
        FXbool sendLine(const FXString &line);
        void sendCommands();
        FXbool sendCommand(const FXString &commandtext);
        void parseLine(const FXString &line);
        void sendEvent(IrcEventType);
        void sendEvent(IrcEventType, const FXString&);
        void sendEvent(IrcEventType, const FXString&, const FXString&);
        void sendEvent(IrcEventType, const FXString&, const FXString&, const FXString&);
        void sendEvent(IrcEventType, const FXString&, const FXString&, const FXString&, const FXString&);
        void numeric(const FXint&, const FXString&);
        void privmsg(const FXString&, const FXString&);
        void ctcp(const FXString&, const FXString&);
        void join(const FXString&, const FXString&);
        void quitirc(const FXString&, const FXString&);
        void part(const FXString&, const FXString&);
        void ping(const FXString&);
        void pong(const FXString&, const FXString&);
        void notice(const FXString&, const FXString&);
        void nick(const FXString&, const FXString&);
        void topic(const FXString&, const FXString&);
        void invite(const FXString&, const FXString&);
        void kick(const FXString&, const FXString&);
        void mode(const FXString&, const FXString&);
        void error(const FXString&);
        void unknown(const FXString&, const FXString&);
        void parseRplsupport(FXString);
        void makeStartChannels();
        void clearChannelsCommands(FXbool);
        void resetReconnect();
        void sendPingTimeout();
        bool hasReconnectAttempt();
};

#endif // IRCENGINE_H
