/*
 *      boatstabitem.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef BOATSTABITEM_H
#define BOATSTABITEM_H

#include "engine/ircengine.h"
#include "dxtabitem.h"
#include "defs.h"
#include "fxext.h"

static const FXint bcolumns = 10;
static const FXint brows = 10;
static const FXint boats = 10;

enum CELLTYPE {
    CLEAR = 0,
    BOAT,
    FIRE,
    WATER,
    HIT,
    PLACED
};

struct BoatsCell
{
    BoatsCell(FXint x1 = -1, FXint y1 = -1, CELLTYPE type1 = CLEAR)
    :   x(x1),
        y(y1),
        type(type1)
    {
    }

    bool operator==(const BoatsCell& cell) const
    {
        return x == cell.x && y == cell.y && type == cell.type;
    }

    FXint x;
    FXint y;
    CELLTYPE type;
};

struct Boat
{
    Boat(FXint x1, FXint y1, FXint cells1, FXbool rotated1)
    :   x(x1),
        y(y1),
        cells(cells1),
        rotated(rotated1)
    {
    }

    Boat()
    : x(-1),
      y(-1),
      cells(0),
      rotated(FALSE)
    {
    }
    
    bool operator ==(const Boat& boat) const
    {
        return x == boat.x && y == boat.y && cells == boat.cells
                && rotated == boat.rotated;
    }
    
    bool isValid() const
    {
        return x != -1 && y != -1;
    }
    
    bool constainsCell(FXint x1, FXint y1)
    {
        if(!isValid())
            return false;
        if(rotated)
        {
            return x1 == x && y1 >= y && y1 < y+cells;
        }
        else
        {
            return y1 == y && x1 >= x && x1 < x+cells;
        }
    }
    
    FXint x;
    FXint y;
    FXint cells;
    FXbool rotated;
};

class dxirc;

class BoatsTabItem: public dxTabItem
{
    FXDECLARE(BoatsTabItem)
    friend class dxirc;
public:
    BoatsTabItem(dxTabBook *tab, const FXString &tabtext, FXbool first, FXIcon *icon=0, FXuint opts=TAB_TOP_NORMAL, FXint id=0, IrcEngine *socket=NULL);
    virtual ~BoatsTabItem();

    void createGeom();
    FXString getServerName() { return m_engine ? m_engine->getServerName() : ""; }
    FXint getServerPort() { return m_engine ? m_engine->getServerPort() : 0; }
    FXString getNickName() { return m_engine ? m_engine->getNickName() : ""; }
    FXString getRealServerName() { return m_engine ? m_engine->getNetworkName() : ""; }
    TYPE getType() { return m_type; }
    void reparentTab();
    void leftGame();

    long onIrcEvent(FXObject *, FXSelector, void*);
    long onPaint(FXObject *, FXSelector, void*);
    long onMotion(FXObject *, FXSelector, void*);
    long onLeftClick(FXObject *, FXSelector, void*);
    long onRightClick(FXObject *, FXSelector, void*);
    long onNewGame(FXObject *, FXSelector, void*);
protected:
    IrcEngine *m_engine;
private:
    BoatsTabItem():m_engine(NULL),m_parent(NULL),m_type(BOATS),m_gameframe(NULL),m_mainframe(NULL)
    ,m_statusframe(NULL),m_gamecanvas(NULL),m_status(NULL),m_newGame(NULL),m_cell(0),m_first(FALSE)
    ,m_placeBoats(FALSE),m_rotated(FALSE),m_previousX(0),m_previousY(0),m_cells(0),m_onFire(FALSE)
    ,m_ready(FALSE)
    {}

    dxTabBook *m_parent;
    TYPE m_type;
    FXVerticalFrame *m_gameframe, *m_mainframe;
    FXHorizontalFrame *m_statusframe;
    FXCanvas *m_gamecanvas;
    FXLabel *m_status;
    FXButton *m_newGame;
    FXint m_cell;
    BoatsCell m_mycells[bcolumns][brows];
    BoatsCell m_hiscells[bcolumns][brows];
    Boat m_boats[boats];
    FXbool m_first; //I fire first
    FXbool m_placeBoats;
    FXbool m_rotated; //Place rotated boat
    FXint m_previousX; //Placed boat
    FXint m_previousY; //Placed boat
    FXint m_cells; //Placed boat
    FXbool m_onFire;
    FXbool m_ready;

    void placeBoats();
    FXColor getCellColor(CELLTYPE type);
    void drawCell(FXint x, FXint y, FXbool my=TRUE);
    void drawCells();
    FXbool isRightForPlaceBoat(FXint x, FXint y);
    void fire(FXint x, FXint y);
    void clearCellsAndBoats();
    FXbool boatDestroyed(FXint x, FXint y);
    FXbool gameOver();
};

#endif // BOATSTABITEM_H
