/*
 *      configdialog.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QDialogButtonBox>
#include <QCheckBox>
#include <QDir>
#include <QDesktopServices>
#include <QFileDialog>
#include <QGroupBox>
#include <QFormLayout>
#include <QMessageBox>
#include <QFontDialog>
#include "configdialog.h"
#include "userdialog.h"
#include "keydialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

#define HEADER "# dxirc smiley settings\n# path is relative to file\n# Example: /home/xxx/smile.png=:)\n"

SmileyDialog::SmileyDialog(const QString &title, const QString &smiley, const QString &path, QWidget *parent)
    : QDialog(parent)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    QFormLayout *form = new QFormLayout;
    m_smiley = new QLineEdit;
    m_smiley->setText(smiley);
    form->addRow(_("Smiley:"), m_smiley);
    QHBoxLayout *pathl = new QHBoxLayout;
    m_path = new QLabel(path);
    m_path->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    m_selectPath = new QPushButton("...");
    connect(m_selectPath, SIGNAL(clicked()), this, SLOT(slotPath()));
    pathl->addWidget(m_path);
    pathl->addWidget(m_selectPath);
    form->addRow(_("Path:"), pathl);
    m_preview = new QLabel;
    m_preview->setPixmap(QPixmap(path));
    form->addRow(_("Preview:"), m_preview);
    mainLayout->addLayout(form);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(title);
}

void SmileyDialog::slotPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString file = QFileDialog::getOpenFileName(this,
                                _("Select file"),
                                m_path->text(),
                                "",
                                NULL,
                                options);
    if(!file.isEmpty())
    {
        m_path->setText(file);
        m_preview->setPixmap(QPixmap(file));
    }
}

ConfigDialog::ConfigDialog(QWidget *parent) :
    QDialog(parent)
{
    m_showImportwarning = true;
    m_ignored = preferences.m_ignoreUsersList;
    m_friends = preferences.m_friendsList;
    m_commands = preferences.m_ignoreCommandsList;
    QTabWidget *tabbook = new QTabWidget(this);
    createGeneralTab(tabbook);
    createLookTab(tabbook);
    createIrctextTab(tabbook);
    createNicklistTab(tabbook);
    createIgnoreTab(tabbook);
    createDccTab(tabbook);
    createNotificationTab(tabbook);
    createSmileysTab(tabbook);
    createKeybindingsTab(tabbook);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    buttons->addButton(new QPushButton(_("&Save&&Close")), QDialogButtonBox::AcceptRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabbook);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(_("Preferences"));
}

void ConfigDialog::accept()
{
    //GeneralTab
    preferences.m_oneInstance = m_oneInstance->isChecked();
    preferences.m_disableSortTabs = m_disableSort->isChecked();
    preferences.m_reconnect = m_reconnect->isChecked();
    preferences.m_numberAttempt = m_numberAttempt->value();
    preferences.m_delayAttempt = m_delayAttempt->value();
    preferences.m_maxAway = m_maxAway->value();
    preferences.m_closeToTray = m_closeToTray->isChecked();
#ifdef HAVE_ENCHANT
    preferences.m_useSpell = m_useSpell->isChecked();
#endif
    preferences.m_ownServerWindow = m_ownServerWindow->isChecked();
    preferences.m_logging = m_logging->isChecked();
    preferences.m_logPath = m_logPath->text();
#ifdef HAVE_LUA
    preferences.m_autoload = m_autoload->isChecked();
    preferences.m_autoloadPath = m_autoloadPath->text();
#endif
    //IconsTab
    preferences.m_themePath = m_themesRoleList->currentItem()->text();
    preferences.m_themesList.clear();
    for(int i=0; i<m_themesRoleList->count(); i++)
        preferences.m_themesList.append(m_themesRoleList->item(i)->text()+";");
    preferences.m_sortUserMode = m_sortUserMode->isChecked();
    //LookTab
    preferences.m_unreadColor = m_unreadtabColor->getColor();
    preferences.m_highlightColor = m_hltabColor->getColor();
    preferences.m_tabPosition = m_tabPosition->currentIndex();
    //IrcTextTab
    preferences.m_colors.text = m_textColor->getColor();
    preferences.m_colors.back = m_backroundColor->getColor();
    preferences.m_colors.notice = m_noticeColor->getColor();
    preferences.m_colors.user = m_userColor->getColor();
    preferences.m_colors.action = m_actionColor->getColor();
    preferences.m_colors.error = m_errorColor->getColor();
    preferences.m_colors.hilight = m_hilightColor->getColor();
    preferences.m_colors.link = m_linkColor->getColor();
    preferences.m_colors.mymsg = m_mymsgColor->getColor();
    preferences.m_colors.log = m_logColor->getColor();
    QFont font;
    font.fromString(m_ircFont->text());
    preferences.m_ircFont = font;
    preferences.m_sameCmd = m_sameCmd->isChecked();
    preferences.m_sameList = m_sameList->isChecked();
    preferences.m_nickCompletionChar = m_nickCompletionChar->text().left(1);
    preferences.m_coloredNick = m_coloredNick->isChecked();
    preferences.m_stripColors = m_stripColors->isChecked();
    preferences.m_loadLog = m_loadLog->isChecked();
    preferences.m_loadLines = m_loadLines->value();
    //NotificationTab
    preferences.m_friendsList = m_friends;
    preferences.m_sounds = m_sounds->isChecked();
    preferences.m_soundConnect = m_soundConnect->isChecked();
    preferences.m_pathConnect = m_pathConnect->text();
    preferences.m_soundDisconnect = m_soundDisconnect->isChecked();
    preferences.m_pathDisconnect = m_pathDisconnect->text();
    preferences.m_soundMessage = m_soundMessage->isChecked();
    preferences.m_pathMessage = m_pathMessage->text();
    preferences.m_notify = m_notify->isChecked();
    if(!dxUtils.haveNotify())
        preferences.m_notifyPosition = m_notifyPosition->currentIndex();
    preferences.m_notifyConnect = m_notifyConnect->isChecked();
    preferences.m_notifyDisconnect = m_notifyDisconnect->isChecked();
    preferences.m_notifyMessage = m_notifyMessage->isChecked();
    preferences.m_notifyTray = m_notifyTray->isChecked();
    //SmileysTab
    preferences.m_useSmileys = m_useSmileys->isChecked();
    preferences.m_smileysMap = m_smileysMap;
    //IgnoreTab
    preferences.m_ignoreCommandsList = m_commands;
    preferences.m_ignoreUsersList = m_ignored;
    //DCCTab
    preferences.m_dccPath = m_dccPath->text();
    preferences.m_dccIP = m_dccIP->text();
    preferences.m_dccPortD = m_dccPortD->value();
    preferences.m_dccPortH = m_dccPortH->value();
    preferences.m_autoDccChat = m_autoDccChat->isChecked();
    preferences.m_autoDccFile = m_autoDccFile->isChecked();
    preferences.save();
    QDialog::accept();
}

void ConfigDialog::createGeneralTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    m_oneInstance = new QCheckBox(_("Run one instance"));
    m_oneInstance->setChecked(preferences.m_oneInstance);
    main->addWidget(m_oneInstance);
    m_disableSort = new QCheckBox(_("Disable tabs sorting"));
    m_disableSort->setChecked(preferences.m_disableSortTabs);
    main->addWidget(m_disableSort);
    m_reconnect = new QCheckBox(_("Reconnect after disconnection"));
    m_reconnect->setChecked(preferences.m_reconnect);
    connect(m_reconnect, SIGNAL(stateChanged(int)), this, SLOT(slotReconnect()));
    main->addWidget(m_reconnect);
    QHBoxLayout *number = new QHBoxLayout;
    number->addWidget(new QLabel(_("Number of attempts (zero for unlimited)")));
    m_numberAttempt = new QSpinBox;
    m_numberAttempt->setRange(0,20);
    m_numberAttempt->setValue(preferences.m_numberAttempt);
    if(!preferences.m_reconnect) m_numberAttempt->setEnabled(false);
    number->addWidget(m_numberAttempt);
    number->addStretch(1);
    main->addLayout(number);
    QHBoxLayout *delay = new QHBoxLayout;
    delay->addWidget(new QLabel(_("Delay between two attempts in seconds")));
    m_delayAttempt = new QSpinBox;
    m_delayAttempt->setRange(5,120);
    m_delayAttempt->setValue(preferences.m_delayAttempt);
    if(!preferences.m_reconnect) m_delayAttempt->setEnabled(false);
    delay->addWidget(m_delayAttempt);
    delay->addStretch(1);
    main->addLayout(delay);
    QHBoxLayout *away = new QHBoxLayout;
    QLabel *awayLabel = new QLabel(_("Max. users number for checking away"));
    away->addWidget(awayLabel);
    m_maxAway = new QSpinBox;
    m_maxAway->setRange(0, 1000);
    m_maxAway->setValue(preferences.m_maxAway);
    m_maxAway->setToolTip(_("Too high number can be reason for ban"));
    away->addWidget(m_maxAway);
    away->addStretch(1);
    main->addLayout(away);
    m_closeToTray = new QCheckBox(_("Close button hides application"));
    m_closeToTray->setChecked(preferences.m_closeToTray);
    main->addWidget(m_closeToTray);
#ifdef HAVE_ENCHANT
    m_useSpell = new QCheckBox(_("Use spellchecking"));
    if(!dxUtils.getAvailableSpellLangsNum())
    {
        m_useSpell->setEnabled(false);
        preferences.m_useSpell = false;
    }
    m_useSpell->setChecked(preferences.m_useSpell);
    main->addWidget(m_useSpell);
#endif
    m_ownServerWindow = new QCheckBox(_("Special tab for server messages"));
    m_ownServerWindow->setChecked(preferences.m_ownServerWindow);
    main->addWidget(m_ownServerWindow);
    m_logging = new QCheckBox(_("Logging chats"));
    m_logging->setChecked(preferences.m_logging);
    connect(m_logging, SIGNAL(stateChanged(int)), this, SLOT(slotLogging()));
    main->addWidget(m_logging);
    QHBoxLayout *loglayout = new QHBoxLayout;
    loglayout->addWidget(new QLabel(_("Log path")));
    m_logPath = new QLabel;
    if(!QDir(preferences.m_logPath).exists())
    {
#if(QT_VERSION > 0x050000)
        m_logPath->setText(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
        m_logPath->setToolTip(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
#else
        m_logPath->setText(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
        m_logPath->setToolTip(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
#endif
    }
    else
    {
        m_logPath->setText(preferences.m_logPath);
        m_logPath->setToolTip(preferences.m_logPath);
    }
    m_logPath->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    loglayout->addWidget(m_logPath,1);
    m_setLogPath = new QPushButton("...");
    if(!preferences.m_logging) m_setLogPath->setEnabled(false);
    connect(m_setLogPath, SIGNAL(clicked()), this, SLOT(slotLogPath()));
    loglayout->addWidget(m_setLogPath);
    main->addLayout(loglayout);
#ifdef HAVE_LUA
    m_autoload = new QCheckBox(_("Automatically load scripts"));
    m_autoload->setChecked(preferences.m_autoload);
    connect(m_autoload, SIGNAL(stateChanged(int)), this, SLOT(slotAutoload()));
    main->addWidget(m_autoload);
    QHBoxLayout *load = new QHBoxLayout;
    load->addWidget(new QLabel(_("Path")));
    m_autoloadPath = new QLabel;
    if(!QDir(preferences.m_autoloadPath).exists())
    {
#if(QT_VERSION > 0x050000)
        m_autoloadPath->setText(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
        m_autoloadPath->setToolTip(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
#else
        m_autoloadPath->setText(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
        m_autoloadPath->setToolTip(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
#endif
    }
    else
    {
        m_autoloadPath->setText(preferences.m_autoloadPath);
        m_autoloadPath->setToolTip(preferences.m_autoloadPath);
    }
    m_autoloadPath->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    load->addWidget(m_autoloadPath,1);
    m_setAutoloadPath = new QPushButton("...");
    if(!preferences.m_autoload) m_setAutoloadPath->setEnabled(false);
    connect(m_setAutoloadPath, SIGNAL(clicked()), this, SLOT(slotAutoloadPath()));
    load->addWidget(m_setAutoloadPath);
    main->addLayout(load);
#endif
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("&General"));
}

void ConfigDialog::slotLogging()
{
    if(m_logging->isChecked())
    {
        m_setLogPath->setEnabled(true);
        m_loadLog->setEnabled(true);
        m_loadLines->setEnabled(true);
    }
    else
    {
        m_setLogPath->setEnabled(false);
        m_loadLog->setEnabled(false);
        m_loadLines->setEnabled(false);
    }
}

void ConfigDialog::slotLogPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select log directory"),
                                m_logPath->text(),
                                options);
    if(!directory.isEmpty())
    {
        m_logPath->setText(directory);
        m_logPath->setToolTip(directory);
    }
}

void ConfigDialog::slotAutoload()
{
    if(m_autoload->isChecked())
        m_setAutoloadPath->setEnabled(true);
    else
        m_setAutoloadPath->setEnabled(false);
}

void ConfigDialog::slotAutoloadPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select autoload directory"),
                                m_autoloadPath->text(),
                                options);
    if(!directory.isEmpty())
    {
        m_autoloadPath->setText(directory);
        m_autoloadPath->setToolTip(directory);
    }
}

void ConfigDialog::slotDccPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select directory"),
                                m_dccPath->text(),
                                options);
    if(!directory.isEmpty())
    {
        m_dccPath->setText(directory);
        m_dccPath->setToolTip(directory);
    }
}

void ConfigDialog::createNicklistTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QGroupBox *rolesGroup = new QGroupBox(_("Icons themes"));
    QHBoxLayout *rolesLayout = new QHBoxLayout;
    QVBoxLayout *rolesListLayout = new QVBoxLayout;
    m_themesRoleList = new QListWidget;
    m_themesRoleList->addItems(preferences.m_themesList.split(';', QString::SkipEmptyParts));
    for(int i=0; i<m_themesRoleList->count(); i++)
    {
        if(preferences.m_themePath.compare(m_themesRoleList->item(i)->text())==0)
            m_themesRoleList->setCurrentItem(m_themesRoleList->item(i));
    }
    connect(m_themesRoleList, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), this, SLOT(slotThemeRoles(QListWidgetItem*)));
    rolesListLayout->addWidget(m_themesRoleList);
    QGridLayout *rolesIconsLayout = new QGridLayout;
    rolesIconsLayout->addWidget(new QLabel(_("Admin:")),0,0);
    m_iconAdmin = new QLabel();
    rolesIconsLayout->addWidget(m_iconAdmin,0,1);
    rolesIconsLayout->addWidget(new QLabel(_("Owner:")),1,0);
    m_iconOwner = new QLabel();
    rolesIconsLayout->addWidget(m_iconOwner,1,1);
    rolesIconsLayout->addWidget(new QLabel(_("Op:")),2,0);
    m_iconOp = new QLabel();
    rolesIconsLayout->addWidget(m_iconOp,2,1);
    rolesIconsLayout->addWidget(new QLabel(_("Halfop:")),3,0);
    m_iconHalfop = new QLabel();
    rolesIconsLayout->addWidget(m_iconHalfop,3,1);
    rolesIconsLayout->addWidget(new QLabel(_("Voice:")),4,0);
    m_iconVoice = new QLabel();
    rolesIconsLayout->addWidget(m_iconVoice,4,1);
    rolesIconsLayout->addWidget(new QLabel(_("Normal:")),5,0);
    m_iconNormal = new QLabel();
    rolesIconsLayout->addWidget(m_iconNormal,5,1);
    rolesIconsLayout->addWidget(new QLabel(_("Away:")),6,0);
    m_iconAway = new QLabel();
    rolesIconsLayout->addWidget(m_iconAway,6,1);
    slotThemeRoles(m_themesRoleList->currentItem());
    rolesListLayout->addLayout(rolesIconsLayout);
    QVBoxLayout *rolesButtonsLayout = new QVBoxLayout;
    m_themeRoleAdd = new QPushButton(_("Add"));
    connect(m_themeRoleAdd, SIGNAL(clicked()), this, SLOT(slotAddRoleTheme()));
    rolesButtonsLayout->addWidget(m_themeRoleAdd);
    m_themeRoleDelete = new QPushButton(_("Delete"));
    connect(m_themeRoleDelete, SIGNAL(clicked()), this, SLOT(slotDeleteRoleTheme()));
    rolesButtonsLayout->addWidget(m_themeRoleDelete);
    if(m_themesRoleList->count()>1) m_themeRoleDelete->setEnabled(true);
    else m_themeRoleDelete->setEnabled(false);
    rolesButtonsLayout->addStretch(1);
    rolesLayout->addLayout(rolesListLayout);
    rolesLayout->addLayout(rolesButtonsLayout);
    rolesGroup->setLayout(rolesLayout);
    main->addWidget(rolesGroup);
    m_sortUserMode = new QCheckBox(_("Sort nick list by user mode"));
    m_sortUserMode->setChecked(preferences.m_sortUserMode);
    main->addWidget(m_sortUserMode);
    tab->setLayout(main);
    tabbook->addTab(tab, _("Nick &list"));
}

void ConfigDialog::slotThemeRoles(QListWidgetItem *current)
{
    if(!current)
    {
        return;
    }
    if(current->text()!="internal")
    {
        m_iconAway->setPixmap(QPixmap(current->text()+"/irc_away.png"));
        m_iconOwner->setPixmap(QPixmap(current->text()+"/irc_owner.png"));
        m_iconAdmin->setPixmap(QPixmap(current->text()+"/irc_admin.png"));
        m_iconHalfop->setPixmap(QPixmap(current->text()+"/irc_halfop.png"));
        m_iconNormal->setPixmap(QPixmap(current->text()+"/irc_normal.png"));
        m_iconOp->setPixmap(QPixmap(current->text()+"/irc_op.png"));
        m_iconVoice->setPixmap(QPixmap(current->text()+"/irc_voice.png"));
    }
    else
    {
        m_iconAway->setText(" ");
        m_iconOwner->setText("*");
        m_iconAdmin->setText("!");
        m_iconHalfop->setText("%");
        m_iconNormal->setText(" ");
        m_iconOp->setText("@");
        m_iconVoice->setText("+");
    }
}

void ConfigDialog::slotAddRoleTheme()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select theme directory"),
#if(QT_VERSION > 0x050000)
                                QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0),
#else
                                QDesktopServices::storageLocation(QDesktopServices::HomeLocation),
#endif
                                options);
    if(!directory.isEmpty() && QFileInfo(directory+"/irc_normal.png").exists())
    {
        m_themesRoleList->addItem(directory);
    }
    if(m_themesRoleList->count()>1) m_themeRoleDelete->setEnabled(true);
}

void ConfigDialog::slotDeleteRoleTheme()
{
    delete m_themesRoleList->currentItem();
    if(m_themesRoleList->count()==1) m_themeRoleDelete->setEnabled(false);
}

void ConfigDialog::createLookTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QHBoxLayout *layout = new QHBoxLayout;
    QVBoxLayout *colors = new QVBoxLayout;
    m_unreadtabColor = new ColorButton(preferences.m_unreadColor, _("Unreaded Tab Color"));
    connect(m_unreadtabColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotTabColor()));
    m_hltabColor = new ColorButton(preferences.m_highlightColor, _("Highlighted Tab Color"));
    connect(m_hltabColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotTabColor()));
    colors->addWidget(m_unreadtabColor);
    colors->addWidget(m_hltabColor);
    QHBoxLayout *position = new QHBoxLayout;
    m_tabPosition = new QComboBox;
    m_tabPosition->addItem(_("Bottom"));
    m_tabPosition->addItem(_("Left"));
    m_tabPosition->addItem(_("Top"));
    m_tabPosition->addItem(_("Right"));
    m_tabPosition->setCurrentIndex(preferences.m_tabPosition);
    connect(m_tabPosition, SIGNAL(currentIndexChanged(int)), this, SLOT(slotTabPosition()));
    position->addWidget(new QLabel(_("Tab position")));
    position->addWidget(m_tabPosition);
    colors->addLayout(position);
    colors->addStretch(1);
    layout->addLayout(colors);
    m_lookTabs = new TabWidget;
    m_lookTabs->addTab(new QLabel(QString("\t")+_("Standard tab")), "linux");
    m_lookTabs->setTabIcon(0, QIcon(DXIRC_DATADIR "/icons/channel.png"));
    m_lookTabs->addTab(new QLabel(QString("\t")+_("Unreaded tab")), "bsd");
    m_lookTabs->setTabIcon(1, QIcon(DXIRC_DATADIR "/icons/channel.png"));
    m_lookTabs->setTabTextColor(1, preferences.m_unreadColor);
    m_lookTabs->addTab(new QLabel(QString("\t")+_("Highlighted tab")), "hurd");
    m_lookTabs->setTabIcon(2, QIcon(DXIRC_DATADIR "/icons/chnewm.png"));
    m_lookTabs->setTabTextColor(2, preferences.m_highlightColor);
    layout->addWidget(m_lookTabs);
    main->addLayout(layout);
    tab->setLayout(main);
    tabbook->addTab(tab, _("Loo&k"));
    updateTabPosition();
}

void ConfigDialog::slotTabColor()
{
    m_lookTabs->setTabTextColor(1, m_unreadtabColor->getColor());
    m_lookTabs->setTabTextColor(2, m_hltabColor->getColor());
}

void ConfigDialog::slotTabPosition()
{
    updateTabPosition();
}

void ConfigDialog::updateTabPosition()
{
    switch(m_tabPosition->currentIndex()) {
    case 0: //bottom
        m_lookTabs->setTabPosition(QTabWidget::South);
        break;
    case 1: //left
        m_lookTabs->setTabPosition(QTabWidget::West);
        break;
    case 2: //top
        m_lookTabs->setTabPosition(QTabWidget::North);
        break;
    case 3: //right
        m_lookTabs->setTabPosition(QTabWidget::East);
        break;
    default:
        m_lookTabs->setTabPosition(QTabWidget::South);
    }
}

void ConfigDialog::createIrctextTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QHBoxLayout *layout = new QHBoxLayout;
    QVBoxLayout *colors = new QVBoxLayout;
    m_textColor = new ColorButton(preferences.m_colors.text, _("Text color"));
    connect(m_textColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_textColor);
    m_backroundColor = new ColorButton(preferences.m_colors.back, _("Text backround color"));
    connect(m_backroundColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_backroundColor);
    m_userColor = new ColorButton(preferences.m_colors.user, _("User events text color"));
    connect(m_userColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_userColor);
    m_actionColor = new ColorButton(preferences.m_colors.action, _("Actions message text color"));
    connect(m_actionColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_actionColor);
    m_noticeColor = new ColorButton(preferences.m_colors.notice, _("Notice text color"));
    connect(m_noticeColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_noticeColor);
    m_errorColor = new ColorButton(preferences.m_colors.error, _("Error text color"));
    connect(m_errorColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_errorColor);
    m_hilightColor = new ColorButton(preferences.m_colors.hilight, _("Highlight message text color"));
    connect(m_hilightColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_hilightColor);
    m_linkColor = new ColorButton(preferences.m_colors.link, _("Link color"));
    connect(m_linkColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_linkColor);
    m_mymsgColor = new ColorButton(preferences.m_colors.mymsg, _("My message color"));
    connect(m_mymsgColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_mymsgColor);
    m_logColor = new ColorButton(preferences.m_colors.log, _("Log line color"));
    connect(m_logColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotIrcTextColor()));
    colors->addWidget(m_logColor);
    QHBoxLayout *font = new QHBoxLayout;
    m_ircFont = new QLabel(preferences.m_ircFont.toString());
    m_ircFont->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    m_ircFont->setFont(preferences.m_ircFont);
    font->addWidget(m_ircFont);
    m_ircFontModify = new QPushButton("...");
    connect(m_ircFontModify, SIGNAL(clicked()), this, SLOT(slotModifyIrcFont()));
    font->addWidget(m_ircFontModify);
    colors->addLayout(font);
    QHBoxLayout *nick = new QHBoxLayout;
    nick->addWidget(new QLabel(_("Nick completion char")));
    m_nickCompletionChar = new QLineEdit;
    m_nickCompletionChar->setText(preferences.m_nickCompletionChar);
    nick->addWidget(m_nickCompletionChar);
    colors->addLayout(nick);
    m_sameCmd = new QCheckBox(_("Use same font for commandline"));
    m_sameCmd->setChecked(preferences.m_sameCmd);
    colors->addWidget(m_sameCmd);
    m_sameList = new QCheckBox(_("Use same font for user list"));
    m_sameList->setChecked(preferences.m_sameList);
    colors->addWidget(m_sameList);
    m_coloredNick = new QCheckBox(_("Use colored nick"));
    m_coloredNick->setChecked(preferences.m_coloredNick);
    colors->addWidget(m_coloredNick);
    m_stripColors = new QCheckBox(_("Strip colors in text"));
    m_stripColors->setChecked(preferences.m_stripColors);
    colors->addWidget(m_stripColors);
    m_loadLog = new QCheckBox(_("Load logged lines"));
    m_loadLog->setChecked(preferences.m_loadLog);
    connect(m_loadLog, SIGNAL(stateChanged(int)), this, SLOT(slotLoadLog()));
    if(!preferences.m_logging) m_loadLog->setEnabled(false);
    colors->addWidget(m_loadLog);
    QHBoxLayout *lines = new QHBoxLayout;
    lines->addWidget(new QLabel(_("Load max. number lines")));
    m_loadLines = new QSpinBox;
    m_loadLines->setRange(0,500);
    m_loadLines->setValue(preferences.m_loadLines);
    if(!preferences.m_logging || !preferences.m_loadLog) m_loadLines->setEnabled(false);
    lines->addWidget(m_loadLines);
    colors->addLayout(lines);
    colors->addStretch(1);
    layout->addLayout(colors);
    m_ircText = new QTextEdit;
    m_ircText->setReadOnly(true);
    m_ircText->setFont(preferences.m_ircFont);
    updateIrcText();
    layout->addWidget(m_ircText);
    main->addLayout(layout);
    tab->setLayout(main);
    tabbook->addTab(tab, _("&Irc Text"));
}

void ConfigDialog::slotIrcTextColor()
{
    updateIrcText();
}

void ConfigDialog::updateIrcText()
{
    m_ircText->clear();
    QString sheet = QString::fromLatin1("a { text-decoration: underline; color: %1 }"
        ).arg(m_linkColor->getColor().name());
    m_ircText->document()->setDefaultStyleSheet(sheet);
    m_ircText->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }"
                                "a { color:%3; }")
                                .arg(m_textColor->getColor().name())
                                .arg(m_backroundColor->getColor().name())
                                .arg(m_linkColor->getColor().name()));
    m_ircText->append(QString("[1.1.2000 00:00:00] <font color=\"%1\">dvx: msg from history</font>").arg(m_logColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">dvx has joined to #dxirc</font>").arg(m_userColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">dvx: Welcome in dxirc</font>").arg(m_mymsgColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">bm: dvx, dxirc is nice</font>").arg(m_hilightColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">dvx is online</font>").arg(m_actionColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">server's NOTICE: nice notice</font>").arg(m_noticeColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">No error</font>").arg(m_errorColor->getColor().name()));
    m_ircText->append(QString("[00:00:00] <font color=\"%1\">dvx:</font> <a href=\"http://dxirc.org\">http://dxirc.org</a>").arg(m_mymsgColor->getColor().name()));
}

void ConfigDialog::createNotificationTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QGroupBox *friendGroup = new QGroupBox(_("Friends"));
    QHBoxLayout *friendLayout = new QHBoxLayout;
    QStringList headers;
    headers << _("Friend") << _("Channel(s)") << _("Network");
    m_friendsList = new QTreeWidget;
    m_friendsList->setHeaderLabels(headers);
    m_friendsList->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_friendsList->setAlternatingRowColors(true);
    m_friendsList->setRootIsDecorated(false);
    m_friendsList->setSelectionMode(QAbstractItemView::SingleSelection);
    friendLayout->addWidget(m_friendsList);
    QVBoxLayout *friendButton = new QVBoxLayout;
    m_friendAdd = new QPushButton(_("Add"));
    connect(m_friendAdd, SIGNAL(clicked()), this, SLOT(slotAddFriend()));
    friendButton->addWidget(m_friendAdd);
    m_friendModify = new QPushButton(_("Modify"));
    connect(m_friendModify, SIGNAL(clicked()), this, SLOT(slotModifyFriend()));
    friendButton->addWidget(m_friendModify);
    m_friendDelete = new QPushButton(_("Delete"));
    connect(m_friendDelete, SIGNAL(clicked()), this, SLOT(slotDeleteFriend()));
    friendButton->addWidget(m_friendDelete);
    friendButton->addStretch(1);
    if(!m_friends.size())
    {
        m_friendModify->setEnabled(false);
        m_friendDelete->setEnabled(false);
    }
    else
    {
        foreach(UserInfo item, m_friends)
        {
            QTreeWidgetItem *w = new QTreeWidgetItem(m_friendsList);
            w->setText(0, item.nick);
            w->setText(1, item.channel);
            w->setText(2, item.network);
        }
        m_friendsList->setCurrentItem(m_friendsList->invisibleRootItem()->child(0));
    }
    friendLayout->addLayout(friendButton);
    friendGroup->setLayout(friendLayout);
    main->addWidget(friendGroup);
    m_sounds = new QCheckBox(_("Use sounds"));
    m_sounds->setChecked(preferences.m_sounds);
    connect(m_sounds, SIGNAL(stateChanged(int)), this, SLOT(slotSound()));
    main->addWidget(m_sounds);
    QGroupBox *soundGroup = new QGroupBox(_("Events"));
    QVBoxLayout *soundLayout = new QVBoxLayout;
    m_soundConnect = new QCheckBox(_("Friend connected"));
    m_soundConnect->setChecked(preferences.m_soundConnect);
    connect(m_soundConnect, SIGNAL(stateChanged(int)), this, SLOT(slotSoundConnect()));
    soundLayout->addWidget(m_soundConnect);
    QHBoxLayout *connectLayout = new QHBoxLayout;
    m_pathConnect = new QLabel(preferences.m_pathConnect);
    m_pathConnect->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    connectLayout->addWidget(m_pathConnect, 1);
    m_setpathConnect = new QPushButton("...");
    connect(m_setpathConnect, SIGNAL(clicked()), this, SLOT(slotPath()));
    connectLayout->addWidget(m_setpathConnect);
    m_playConnect = new QPushButton(QIcon(DXIRC_DATADIR "/icons/play.png"), "");
    connect(m_playConnect, SIGNAL(clicked()), this, SLOT(slotPlay()));
    connectLayout->addWidget(m_playConnect);
    soundLayout->addLayout(connectLayout);
    m_soundDisconnect = new QCheckBox(_("Friend disconnected"));
    m_soundDisconnect->setChecked(preferences.m_soundDisconnect);
    connect(m_soundDisconnect, SIGNAL(stateChanged(int)), this, SLOT(slotSoundDisconnect()));
    soundLayout->addWidget(m_soundDisconnect);
    QHBoxLayout *disconnectLayout = new QHBoxLayout;
    m_pathDisconnect = new QLabel(preferences.m_pathDisconnect);
    m_pathDisconnect->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    disconnectLayout->addWidget(m_pathDisconnect, 1);
    m_setpathDisconnect = new QPushButton("...");
    connect(m_setpathDisconnect, SIGNAL(clicked()), this, SLOT(slotPath()));
    disconnectLayout->addWidget(m_setpathDisconnect);
    m_playDisconnect = new QPushButton(QIcon(DXIRC_DATADIR "/icons/play.png"), "");
    connect(m_playDisconnect, SIGNAL(clicked()), this, SLOT(slotPlay()));
    disconnectLayout->addWidget(m_playDisconnect);
    soundLayout->addLayout(disconnectLayout);
    m_soundMessage = new QCheckBox(_("Highlighted message or query message"));
    m_soundMessage->setChecked(preferences.m_soundMessage);
    connect(m_soundMessage, SIGNAL(stateChanged(int)), this, SLOT(slotSoundMessage()));
    soundLayout->addWidget(m_soundMessage);
    QHBoxLayout *messageLayout = new QHBoxLayout;
    m_pathMessage = new QLabel(preferences.m_pathMessage);
    m_pathMessage->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    messageLayout->addWidget(m_pathMessage, 1);
    m_setpathMessage = new QPushButton("...");
    connect(m_setpathMessage, SIGNAL(clicked()), this, SLOT(slotPath()));
    messageLayout->addWidget(m_setpathMessage);
    m_playMessage = new QPushButton(QIcon(DXIRC_DATADIR "/icons/play.png"), "");
    connect(m_playMessage, SIGNAL(clicked()), this, SLOT(slotPlay()));
    messageLayout->addWidget(m_playMessage);
    soundLayout->addLayout(messageLayout);
    soundGroup->setLayout(soundLayout);
    main->addWidget(soundGroup);
    QHBoxLayout *notifyuseLayout = new QHBoxLayout;
    m_notify = new QCheckBox(_("Use notify popup"));
    m_notify->setChecked(preferences.m_notify);
    connect(m_notify, SIGNAL(stateChanged(int)), this, SLOT(slotNotify()));
    notifyuseLayout->addWidget(m_notify);
    m_showNotify = new QPushButton(_("Show"));
    connect(m_showNotify, SIGNAL(clicked()), this, SLOT(slotShowNotify()));
    notifyuseLayout->addWidget(m_showNotify);
    notifyuseLayout->addStretch(1);
    main->addLayout(notifyuseLayout);
    if(!dxUtils.haveNotify())
    {
        QHBoxLayout *notifyposLayout = new QHBoxLayout;
        notifyposLayout->addWidget(new QLabel(_("Notify position")));
        m_notifyPosition = new QComboBox;
        m_notifyPosition->addItem(_("Left Top"));
        m_notifyPosition->addItem(_("Right Top"));
        m_notifyPosition->addItem(_("Left Bottom"));
        m_notifyPosition->addItem(_("Right Bottom"));
        if(preferences.m_notifyPosition>=0 && preferences.m_notifyPosition<4) m_notifyPosition->setCurrentIndex(preferences.m_notifyPosition);
        notifyposLayout->addWidget(m_notifyPosition);
        notifyposLayout->addStretch(1);
        main->addLayout(notifyposLayout);
    }
    QGroupBox *notifyGroup = new QGroupBox(_("Events"));
    QVBoxLayout *notifyLayout = new QVBoxLayout;
    m_notifyConnect = new QCheckBox(_("Friend connected"));
    m_notifyConnect->setChecked(preferences.m_notifyConnect);
    notifyLayout->addWidget(m_notifyConnect);
    m_notifyDisconnect = new QCheckBox(_("Friend disconnected"));
    m_notifyDisconnect->setChecked(preferences.m_notifyDisconnect);
    notifyLayout->addWidget(m_notifyDisconnect);
    m_notifyMessage = new QCheckBox(_("Highlighted message or query message"));
    m_notifyMessage->setChecked(preferences.m_notifyMessage);
    notifyLayout->addWidget(m_notifyMessage);
    notifyGroup->setLayout(notifyLayout);
    main->addWidget(notifyGroup);
    m_notifyTray = new QCheckBox(_("Change tray icon for all messages if application in tray"));
    m_notifyTray->setChecked(preferences.m_notifyTray);
    main->addWidget(m_notifyTray);
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("N&otification"));
    slotSound();
    slotNotify();
}

void ConfigDialog::createDccTab(QTabWidget* tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    main->addWidget(new QLabel(_("Directory for saving files:")));
    QHBoxLayout *path = new QHBoxLayout;
    m_dccPath = new QLabel(preferences.m_dccPath);
    m_dccPath->setToolTip(preferences.m_dccPath);
    m_dccPath->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    path->addWidget(m_dccPath,1);
    m_setDccPath = new QPushButton("...");
    connect(m_setDccPath, SIGNAL(clicked()), this, SLOT(slotDccPath()));
    path->addWidget(m_setDccPath);
    main->addLayout(path);
    QHBoxLayout *address = new QHBoxLayout;
    address->addWidget(new QLabel(_("DCC IP address:")));
    QRegExp rx("((1{0,1}[0-9]{0,2}|2[0-4]{1,1}[0-9]{1,1}|25[0-5]{1,1})\\.)"\
               "{3,3}(1{0,1}[0-9]{0,2}|2[0-4]{1,1}[0-9]{1,1}|25[0-5]{1,1})");
    QRegExpValidator *ip = new QRegExpValidator(rx, this);
    m_dccIP = new QLineEdit;
    m_dccIP->setValidator(ip);
    m_dccIP->setText(preferences.m_dccIP);
    address->addWidget(m_dccIP,1);
    main->addLayout(address);
    main->addWidget(new QLabel(_("(Leave blank for use IP address from server)")));
    QHBoxLayout *ports = new QHBoxLayout;
    ports->addWidget(new QLabel(_("DCC ports:")));
    m_dccPortD = new QSpinBox;
    m_dccPortD->setRange(0,65536);
    m_dccPortD->setValue(preferences.m_dccPortD);
    ports->addWidget(m_dccPortD);
    ports->addWidget(new QLabel("-"));
    m_dccPortH = new QSpinBox;
    m_dccPortH->setRange(0,65536);
    m_dccPortH->setValue(preferences.m_dccPortH);
    ports->addWidget(m_dccPortH);
    ports->addStretch(1);
    main->addLayout(ports);
    main->addWidget(new QLabel(_("(Set 0 for use ports from OS)")));
    QHBoxLayout *time = new QHBoxLayout;
    time->addWidget(new QLabel(_("Time for waiting for connection in seconds")));
    m_dccTimeout = new QSpinBox;
    m_dccTimeout->setRange(0,200);
    m_dccTimeout->setValue(preferences.m_dccTimeout);
    time->addWidget(m_dccTimeout);
    time->addStretch(1);
    main->addLayout(time);
    m_autoDccChat = new QCheckBox(_("Automatically connect offered chat"));
    m_autoDccChat->setChecked(preferences.m_autoDccChat);
    main->addWidget(m_autoDccChat);
    m_autoDccFile = new QCheckBox(_("Automatically receive offered file"));
    m_autoDccFile->setChecked(preferences.m_autoDccFile);
    main->addWidget(m_autoDccFile);
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("&DCC"));
}

void ConfigDialog::createIgnoreTab(QTabWidget* tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QGroupBox *commandGroup = new QGroupBox(_("Commands"));
    QHBoxLayout *commandLayout = new QHBoxLayout;
    m_commandsList = new QListWidget;
    m_commandsList->addItems(m_commands);
    commandLayout->addWidget(m_commandsList);
    QVBoxLayout *commandButton = new QVBoxLayout;
    m_commandAdd = new QPushButton(_("Add"));
    connect(m_commandAdd, SIGNAL(clicked()), this, SLOT(slotAddCommand()));
    commandButton->addWidget(m_commandAdd);
    m_commandDelete = new QPushButton(_("Delete"));
    connect(m_commandDelete, SIGNAL(clicked()), this, SLOT(slotDeleteCommand()));
    commandButton->addWidget(m_commandDelete);
    commandButton->addStretch(1);
    if(!m_commands.size())
    {
        m_commandDelete->setEnabled(false);
    }
    commandLayout->addLayout(commandButton);
    commandGroup->setLayout(commandLayout);
    main->addWidget(commandGroup);
    QGroupBox *ignoredGroup = new QGroupBox(_("Users"));
    QHBoxLayout *ignoredLayout = new QHBoxLayout;
    QStringList headers;
    headers << _("User") << _("Channel(s)") << _("Network");
    m_ignoredList = new QTreeWidget;
    m_ignoredList->setHeaderLabels(headers);
    m_ignoredList->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_ignoredList->setAlternatingRowColors(true);
    m_ignoredList->setRootIsDecorated(false);
    m_ignoredList->setSelectionMode(QAbstractItemView::SingleSelection);
    ignoredLayout->addWidget(m_ignoredList);
    QVBoxLayout *ignoredButton = new QVBoxLayout;
    m_ignoredAdd = new QPushButton(_("Add"));
    connect(m_ignoredAdd, SIGNAL(clicked()), this, SLOT(slotAddIgnored()));
    ignoredButton->addWidget(m_ignoredAdd);
    m_ignoredModify = new QPushButton(_("Modify"));
    connect(m_ignoredModify, SIGNAL(clicked()), this, SLOT(slotModifyIgnored()));
    ignoredButton->addWidget(m_ignoredModify);
    m_ignoredDelete = new QPushButton(_("Delete"));
    connect(m_ignoredDelete, SIGNAL(clicked()), this, SLOT(slotDeleteIgnored()));
    ignoredButton->addWidget(m_ignoredDelete);
    ignoredButton->addStretch(1);
    if(!m_ignored.size())
    {
        m_ignoredModify->setEnabled(false);
        m_ignoredDelete->setEnabled(false);
    }
    else
    {
        foreach(UserInfo item, m_ignored)
        {
            QTreeWidgetItem *w = new QTreeWidgetItem(m_ignoredList);
            w->setText(0, item.nick);
            w->setText(1, item.channel);
            w->setText(2, item.network);
        }
        m_ignoredList->setCurrentItem(m_ignoredList->invisibleRootItem()->child(0));
    }
    ignoredLayout->addLayout(ignoredButton);
    ignoredGroup->setLayout(ignoredLayout);
    main->addWidget(ignoredGroup);
    tab->setLayout(main);
    tabbook->addTab(tab, _("Ig&nore"));
}

void ConfigDialog::createKeybindingsTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QPushButton *modify = new QPushButton(QIcon(dxutils::getDataPath()+"/icons/keyboard.png"), _("Modify key bindings..."));
    connect(modify, SIGNAL(clicked()), this, SLOT(slotModifyKeybindings()));
    main->addWidget(modify, 1);
    QPushButton *restore = new QPushButton(QIcon(dxutils::getDataPath()+"/icons/arrow_refresh.png"), _("Restore default key bindings..."));
    connect(restore, SIGNAL(clicked()), this, SLOT(slotRestoreDefaultKeybindings()));
    main->addWidget(restore, 1);
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("Key&bindings"));
}

void ConfigDialog::slotSound()
{
    if(m_sounds->isChecked())
    {
        m_soundConnect->setEnabled(true);
        m_soundDisconnect->setEnabled(true);
        m_soundMessage->setEnabled(true);
    }
    else
    {
        m_soundConnect->setEnabled(false);
        m_soundDisconnect->setEnabled(false);
        m_soundMessage->setEnabled(false);
    }
    slotSoundConnect();
    slotSoundDisconnect();
    slotSoundMessage();
}

void ConfigDialog::slotSoundConnect()
{
    if(m_sounds->isChecked() && m_soundConnect->isChecked())
    {
        m_setpathConnect->setEnabled(true);
        m_playConnect->setEnabled(true);
    }
    else
    {
        m_setpathConnect->setEnabled(false);
        m_playConnect->setEnabled(false);
    }
}

void ConfigDialog::slotSoundDisconnect()
{
    if(m_sounds->isChecked() && m_soundDisconnect->isChecked())
    {
        m_setpathDisconnect->setEnabled(true);
        m_playDisconnect->setEnabled(true);
    }
    else
    {
        m_setpathDisconnect->setEnabled(false);
        m_playDisconnect->setEnabled(false);
    }
}

void ConfigDialog::slotSoundMessage()
{
    if(m_sounds->isChecked() && m_soundMessage->isChecked())
    {
        m_setpathMessage->setEnabled(true);
        m_playMessage->setEnabled(true);
    }
    else
    {
        m_setpathMessage->setEnabled(false);
        m_playMessage->setEnabled(false);
    }
}

void ConfigDialog::slotPath()
{
    QPushButton *btn = qobject_cast<QPushButton*>(sender());
    if(btn == m_setpathConnect)
    {
        QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
        QString file = QFileDialog::getOpenFileName(this,
                                    _("Select file"),
                                    m_pathConnect->text(),
                                    _("Sound file (*.wav)"),
                                    NULL,
                                    options);
        if(!file.isEmpty())
        {
            m_pathConnect->setText(file);
        }
    }
    else if(btn == m_setpathDisconnect)
    {
        QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
        QString file = QFileDialog::getOpenFileName(this,
                                    _("Select file"),
                                    m_pathDisconnect->text(),
                                    _("Sound file (*.wav)"),
                                    NULL,
                                    options);
        if(!file.isEmpty())
        {
            m_pathDisconnect->setText(file);
        }
    }
    else if(btn == m_setpathMessage)
    {
        QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
        QString file = QFileDialog::getOpenFileName(this,
                                    _("Select file"),
                                    m_pathMessage->text(),
                                    _("Sound file (*.wav)"),
                                    NULL,
                                    options);
        if(!file.isEmpty())
        {
            m_pathMessage->setText(file);
        }
    }
}

void ConfigDialog::slotPlay()
{
    QPushButton *btn = qobject_cast<QPushButton*>(sender());
    if(btn == m_playConnect)
        dxutils::playFile(m_pathConnect->text());
    else if(btn == m_playDisconnect)
        dxutils::playFile(m_pathDisconnect->text());
    else if(btn == m_playMessage)
        dxutils::playFile(m_pathMessage->text());
}

void ConfigDialog::slotNotify()
{
    if(m_notify->isChecked())
    {
        if(!dxUtils.haveNotify())
            m_notifyPosition->setEnabled(true);
        m_notifyConnect->setEnabled(true);
        m_notifyDisconnect->setEnabled(true);
        m_notifyMessage->setEnabled(true);
    }
    else
    {
        if(!dxUtils.haveNotify())
            m_notifyPosition->setEnabled(false);
        m_notifyConnect->setEnabled(false);
        m_notifyDisconnect->setEnabled(false);
        m_notifyMessage->setEnabled(false);
    }
}

void ConfigDialog::slotShowNotify()
{
    if(dxUtils.haveNotify())
        dxUtils.showNotify(_("%1 has joined to %2").arg("dvx").arg("#dxirc"));
    else
        dxUtils.showNotify(_("%1 has joined to %2").arg("dvx").arg("#dxirc"), m_notifyPosition->currentIndex());
}

void ConfigDialog::createSmileysTab(QTabWidget* tabbook)
{
    m_smileysMap = preferences.m_smileysMap;
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    m_useSmileys = new QCheckBox(_("Use smileys"));
    m_useSmileys->setChecked(preferences.m_useSmileys);
    connect(m_useSmileys, SIGNAL(clicked()), this, SLOT(slotUseSmileys()));
    main->addWidget(m_useSmileys);
    QHBoxLayout *smileys = new QHBoxLayout;
    m_smileysList = new QListWidget;
    fillSmileys();
    smileys->addWidget(m_smileysList);
    QVBoxLayout *buttons = new QVBoxLayout;
    m_addSmiley = new QPushButton(_("Add"));
    connect(m_addSmiley, SIGNAL(clicked()), this, SLOT(slotAddSmiley()));
    buttons->addWidget(m_addSmiley);
    m_modifySmiley = new QPushButton(_("Modify"));
    connect(m_modifySmiley, SIGNAL(clicked()), this, SLOT(slotModifySmiley()));
    buttons->addWidget(m_modifySmiley);
    m_deleteSmiley = new QPushButton(_("Delete"));
    connect(m_deleteSmiley, SIGNAL(clicked()), this, SLOT(slotDeleteSmiley()));
    buttons->addWidget(m_deleteSmiley);
    m_importSmiley = new QPushButton(_("Import"));
    connect(m_importSmiley, SIGNAL(clicked()), this, SLOT(slotImportSmileys()));
    buttons->addWidget(m_importSmiley);
    m_exportSmiley = new QPushButton(_("Export"));
    connect(m_exportSmiley, SIGNAL(clicked()), this, SLOT(slotExportSmileys()));
    buttons->addWidget(m_exportSmiley);
    buttons->addStretch(1);
    smileys->addLayout(buttons);
    main->addLayout(smileys);
    tab->setLayout(main);
    tabbook->addTab(tab, _("S&mileys"));
    slotUseSmileys();
}

void ConfigDialog::slotUseSmileys()
{
    if(m_useSmileys->isChecked())
    {
        m_addSmiley->setEnabled(true);
        m_importSmiley->setEnabled(true);
        if(m_smileysList->count())
        {
            m_modifySmiley->setEnabled(true);
            m_deleteSmiley->setEnabled(true);
            m_exportSmiley->setEnabled(true);
        }
        else
        {
            m_modifySmiley->setEnabled(false);
            m_deleteSmiley->setEnabled(false);
            m_exportSmiley->setEnabled(false);
        }
    }
    else
    {
        m_addSmiley->setEnabled(false);
        m_modifySmiley->setEnabled(false);
        m_deleteSmiley->setEnabled(false);
        m_importSmiley->setEnabled(false);
        m_exportSmiley->setEnabled(false);
    }
}

void ConfigDialog::slotAddSmiley()
{
    SmileyDialog dialog(_("Add smiley"), "", "");
    if(dialog.exec() == QDialog::Accepted)
    {
        m_smileysMap.insert(dialog.getSmiley(), dialog.getPath());
        fillSmileys();
        m_smileysList->setCurrentRow(m_smileysList->count()-1);
        slotUseSmileys();
    }
}

void ConfigDialog::slotModifySmiley()
{
    QMap<QString,QString>::iterator it = m_smileysMap.find(m_smileysList->currentItem()?m_smileysList->currentItem()->text():m_smileysList->item(0)->text());
    if(it!=m_smileysMap.end())
    {
        SmileyDialog dialog(_("Modify smiley"), it.key(), it.value());
        if(dialog.exec() == QDialog::Accepted)
        {
            m_smileysMap.remove(it.key());
            m_smileysMap.insert(dialog.getSmiley(), dialog.getPath());
            fillSmileys();
            slotUseSmileys();
        }
    }
}

void ConfigDialog::slotDeleteSmiley()
{
    QMap<QString,QString>::iterator it = m_smileysMap.find(m_smileysList->currentItem()?m_smileysList->currentItem()->text():m_smileysList->item(0)->text());
    if(it!=m_smileysMap.end())
    {
        if(QMessageBox::question(this, _("Question"), _("Delete smiley '%1'?").arg(it.key()), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
        {
            m_smileysMap.remove(it.key());
            fillSmileys();
            slotUseSmileys();
        }
    }
}

void ConfigDialog::slotImportSmileys()
{
    if(m_showImportwarning && m_smileysMap.size())
    {
        if(QMessageBox::question(this, _("Warning"), _("File import overwrites current settings"), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Cancel)
        {
            m_showImportwarning = false;
            return;
        }
    }
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString path = QFileDialog::getOpenFileName(this,
                                _("Select file"),
                                m_showImportwarning?DXIRC_DATADIR "/icons/smileys/dxirc.smiley":"",
                                _("Smileys config files *.smiley"),
                                NULL,
                                options);
    if(!path.isEmpty())
    {
        m_smileysList->clear();
        m_smileysMap.clear();
        QFile file(path);
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
        {
            QMessageBox::warning(this, _("Error Loading File"), _("Unable to open file: %1").arg(path));
            return;
        }
        QDir dir(QFileInfo(path).path());
        while(!file.atEnd())
        {
            QByteArray line = file.readLine();
            QString sline = QString(line);
            if(sline[0]!='#') //# start comment line
            {
                QString smiley = sline.section('=',1).trimmed();
                QString spath = dir.cleanPath(dir.absoluteFilePath(sline.section('=',0,0)));
                if(!smiley.isEmpty() && !spath.isEmpty()) m_smileysMap.insert(smiley, spath);
            }
        }
        fillSmileys();
        slotUseSmileys();
    }
    m_showImportwarning = false;
}

void ConfigDialog::slotExportSmileys()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString path = QFileDialog::getSaveFileName(this,
                                _("Save smiley settings as"),
#if(QT_VERSION > 0x050000)
                                QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0),
#else
                                QDesktopServices::storageLocation(QDesktopServices::HomeLocation),
#endif
                                "*.scfg",
                                NULL,
                                options);
    if(!path.isEmpty())
    {
        QFile file(path);
        if(!file.open(QIODevice::WriteOnly | QIODevice::Text))
        {
            QMessageBox::warning(this, _("Error Writing File"), _("Unable to open file: %1").arg(path));
            return;
        }
        QDir dir(QFileInfo(path).path());
        file.write(HEADER);
        QMap<QString,QString>::iterator it;
        for(it=m_smileysMap.begin(); it!=m_smileysMap.end(); ++it)
        {
            file.write(dir.relativeFilePath(it.value()).toLatin1());
            file.write("=");
            file.write(it.key().toLatin1());
            file.write("\n");
        }
        file.close();
    }
}

void ConfigDialog::fillSmileys()
{
    m_smileysList->clear();
    QMap<QString,QString>::iterator it;
    for(it=m_smileysMap.begin(); it!=m_smileysMap.end(); ++it)
    {
        m_smileysList->addItem(new QListWidgetItem(QIcon(it.value()), it.key()));
    }
}

void ConfigDialog::slotModifyIrcFont()
{
    bool ok;
    QFont font;
    font.fromString(m_ircFont->text());
    font = QFontDialog::getFont(&ok, font, this);
    if(ok)
    {
        m_ircFont->setText(font.toString());
        m_ircFont->setFont(font);
        m_ircText->setFont(font);
    }
}

void ConfigDialog::slotReconnect()
{
    if(m_reconnect->isChecked())
    {
        m_numberAttempt->setEnabled(true);
        m_delayAttempt->setEnabled(true);
    }
    else
    {
        m_numberAttempt->setEnabled(false);
        m_delayAttempt->setEnabled(false);
    }
}

void ConfigDialog::slotLoadLog()
{
    if(m_loadLog->isChecked()) m_loadLines->setEnabled(true);
    else m_loadLines->setEnabled(false);
}

void ConfigDialog::slotAddFriend()
{
    UserDialog *dialog = new UserDialog(true, this);
    if(dialog->exec() == QDialog::Accepted)
    {
        if(!dialog->nick().isEmpty() && !nickExists(dialog->nick(), true))
        {
            UserInfo info;
            info.nick = dialog->nick();
            dialog->channels().isEmpty() ? info.channel = "all" : info.channel = dialog->channels();
            dialog->server().isEmpty() ? info.network = "all" : info.network = dialog->server();
            m_friends.append(info);
            QTreeWidgetItem *item = new QTreeWidgetItem(m_friendsList);
            item->setText(0, info.nick);
            item->setText(1, info.channel);
            item->setText(2, info.network);
            m_friendModify->setEnabled(true);
            m_friendDelete->setEnabled(true);
            m_friendsList->setCurrentItem(item);
        }
    }
}

void ConfigDialog::slotModifyFriend()
{
    QString oldnick = m_friendsList->currentItem()->text(0);
    int index = nickIndex(oldnick, true);
    if(index==-1) return;
    UserDialog *dialog = new UserDialog(true, oldnick, m_friendsList->currentItem()->text(1), m_friendsList->currentItem()->text(2), false, this);
    if(dialog->exec() == QDialog::Accepted)
    {
        if(!dialog->nick().isEmpty() && (!nickExists(dialog->nick(), true) || oldnick == dialog->nick()))
        {
            m_friends[index].nick = dialog->nick();
            dialog->channels().isEmpty() ? m_friends[index].channel = "all" : m_friends[index].channel = dialog->channels();
            dialog->server().isEmpty() ? m_friends[index].network = "all" : m_friends[index].network = dialog->server();
            QTreeWidgetItem *item = m_friendsList->currentItem();
            item->setText(0, m_friends[index].nick);
            item->setText(1, m_friends[index].channel);
            item->setText(2, m_friends[index].network);
        }
    }
}

void ConfigDialog::slotDeleteFriend()
{
    QTreeWidgetItem *current = m_friendsList->currentItem();
    if(current)
    {
        int index = nickIndex(current->text(0), true);
        if(index==-1) return;
        m_friends.removeAt(index);
        delete current;
        if(m_friends.size())
        {
            m_friendsList->setCurrentItem(m_friendsList->invisibleRootItem()->child(0));
        }
        else
        {
            m_friendModify->setEnabled(false);
            m_friendDelete->setEnabled(false);
        }
    }
}

bool ConfigDialog::nickExists(const QString& nick, bool frien)
{
    if(frien)
    {
        foreach(UserInfo item, m_friends)
        {
            if(item.nick == nick) return true;
        }
    }
    else
    {
        foreach(UserInfo item, m_ignored)
        {
            if(item.nick == nick) return true;
        }
    }
    return false;
}

int ConfigDialog::nickIndex(const QString& nick, bool frien)
{
    if(frien)
    {
        for(int i=0; i<m_friends.size(); ++i)
        {
            if(m_friends[i].nick == nick) return i;
        }
    }
    else
    {
        for(int i=0; i<m_ignored.size(); ++i)
        {
            if(m_ignored[i].nick == nick) return i;
        }
    }
    return -1;
}

void ConfigDialog::slotAddIgnored()
{
    UserDialog *dialog = new UserDialog(false, this);
    if(dialog->exec() == QDialog::Accepted)
    {
        if(!dialog->nick().isEmpty() && !nickExists(dialog->nick(), true))
        {
            UserInfo info;
            info.nick = dialog->nick();
            dialog->channels().isEmpty() ? info.channel = "all" : info.channel = dialog->channels();
            dialog->server().isEmpty() ? info.network = "all" : info.network = dialog->server();
            m_ignored.append(info);
            QTreeWidgetItem *item = new QTreeWidgetItem(m_ignoredList);
            item->setText(0, info.nick);
            item->setText(1, info.channel);
            item->setText(2, info.network);
            m_ignoredModify->setEnabled(true);
            m_ignoredDelete->setEnabled(true);
            m_ignoredList->setCurrentItem(item);
        }
    }
}

void ConfigDialog::slotModifyIgnored()
{
    QString oldnick = m_ignoredList->currentItem()->text(0);
    int index = nickIndex(oldnick, false);
    if(index==-1) return;
    UserDialog *dialog = new UserDialog(false, oldnick, m_ignoredList->currentItem()->text(1), m_ignoredList->currentItem()->text(2), false, this);
    if(dialog->exec() == QDialog::Accepted)
    {
        if(!dialog->nick().isEmpty() && (!nickExists(dialog->nick(), true) || oldnick == dialog->nick()))
        {
            m_ignored[index].nick = dialog->nick();
            dialog->channels().isEmpty() ? m_ignored[index].channel = "all" : m_ignored[index].channel = dialog->channels();
            dialog->server().isEmpty() ? m_ignored[index].network = "all" : m_ignored[index].network = dialog->server();
            QTreeWidgetItem *item = m_ignoredList->currentItem();
            item->setText(0, m_ignored[index].nick);
            item->setText(1, m_ignored[index].channel);
            item->setText(2, m_ignored[index].network);
        }
    }
}

void ConfigDialog::slotDeleteIgnored()
{
    QTreeWidgetItem *current = m_ignoredList->currentItem();
    if(current)
    {
        int index = nickIndex(current->text(0), false);
        if(index==-1) return;
        m_ignored.removeAt(index);
        delete current;
        if(m_ignored.size())
        {
            m_ignoredList->setCurrentItem(m_ignoredList->invisibleRootItem()->child(0));
        }
        else
        {
            m_ignoredModify->setEnabled(false);
            m_ignoredDelete->setEnabled(false);
        }
    }
}

void ConfigDialog::slotAddCommand()
{
    QString available[11] = { "away", "ban", "ctcp", "invite", "join", "me", "nick", "notice", "mode", "part", "quit"};
    QStringList items;
    for(int i=0; i<11; i++)
    {
        bool exists = false;
        for(int j=0; j<m_commandsList->count(); ++j)
        {
            if(!available[i].compare(m_commandsList->item(j)->text()))
            {
                exists = true;
                break;
            }
        }
        if(!exists) items.append(available[i]);
    }
    if(!items.size()) return;
    bool ok;
    QString item = QInputDialog::getItem(this, _("Select ignore command"),
                                          _("Command:"), items, 0, false, &ok);
    if(ok && !item.isEmpty())
    {
        m_commandsList->addItem(item);
        m_commands.append(item);
    }
    m_commandDelete->setEnabled(true);
}

void ConfigDialog::slotDeleteCommand()
{
    QListWidgetItem *item = m_commandsList->currentItem()?m_commandsList->currentItem():m_commandsList->item(0);
    delete item;
    m_commands.clear();
    for(int i=0; i<m_commandsList->count(); ++i)
    {
        m_commands.append(m_commandsList->item(i)->text());
    }
    if(!m_commands.size())
    {
        m_commandDelete->setEnabled(false);
    }
}

void ConfigDialog::slotModifyKeybindings()
{
    KeyDialog *dialog = new KeyDialog(this);
    if(dialog->exec() == QDialog::Accepted && dialog->restartApp())
    {
        QDialog::reject();
        emit restartApp();
    }
}

void ConfigDialog::slotRestoreDefaultKeybindings()
{
    if(QMessageBox::question(this, _("Restore default key bindings"), _("Do you really want to restore the default key bindings?\nAll your customizations will be lost!"), QMessageBox::Ok|QMessageBox::Cancel) != QMessageBox::Ok)
        return;
    preferences.m_keybindings = preferences.getDefaultKeybindings();
    if(QMessageBox::question(this,_("Restart"), _("Key bindings will be changed after restart.\nRestart dxirc now?"), QMessageBox::Yes|QMessageBox::No)==QMessageBox::Yes)
    {
        QDialog::reject();
        emit restartApp();
    }
}
