/*
 *      dccengine.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QTimer>
#include <QDateTime>
#include <QFile>
#include <QFileInfo>
#include "dccengine.h"
#include "i18n.h"
#include "preferences.h"

static quint32 bswap(quint32 value)
{
    return ((((value)&0x000000FF)<<24) | (((value)&0xFF000000) >> 24) | (((value)&0x0000FF00) << 8) | (((value) &0x00FF0000) >> 8));
}

DccEngine::DccEngine(DccFile file, IrcEngine* engine)
        : m_file(file), m_engine(engine)
{
    m_connected = false;
    m_socket = new dxSocket(false);
    m_positionChanged = false;
    m_lastChange = 0;
    m_dataAmount = 0;
    m_dccTimeout = preferences.m_dccTimeout;
    m_timerClose = new QTimer(this);
    m_timerPosition = new QTimer(this);
    
    connect(m_timerClose, SIGNAL(timeout()), this, SLOT(slotCloseTimeout()));
    connect(m_timerPosition, SIGNAL(timeout()), this, SLOT(slotPositionTimeout()));
}

DccEngine::~DccEngine()
{
    m_connected = false;
    closeFile();
    m_socket->disconnectFrom();
    delete m_socket;
}

//for offered connectin by me
void DccEngine::slotCloseTimeout()
{
    m_connected = false;
    m_file.canceled = true;
    updateDcc();
    m_socket->disconnectFrom();
    emit dccError(_("Connection closed. Client didn't connect in given timeout"));
}

//fired current position
void DccEngine::slotPositionTimeout()
{
    updateDcc();
    if(m_file.currentPosition < m_file.size && m_connected)
        m_timerPosition->start(1000);
    else
        m_timerPosition->stop();
}

void DccEngine::slotCanRead()
{
    readData();
}


void DccEngine::slotWritten(qint64 bytes)
{
    if(m_file.type == DCC_OUT || m_file.type == DCC_POUT)
    {
        m_dataAmount += bytes;
        m_file.currentPosition += bytes;
        m_positionChanged = true;
    }
}

void DccEngine::slotConnected()
{
    if(m_file.type == DCC_IN)
    {
        if(m_file.currentPosition)
        {
            m_receivedFile.open(QString(m_file.path+".part").toLatin1(), std::ios_base::binary|std::ios_base::ate|std::ios_base::app);
        }
        else
            m_receivedFile.open(QString(m_file.path+".part").toLatin1(), std::ios_base::binary);
        m_file.started = true;
    }
    if(m_file.type == DCC_POUT)
    {
        m_sentFile.open(m_file.path.toLatin1(), std::ios_base::binary);
        m_file.started = true;
        if(m_file.currentPosition) m_sentFile.seekg(m_file.currentPosition, std::ios::beg);
        writeData();
    }
    m_connected = true;
    m_timerPosition->start(1000);
}

void DccEngine::slotDisconnected()
{
    m_connected = false;
    m_timerClose->stop();
    closeFile();
}

void DccEngine::slotError(QString error) 
{
    m_connected = false;
    m_timerClose->stop();
    closeFile();
    if(!m_file.finished) emit dccError(error);
}

void DccEngine::slotListen()
{
    m_timerClose->stop();
    if(m_file.type == DCC_OUT)
    {
        m_sentFile.open(m_file.path.toLatin1(), std::ios_base::binary);
        if(m_file.currentPosition) m_sentFile.seekg(m_file.currentPosition, std::ios::beg);
        writeData();
    }
    if(m_file.type == DCC_PIN)
    {
        m_receivedFile.open(QString(m_file.path+".part").toLatin1(), std::ios_base::binary);
    }
    m_file.started = true;
    m_connected = true;
    m_timerPosition->start(1000);
}

void DccEngine::slotStartAccept(int port)
{
    if(m_file.type == DCC_OUT)
    {
        m_file.port = port;
        m_engine->sendCtcp(m_file.nick, "DCC SEND "+dxutils::removeSpaces(QFileInfo(m_file.path).fileName())+" "+m_socket->stringIPToBinary(m_file.ip)+" "+QString::number(m_file.port)+" "+QString::number(m_file.size));
    }
    else if(m_file.type == DCC_PIN)
    {
        m_file.port = port;
        m_engine->sendCtcp(m_file.nick, "DCC SEND "+dxutils::removeSpaces(QFileInfo(m_file.path).fileName())+" "+m_socket->stringIPToBinary(m_file.ip)+" "+QString::number(m_file.port)+" "+QString::number(m_file.size)+" "+QString::number(m_file.token));
    }
    m_timerClose->start(m_dccTimeout*1000);
}

void DccEngine::updateDcc()
{
    if(m_positionChanged)
    {
        m_file.speed = m_dataAmount - m_file.speed;
        m_lastChange = QDateTime::currentMSecsSinceEpoch();
        m_dataAmount = m_file.speed;
    }
    else
    {
        if(QDateTime::currentMSecsSinceEpoch() != m_lastChange) m_file.speed = m_dataAmount/(QDateTime::currentMSecsSinceEpoch()-m_lastChange);
        else m_file.speed = 0;
    }
    emit dccPosition(m_file);
    m_positionChanged = false;
}

void DccEngine::startConnection()
{
    if(m_connected)
        return;
    connect(m_socket, SIGNAL(socketConnected()), this, SLOT(slotConnected()));
    connect(m_socket, SIGNAL(socketDisconnected()), this, SLOT(slotDisconnected()));
    connect(m_socket, SIGNAL(socketCanRead()), this, SLOT(slotCanRead()));
    connect(m_socket, SIGNAL(socketError(QString)), this, SLOT(slotError(QString)));
    connect(m_socket, SIGNAL(socketWritten(qint64)), this, SLOT(slotWritten(qint64)));
    connect(m_socket, SIGNAL(socketStartAccept(int)), this, SLOT(slotStartAccept(int)));
    connect(m_socket, SIGNAL(socketSomeConnected()), this, SLOT(slotListen()));
    bool listen;
    if(m_file.type == DCC_OUT || m_file.type == DCC_PIN)
    {
        listen = true;
        QString address = preferences.m_dccIP;
        if(address.isEmpty()) address = m_engine->getMyUserHost();
        if(address.isEmpty()) address = m_engine->getLocalIP();
        if(address.isEmpty())
        {
            m_file.canceled = true;
            return;
        }
        m_file.ip = address;
    }
    else
        listen = false;
    m_socket->setIsSSL(false);
    m_socket->setHost(m_file.ip, m_file.port);
    if(listen) m_socket->listenOn();
    else m_socket->connectTo();
}

void DccEngine::disconnect()
{
    m_connected = false;
    closeFile();
    m_socket->disconnectFrom();
}

void DccEngine::readData()
{
    if(m_file.type == DCC_IN || m_file.type == DCC_PIN)
    {
        int read = m_socket->getBytesAvailable();
        if(read<=0) read = 4096;
        char *buffer = new char[read];
        int size = 0;
        size = m_socket->read(buffer, read);
        if(size >= 0)
        {
            m_file.currentPosition += size;
            if(m_receivedFile.good())
                m_receivedFile.write(buffer, size);
            quint32 pos = bswap(m_file.currentPosition);
            m_socket->write((char*)&pos, 4);
        }
        if(m_file.currentPosition >= m_file.size)
            closeFile();
        m_positionChanged = true;
        m_dataAmount += size;
        delete []buffer;
    }
    else
    {
        while(m_socket->getBytesAvailable())
        {
            quint32 pos;
            m_socket->read((char*)&pos, 4);
            m_file.finishedPosition = bswap(pos);
        }
        if(m_file.finishedPosition >= m_file.size)
        {
            closeFile();
            m_socket->disconnectFrom();
        }
        writeData();
    }
}

void DccEngine::writeData()
{
    if(m_file.type == DCC_IN || m_file.type == DCC_PIN)
        return;
    if(m_file.currentPosition >= m_file.size)
        return;
    char buf[4096];
    if(m_sentFile.good())
        m_sentFile.read(buf, 4096);
    quint32 readedChars = m_sentFile.gcount();
    m_socket->write(buf, qMin(readedChars, m_file.size-m_file.currentPosition));
}

void DccEngine::closeFile()
{
    if(m_file.type == DCC_IN || m_file.type == DCC_PIN)
    {
        while(m_socket->getBytesAvailable())
        {
            char buffer[4096];
            int size = 0;
            size = m_socket->read(buffer, 4096);
            if(size < 0)
                break;
            m_file.currentPosition += size;
            if(m_receivedFile.good())
                m_receivedFile.write(buffer, size);
            quint32 pos = bswap(m_file.currentPosition);
            if(m_socket->isConnected()) m_socket->write((char*)&pos, 4);
        }
        m_receivedFile.close();
        if(m_file.currentPosition < m_file.size)
            m_file.canceled = true;
        else
        {
            QFile::rename(QString(m_file.path+".part"), m_file.path);
            m_file.finished = true;
        }
    }
    if(m_file.type == DCC_POUT || m_file.type == DCC_OUT)
    {
        m_sentFile.close();
        while(m_socket->getBytesAvailable())
        {
            quint32 pos;
            m_socket->read((char*)&pos, 4);
            m_file.finishedPosition = bswap(pos);
        }
        if(m_file.finishedPosition < m_file.size)
            m_file.canceled = true;
        else
            m_file.finished = true;
    }
}

//change position in file mainly for resumed file
bool DccEngine::setDccPosition(quint32 position)
{
    if(position >= m_file.size) return false;
    m_file.currentPosition = position;
    m_file.speed = 0;
    m_file.finishedPosition = position;
    m_file.canceled = false;
    return true;
}

//check dccfile for resume
bool DccEngine::isForResume(const QString& nick, const QString& name, int port)
{
    return m_file.nick == nick && QFileInfo(m_file.path).fileName() == name
                && m_file.port == port;
}

//check dccfile for resume
bool DccEngine::isForResume(int token)
{
    return m_file.token == token;
}
