/*
 *      dxsocket.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "dxsocket.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

dxSocket::dxSocket(bool isSSL)
    : m_isSsl(isSSL), m_portD(0), m_portH(0), m_listen(false)
{
    m_connecting = false;
    m_connected = false;
    m_localIP = "";
    m_remoteIP = "";
    m_tcp = new QTcpSocket;
    connect(m_tcp, SIGNAL(connected()), this, SLOT(slotConnected()));
    connect(m_tcp, SIGNAL(disconnected()), this, SLOT(slotDisconnected()));
    connect(m_tcp, SIGNAL(readyRead()), this, SLOT(slotCanRead()));
    connect(m_tcp, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(slotError(QAbstractSocket::SocketError)));
    connect(m_tcp, SIGNAL(bytesWritten(qint64)), this, SLOT(slotWritten(qint64)));
    m_ssl = new QSslSocket;
    m_ssl->setPeerVerifyMode(QSslSocket::QueryPeer);
    connect(m_ssl, SIGNAL(connected()), this, SLOT(slotConnected()));
    connect(m_ssl, SIGNAL(disconnected()), this, SLOT(slotDisconnected()));
    connect(m_ssl, SIGNAL(readyRead()), this, SLOT(slotCanRead()));
    connect(m_ssl, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(slotError(QAbstractSocket::SocketError)));
    connect(m_ssl, SIGNAL(sslErrors(QList<QSslError>)), this, SLOT(slotSslError(QList<QSslError>)));
    m_server = new QTcpServer;
    connect(m_server, SIGNAL(newConnection()), this, SLOT(slotSomeConnected()));
    m_client = NULL;
}

dxSocket::~dxSocket()
{}

void dxSocket::slotConnected()
{
    if(sender() == m_ssl) dxutils::debugLine("Connected by ssl");
    m_connecting = false;
    m_connected = true;
    emit socketConnected();
}

void dxSocket::slotDisconnected()
{
    m_connecting = false;
    m_connected = false;
    emit socketDisconnected();
}

void dxSocket::slotError(QAbstractSocket::SocketError err)
{
    if(!m_connecting && !m_connected)
        return;
    m_connecting = false;
    m_connected = false;
    switch(err) {
        case QAbstractSocket::ConnectionRefusedError: emit socketError(_("The connection was refused by the peer (or timed out)."));
        break;
        case QAbstractSocket::RemoteHostClosedError: emit socketError(_("The remote host closed the connection. Note that the client socket (i.e., this socket) will be closed after the remote close notification has been sent."));
        break;
        case QAbstractSocket::HostNotFoundError: emit socketError(_("The host address was not found."));
        break;
        case QAbstractSocket::SocketAccessError: emit socketError(_("The socket operation failed because the application lacked the required privileges."));
        break;
        case QAbstractSocket::SocketResourceError: emit socketError(_("The local system ran out of resources (e.g., too many sockets)."));
        break;
        case QAbstractSocket::SocketTimeoutError: emit socketError(_("The socket operation timed out."));
        break;
        case QAbstractSocket::DatagramTooLargeError: emit socketError(_("The datagram was larger than the operating system's limit (which can be as low as 8192 bytes)."));
        break;
        case QAbstractSocket::NetworkError: emit socketError(_("An error occurred with the network (e.g., the network cable was accidentally plugged out)."));
        break;
        case QAbstractSocket::AddressInUseError: emit socketError(_("The address specified to QUdpSocket::bind() is already in use and was set to be exclusive."));
        break;
        case QAbstractSocket::SocketAddressNotAvailableError: emit socketError(_("The address specified to QUdpSocket::bind() does not belong to the host."));
        break;
        case QAbstractSocket::UnsupportedSocketOperationError: emit socketError(_("The requested socket operation is not supported by the local operating system (e.g., lack of IPv6 support)."));
        break;
        case QAbstractSocket::ProxyAuthenticationRequiredError: emit socketError(_("The socket is using a proxy, and the proxy requires authentication."));
        break;
        case QAbstractSocket::SslHandshakeFailedError: emit socketError(_("The SSL/TLS handshake failed, so the connection was closed"));
        break;
        case QAbstractSocket::UnfinishedSocketOperationError: emit socketError(_("Used by QAbstractSocketEngine only, The last operation attempted has not finished yet (still in progress in the background)."));
        break;
        case QAbstractSocket::ProxyConnectionRefusedError: emit socketError(_("Could not contact the proxy server because the connection to that server was denied"));
        break;
        case QAbstractSocket::ProxyConnectionClosedError: emit socketError(_("The connection to the proxy server was closed unexpectedly (before the connection to the final peer was established)"));
        break;
        case QAbstractSocket::ProxyConnectionTimeoutError: emit socketError(_("The connection to the proxy server timed out or the proxy server stopped responding in the authentication phase."));
        break;
        case QAbstractSocket::ProxyNotFoundError: emit socketError(_("The proxy address set with setProxy() (or the application proxy) was not found."));
        break;
        case QAbstractSocket::ProxyProtocolError: emit socketError(_("The connection negotiation with the proxy server because the response from the proxy server could not be understood."));
        break;
#if(QT_VERSION >= 0x050000)
        case QAbstractSocket::OperationError: emit socketError(_("An operation was attempted while the socket was in a state that did not permit it."));
        break;
        case QAbstractSocket::SslInternalError: emit socketError(_("The SSL library being used reported an internal error. This is probably the result of a bad installation or misconfiguration of the library."));
        break;
        case QAbstractSocket::SslInvalidUserDataError: emit socketError(_("Invalid data (certificate, key, cypher, etc.) was provided and its use resulted in an error in the SSL library."));
        break;
        case QAbstractSocket::TemporaryError: emit socketError(_("A temporary error occurred (e.g., operation would block and socket is non-blocking)"));
        break;
#endif
        case QAbstractSocket::UnknownSocketError: emit socketError(_("An unidentified error occurred."));
        break;
        default: emit socketError(_("Some socket error."));
    }
}

void dxSocket::slotSslError(const QList<QSslError> & errors)
{
    foreach(QSslError error, errors)
    {
        dxutils::debugLine(error.errorString());
    }
    m_ssl->ignoreSslErrors();
}

void dxSocket::slotCanRead()
{
    emit socketCanRead();
}

void dxSocket::slotSomeConnected()
{
    m_client = m_server->nextPendingConnection();
    connect(m_client, SIGNAL(connected()), this, SLOT(slotConnected()));
    connect(m_client, SIGNAL(disconnected()), this, SLOT(slotDisconnected()));
    connect(m_client, SIGNAL(readyRead()), this, SLOT(slotCanRead()));
    connect(m_client, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(slotError(QAbstractSocket::SocketError)));
    connect(m_client, SIGNAL(bytesWritten(qint64)), this, SLOT(slotWritten(qint64)));
    m_connected = true;
    emit socketSomeConnected();
}

void dxSocket::slotWritten(qint64 bytes)
{
    emit socketWritten(bytes);
}

void dxSocket::setHost(const QString& address, int portD, int portH)
{
    m_address = address;
    m_portD = portD;
    m_portH = portH;
}

void dxSocket::connectTo()
{
    if(m_connecting || m_connected)
        return;
    m_connecting = true;;
    m_listen = false;
    if(m_isSsl)
    {
        m_ssl->connectToHostEncrypted(m_address, m_portD);
    }
    else
    {
        m_tcp->connectToHost(m_address, m_portD);
    }
}

void dxSocket::listenOn()
{
    if(m_connecting || m_connected)
        return;
    m_connecting = true;
    m_listen = true;
    if(m_server->listen(QHostAddress(m_address), m_portD))
    {
        emit socketStartAccept(m_server->serverPort());
        return;
    }
    if(m_portD && m_portD<m_portH)
    {
        int port = m_portD + 1;
        while(port < m_portH+1)
        {
            if(m_server->listen(QHostAddress(m_address), m_portD))
            {
                emit socketStartAccept(m_server->serverPort());
                return;
            }
            else
                port++;
        }
    }
}

void dxSocket::disconnectFrom()
{
    if(!m_connected)
        return;
    if(m_listen)
    {
        m_server->close();
        if(m_client) m_client->disconnectFromHost();
    }
    else
    {
        if(m_isSsl)
            m_ssl->disconnectFromHost();
        else
            m_tcp->disconnectFromHost();
    }
}

qint64 dxSocket::read(char* buffer, int len)
{
    if(!m_connected)
        return -1;
    if(len<0)
    {
        return -1;
    }
    if(m_listen)
    {
        if(m_client) return m_client->read(buffer, len);
        else return -1;
    }
    else
    {
        if(m_isSsl)
            return m_ssl->read(buffer, len);
        else
            return m_tcp->read(buffer, len);
    }
}

qint64 dxSocket::write(const char *buffer, size_t len)
{
    if(!m_connected) return 0;
    if(m_listen)
    {
        if(m_client) return m_client->write(buffer, len);
        else return -1;
    }
    else
    {
        if(m_isSsl)
            return m_ssl->write(buffer, len);
        else
            return m_tcp->write(buffer, len);
    }
}

qint64 dxSocket::write(const QString &line)
{
    if(!m_connected) return 0;
    if(m_listen)
    {
        if(m_client) return m_client->write(line.toUtf8());
        else return -1;
    }
    else
    {
        if(m_isSsl)
            return m_ssl->write(line.toUtf8());
        else
            return m_tcp->write(line.toUtf8());
    }
}

void dxSocket::setIsSSL(bool isSSL)
{
    m_isSsl = isSSL;
}

//address in ASCII order
bool dxSocket::isIPv4(const QString& address)
{
    if(address.contains(':'))
        return false;
    if(address.count('.') == 3)
    {
        QRegExp rex("\\l");
        if(!rex.exactMatch(address))
            return true;
    }
    return true;
}

//ipaddr in byte order
bool dxSocket::isRoutableIP(quint32 ipaddr)
{
    if(!ipaddr) return false;
    unsigned char * ip = (unsigned char *)&ipaddr;
    int a, b, c;
    (*ip)++;
    c = (int)*ip++;
    b = (int)*ip++;
    a = (int)*ip;
    if(a == 0) return false;    // old-style broadcast
    if(a == 10) return false;   // Class A VPN
    if(a == 127) return false;   // loopback
    if((a == 172) && (b >= 16) && (b <= 31)) return false; // Class B VPN
    if((a == 192) && (b == 168)) return false; // Class C VPN
    if((a == 169) && (b == 254)) return false; // APIPA
    if((a == 192) && (b == 0) && (c == 2)) return false; // Class B VPN
    if(a >= 224) return false; // class D multicast and class E reserved
    return true;
}

//address as "2130706433" or "2001:db8:63b3:1::3490"
QString dxSocket::binaryIPToString(const QString &ip)
{
    if(!ip.contains(':')) /* normal IPv4 address in 32bit number form */
    {
        int ipnum = ip.toUInt();
        unsigned char bytes[4];
        int i;
        for(i=0; i<4; i++) {
           bytes[i] = (ipnum >> i*8) & 0xFF;
        }
        return QString("%1.%2.%3.%4").arg(bytes[3]).arg(bytes[2]).arg(bytes[1]).arg(bytes[0]);
    }
    return ip;
}

//Return IP usefull for DCC (byte order if IPv4)
//address as "127.0.0.1" or "2001:db8:63b3:1::3490"
QString dxSocket::stringIPToBinary(const QString &ip)
{
    if(isIPv4(ip))
    {
        QHostAddress a(ip);
        return QString("%1").arg(a.toIPv4Address());
    }
    else
        return ip;
}

//return string in dots-and-numbers format
QString dxSocket::getLocalIP()
{
    if(m_connected && m_localIP.isEmpty())
    {
        if(m_listen) m_localIP = m_address;
        else
        {
            if(m_isSsl) m_localIP = m_tcp->localAddress().toString();
            else m_localIP = m_ssl->localAddress().toString();
        }
    }
    return m_localIP;
}

//return string in dots-and-numbers format
QString dxSocket::getRemoteIP()
{
    if(m_connected && m_remoteIP.isEmpty())
    {
        if(m_listen) m_remoteIP = m_client ? m_client->peerAddress().toString() : "";
        else
        {
            if(m_isSsl) m_remoteIP = m_tcp->peerAddress().toString();
            else m_remoteIP = m_ssl->peerAddress().toString();
        }
    }
    return m_remoteIP;
}

//address as "127.0.0.1" or "2001:db8:63b3:1::3490"
QString dxSocket::getHostname(const QString& ip)
{
    if(m_connected)
    {
        QHostInfo i = QHostInfo::fromName(ip);
        if(i.addresses().size())
            return i.hostName();
    }
    return "";
}

QString dxSocket::getIP(const QString& hostname)
{
    if(m_connected)
    {
        QHostInfo i = QHostInfo::fromName(hostname);
        if(i.addresses().size())
            return i.addresses()[0].toString();
    }
    return hostname;
}

qint64 dxSocket::getBytesAvailable()
{
    if(m_listen)
    {
        return m_client->bytesAvailable();
    }
    else
    {
        if(m_isSsl)
            return m_ssl->bytesAvailable();
        else
            return m_tcp->bytesAvailable();
    }
}

