/*
 *      dxirc.cpp
 *
 *      Copyright 2008 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifdef WIN32
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
#else
 #include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <signal.h>
#endif

#include <string>
#include "defs.h"
#include "config.h"
#include "dxutils.h"
#include "FXTrayApp.h"
#include "dialogs/logviewer.h"
#include "dialogs/dccdialog.h"
#include "tabitems/dxtabbook.h"
#include "engine/ircengine.h"
#include "engine/dccengine.h"
#include "icons.h"
#include "i18n.h"
#include "help.h"
#include "tabitems/irctabitem.h"
#include "tabitems/dcctabitem.h"
#include "dialogs/configdialog.h"
#include "dialogs/serverdialog.h"
#include "dialogs/aliasdialog.h"
#include "tabitems/tetristabitem.h"
#include "tabitems/boatstabitem.h"
#include "dialogs/scriptdialog.h"
#include "dialogs/aboutdialog.h"
#include "dialogs/helpdialog.h"
#include "dialogs/listdialog.h"
#ifdef HAVE_X11
 #include <X11/Xlib.h>
#endif //HAVE_X11
#ifdef HAVE_LUA
extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}
#endif
#include "dxirc.h"

#define DISPLAY(app) ((Display*)((app)->getDisplay()))

int argn; //argc for restart
char** args; //argv for restart

#ifndef WIN32
void dxunlink(void)
{
    unlink(FXString(FXSystem::getTempDirectory()+PATHSEPSTRING+"dxirc-"+FXSystem::currentUserName()).text());
}
#endif

// For handling Single Instance of Application
void IsInstanceRunningAlready()
{
#ifdef WIN32
    HANDLE hSingleInstance = NULL;
    hSingleInstance = CreateMutex( NULL, FALSE, TEXT("dxirc.exe"));
    DWORD dwLastError = GetLastError();
    if(dwLastError == ERROR_ALREADY_EXISTS)
    {
        MessageBox(NULL, TEXT("dxirc already running"), TEXT("dxirc"), MB_OK | MB_ICONINFORMATION);
        CloseHandle(hSingleInstance);
        exit(1);
    }
#else
    FXString path = FXSystem::getTempDirectory()+PATHSEPSTRING+"dxirc-"+FXSystem::currentUserName();
    struct flock fl;
    int fd;
    fd = open(path.text(), O_RDWR | O_CREAT, 0600);
    if(fd < 0)
    {
        perror(path.text());
        _exit(1);
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    if(fcntl(fd, F_SETLK, &fl) < 0)
    {
        FXApp application("dxirc");
        application.init(argn, args);
        FXMainWindow *main = new FXMainWindow(&application, "dxirc", NULL, NULL, DECOR_TITLE|DECOR_CLOSE);
        new FXLabel(main, "dxirc already running");
        new FXButton(main, "C&lose", NULL, &application, FXApp::ID_QUIT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X);
        application.create();
        main->show(PLACEMENT_SCREEN);
        application.run();
        _exit(1);
    }
    atexit(dxunlink);
#endif
}

int CompareTabs(const void **a,const void **b)
{
    dxTabItem *fa = (dxTabItem*)*a;
    dxTabItem *fb = (dxTabItem*)*b;
    return comparecase((fa->getType() == SERVER ? fa->getRealServerName() : fa->getRealServerName()+fa->getText()), (fb->getType() == SERVER ? fb->getRealServerName() : fb->getRealServerName()+fb->getText()));
}

/*from Goggles Music Manager
thanks Sander Jansen */
#ifdef ENABLE_NLS
#include <FXTranslator.h>
#include <fxdefs.h>

class dxTranslator : public FXTranslator
{
    FXDECLARE(dxTranslator)
private:
    dxTranslator(const dxTranslator&);
    dxTranslator & operator=(const dxTranslator&);
protected:
    dxTranslator() {
    }
public:
    /// Construct translator

    dxTranslator(FXApp* a) : FXTranslator(a) {
#ifndef WIN32
        setlocale(LC_MESSAGES, "");
        setlocale(LC_NUMERIC, "C");
#endif //WIN32
        bindtextdomain("dxirc", LOCALEDIR);
        bind_textdomain_codeset("dxirc", "UTF-8");
        textdomain("dxirc");
        dxUtils.debugLine(FXStringFormat("localedir: %s", LOCALEDIR));
    }

    virtual const FXchar* tr(const FXchar* context, const FXchar* message, const FXchar* hint = NULL) const;

    ~dxTranslator() {}
};

FXIMPLEMENT(dxTranslator, FXTranslator, NULL, 0)

const FXchar* dxTranslator::tr(const FXchar*, const FXchar* message, const FXchar*) const {
    return gettext(message);
}
#endif //ENABLE_NLS

FXDEFMAP(dxirc) dxircMap[] = {
    FXMAPFUNC(SEL_CLOSE,            0,                      dxirc::onCmdClose),
    FXMAPFUNC(SEL_COMMAND,          dxirc_QUIT,             dxirc::onCmdQuit),
    FXMAPFUNC(SEL_COMMAND,          dxirc_RESTART,          dxirc::onCmdRestart),
    FXMAPFUNC(SEL_COMMAND,          dxirc_ABOUT,            dxirc::onCmdAbout),
    FXMAPFUNC(SEL_COMMAND,          dxirc_CONNECT,          dxirc::onCmdConnect),
    FXMAPFUNC(SEL_COMMAND,          dxirc_DISCONNECT,       dxirc::onCmdDisconnect),
    FXMAPFUNC(SEL_COMMAND,          dxirc_RECONNECT,        dxirc::onCmdReconnect),
    FXMAPFUNC(SEL_COMMAND,          dxirc_SERVERS,          dxirc::onCmdServers),
    FXMAPFUNC(SEL_COMMAND,          dxirc_TABS,             dxirc::onTabBook),
    FXMAPFUNC(SEL_COMMAND,          dxirc_CLEAR,            dxirc::onCmdClear),
    FXMAPFUNC(SEL_COMMAND,          dxirc_CLEARALL,         dxirc::onCmdClearAll),
    FXMAPFUNC(SEL_COMMAND,          dxirc_CLOSETAB,         dxirc::onCmdCloseTab),
    FXMAPFUNC(SEL_COMMAND,          dxirc_USERS,            dxirc::onCmdUsers),
    FXMAPFUNC(SEL_COMMAND,          dxirc_STATUS,           dxirc::onCmdStatus),
    FXMAPFUNC(SEL_COMMAND,          dxirc_HELP,             dxirc::onCmdHelp),
    FXMAPFUNC(SEL_COMMAND,          dxirc_OPTIONS,          dxirc::onCmdOptions),
    FXMAPFUNC(SEL_COMMAND,          dxirc_SELECTTAB,        dxirc::onCmdSelectTab),
    FXMAPFUNC(SEL_COMMAND,          dxirc_NEXTTAB,          dxirc::onCmdNextTab),
    FXMAPFUNC(SEL_COMMAND,          dxirc_MOVETAB,          dxirc::onCmdMoveTab),
    FXMAPFUNC(SEL_MOUSEWHEEL,       dxirc_TABS,             dxirc::onMouseWheel),
    FXMAPFUNC(SEL_COMMAND,          dxirc_NEXTUNREAD,       dxirc::onCmdNextUnread),
    FXMAPFUNC(SEL_COMMAND,          dxirc_ALIAS,            dxirc::onCmdAlias),
    FXMAPFUNC(SEL_COMMAND,          dxirc_LOG,              dxirc::onCmdLog),
    FXMAPFUNC(SEL_COMMAND,          dxirc_TRANSFERS,        dxirc::onCmdTransfers),
    FXMAPFUNC(SEL_COMMAND,          dxirc_TRAY,             dxirc::onTrayClicked),
    FXMAPFUNC(SEL_COMMAND,          dxirc_SCRIPTS,          dxirc::onCmdScripts),
    FXMAPFUNC(SEL_TIMEOUT,          dxirc_STIMEOUT,         dxirc::onStatusTimeout),
    FXMAPFUNC(SEL_KEYPRESS,         0,                      dxirc::onKeyPress),
    FXMAPFUNC(SEL_COMMAND,          IrcEngine_SERVER,       dxirc::onIrcEvent),
    FXMAPFUNC(SEL_COMMAND,          DccEngine_DCC,          dxirc::onDccEvent),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_CDIALOG,     dxirc::onCmdConnect),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_CSERVER,     dxirc::onTabConnect),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_CQUIT,       dxirc::onCmdQuit),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_NEWMSG,      dxirc::onNewMsg),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_NEWMSG,      dxirc::onNewMsg),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_LUA,         dxirc::onLua),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_LUA,         dxirc::onLua),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_COMMAND,     dxirc::onIrcCommand),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_COMMAND,     dxirc::onIrcCommand),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_OWNMSG,      dxirc::onIrcOwnMsg),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_OWNMSG,      dxirc::onIrcOwnMsg),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_NEWTETRIS,   dxirc::onNewTetris),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_NEWTETRIS,   dxirc::onNewTetris),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_ADDICOMMAND, dxirc::onAddIgnoreCommand),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_RMICOMMAND,  dxirc::onRemoveIgnoreCommand),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_ADDIUSER,    dxirc::onAddIgnoreUser),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_RMIUSER,     dxirc::onRemoveIgnoreUser),
    FXMAPFUNC(SEL_COMMAND,          DccDialog_DCCCANCEL,    dxirc::onCmdDccCancel),
    FXMAPFUNC(SEL_COMMAND,          dxirc_SPELL,            dxirc::onCmdSpell),
    FXMAPFUNC(SEL_COMMAND,          dxirc_FORCEFOCUS,       dxirc::onCmdForceFocus),
    FXMAPFUNC(SEL_TIMEOUT,          dxirc_AWAYTIMEOUT,      dxirc::onAwayTimeout),
    FXMAPFUNC(SEL_COMMAND,          dxirc_VERSION,          dxirc::onCmdCheckVersion),
    FXMAPFUNC(SOCKET_CONNECTED,     dxirc_VERSIONSOCKET,    dxirc::onVersionSocketConnected),
    FXMAPFUNC(SOCKET_CANREAD,       dxirc_VERSIONSOCKET,    dxirc::onVersionSocketCanRead),
    FXMAPFUNC(SOCKET_ERR,           dxirc_VERSIONSOCKET,    dxirc::onVersionSocketError),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_SHOWLOG,     dxirc::onShowLog),
    FXMAPFUNC(SEL_COMMAND,          IrcTabItem_SHOWLOG,     dxirc::onShowLog),
    FXMAPFUNC(SEL_TIMEOUT,          dxirc_NOTIFYHIDE,       dxirc::onNotifyHide),
    FXMAPFUNC(SEL_COMMAND,          dxirc_LIST,             dxirc::onCmdList)
};

FXIMPLEMENT(dxirc, FXMainWindow, dxircMap, ARRAYNUMBER(dxircMap))

dxirc *dxirc::_pThis = NULL;
#ifdef HAVE_LUA
static luaL_Reg dxircFunctions[] = {
    {"AddCommand",      dxirc::onLuaAddCommand},
    {"AddEvent",        dxirc::onLuaAddEvent},
    {"RemoveName",      dxirc::onLuaRemoveName},
    {"Command",         dxirc::onLuaCommand},
    {"Print",           dxirc::onLuaPrint},
    {"GetServers",      dxirc::onLuaGetServers},
    {"GetTab",          dxirc::onLuaGetTab},
    {"GetCurrentTab",   dxirc::onLuaGetCurrentTab},
    {"GetVersion",      dxirc::onLuaGetVersion},
    {"GetTabInfo",      dxirc::onLuaGetTabInfo},
    {"SetTab",          dxirc::onLuaSetTab},
    {"CreateTab",       dxirc::onLuaCreateTab},
    {"GetTabCount",     dxirc::onLuaGetTabCount},
    {"Clear",           dxirc::onLuaClear},
    {"GetChannelNames", dxirc::onLuaGetChannelNames},
    {NULL,              NULL}
};
#endif //HAVE_LUA

dxirc::dxirc(FXApp *app)
    : FXMainWindow(app, "dxirc", 0, 0, DECOR_ALL, 0, 0, 800, 600), m_app(app), m_trayIcon(NULL)
{
    setIcon(ICO_BIG);
    setMiniIcon(ICO_SMALL);

    m_viewer = NULL;
    m_transfers = NULL;
    m_traymenu = NULL;
    m_lastToken = 0;
    m_lastID = 0;
    m_versionSocket = NULL;
    m_firstNotify = false;
    m_notifiesHeight = 0;

    m_preferences.load();
    if(!m_preferences.m_ircFont.empty())
    {
        m_ircFont = new FXFont(m_app, m_preferences.m_ircFont);
        m_ircFont->create();
    }
    else
    {
        m_app->getNormalFont()->create();
        FXFontDesc fontdescription;
        m_app->getNormalFont()->getFontDesc(fontdescription);
        m_ircFont = new FXFont(m_app,fontdescription);
        m_ircFont->create();
    }
    setTempKeybindings(m_preferences.m_keybindings);

    m_menubar = new FXMenuBar(this, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

    m_servermenu = new FXMenuPane(this);
    new FXMenuCommand(m_servermenu, _("&Server list")+TAB+m_preferences.getBinding("serverList"), ICO_SERVERLIST, this, dxirc_SERVERS);
    new FXMenuCommand(m_servermenu, _("Quick &connect")+TAB+m_preferences.getBinding("quickConnect"), ICO_CONNECT, this, dxirc_CONNECT);
    m_reconnect = new FXMenuCommand(m_servermenu, _("&Reconnect all"), NULL, this, dxirc_RECONNECT);
    m_reconnect->disable();
    m_disconnect = new FXMenuCommand(m_servermenu, _("&Disconnect")+TAB+m_preferences.getBinding("disconnect"), ICO_DISCONNECT, this, dxirc_DISCONNECT);
    m_disconnect->disable();
    new FXMenuSeparator(m_servermenu);
    new FXMenuCommand(m_servermenu, _("DCC &transfers")+TAB+m_preferences.getBinding("dccTransfers"), ICO_TRANSFER, this, dxirc_TRANSFERS);
#ifdef HAVE_LUA
    new FXMenuSeparator(m_servermenu);
    new FXMenuCommand(m_servermenu, _("S&cripts")+TAB+m_preferences.getBinding("scripts"), ICO_SCRIPT, this, dxirc_SCRIPTS);
#endif //HAVE_LUA
    new FXMenuSeparator(m_servermenu);
    m_logviewer = new FXMenuCommand(m_servermenu, _("&Log viewer")+TAB+m_preferences.getBinding("logViewer"), ICO_LOGS, this, dxirc_LOG);
    if(!m_preferences.m_logging) m_logviewer->disable();
    new FXMenuSeparator(m_servermenu);
    new FXMenuCommand(m_servermenu, _("&Quit")+TAB+m_preferences.getBinding("quit"), ICO_QUIT, this, dxirc_QUIT);
    new FXMenuTitle(m_menubar, _("&Server"), NULL, m_servermenu);

    m_editmenu = new FXMenuPane(this);
    m_closeTab = new FXMenuCommand(m_editmenu, _("Close current tab")+TAB+m_preferences.getBinding("closeCurrentTab"), ICO_CLOSE, this, dxirc_CLOSETAB);
    m_closeTab->disable();
    new FXMenuSeparator(m_editmenu);
    m_clearTab = new FXMenuCommand(m_editmenu, _("Clear window")+TAB+m_preferences.getBinding("clearWindow"), ICO_CLEAR, this, dxirc_CLEAR);
    m_clearTabs = new FXMenuCommand(m_editmenu, _("Clear all windows")+TAB+m_preferences.getBinding("clearAllWindows"), NULL, this, dxirc_CLEARALL);
    new FXMenuSeparator(m_editmenu);
    m_preferences.m_usersShown = dxUtils.getBoolIniEntry("SETTINGS", "usersShown", TRUE);
    m_users = new FXMenuCheck(m_editmenu, _("Users list")+TAB+m_preferences.getBinding("usersList")+TAB+_("Show/Hide users list"), this, dxirc_USERS);
    m_users->setCheck(m_preferences.m_usersShown);
#ifdef HAVE_ENCHANT
    m_spellCombo = new FXMenuCheck(m_editmenu, _("Spellchecking language list")+TAB+m_preferences.getBinding("spellList")+TAB+_("Show/Hide spellchecking language list"), this, dxirc_SPELL);
    if(!(m_preferences.m_useSpell && dxUtils.getAvailableSpellLangsNum()))
    {
        m_preferences.m_showSpellCombo = FALSE;
        m_spellCombo->disable();
    }
    m_spellCombo->setCheck(m_preferences.m_showSpellCombo);
#endif //HAVE_ENCHANT
    m_status = new FXMenuCheck(m_editmenu, _("Status bar"), this, dxirc_STATUS);
    m_status->setCheck(m_preferences.m_statusShown);
    new FXMenuCommand(m_editmenu, _("&Aliases"), NULL, this, dxirc_ALIAS);
    new FXMenuCommand(m_editmenu, _("&Preferences"), ICO_OPTION, this, dxirc_OPTIONS);
    new FXMenuTitle(m_menubar, _("&Edit"), NULL, m_editmenu);

    m_helpmenu = new FXMenuPane(this);
    new FXMenuCommand(m_helpmenu, _("&Check latest version"), NULL, this, dxirc_VERSION);
    new FXMenuCommand(m_helpmenu, _("&About..."), NULL, this, dxirc_ABOUT);
    new FXMenuCommand(m_helpmenu, _("&Help")+TAB+m_preferences.getBinding("help"), ICO_HELP, this, dxirc_HELP);
    new FXMenuTitle(m_menubar, _("&Help"), NULL, m_helpmenu);

    m_mainframe = new FXVerticalFrame(this, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 1,1,1,1);

    m_tabbook = new dxTabBook(m_mainframe, this, dxirc_TABS, PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    IrcEngine *server = new IrcEngine(app, this, "", "");
    m_ircengines.append(server);

    m_statusbar = new FXHorizontalFrame(m_mainframe, LAYOUT_LEFT|JUSTIFY_LEFT|LAYOUT_FILL_X|FRAME_NONE, 0,0,0,0, 1,1,1,1);
    m_statuslabel = new FXLabel(m_statusbar, "dxirc", NULL, FRAME_SUNKEN|LAYOUT_LEFT|JUSTIFY_LEFT|LAYOUT_FILL_X);
    if (m_preferences.m_statusShown)
        m_statusbar->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_SHOW), NULL);
    else
        m_statusbar->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_HIDE), NULL);

#ifdef HAVE_TRAY
    if(m_preferences.m_useTray)
    {
        m_trayIcon = new FXTrayIcon(app, "dxirc", ICO_TRAY, 0, this, dxirc_TRAY, TRAY_CMD_ON_LEFT|TRAY_MENU_ON_RIGHT);
        if(m_traymenu == NULL)
            m_traymenu = new FXPopup(m_trayIcon);
        new FXMenuCommand(m_traymenu, _("&Quit"), ICO_QUIT, this, dxirc_QUIT);
        m_trayIcon->setMenu(m_traymenu);
    }
#endif //HAVE_TRAY

    new FXToolTip(app,0);

    //set mainwindow size and position
    setX(m_preferences.m_x);
    setY(m_preferences.m_y);
    setWidth(m_preferences.m_w);
    setHeight(m_preferences.m_h);

    updateTheme(m_preferences.m_unreadColor, m_preferences.m_highlightColor);
    updateFont();
    updateTabs(m_preferences.m_useSmileys && (FXint)m_preferences.m_smileysMap.size());
    updateTabPosition();
    updateTrayColor();

    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("firstTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("secondTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("thirdTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("fourthTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("fifthTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("sixthTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("seventhTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("eighthTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("ninethTab")), this, FXSEL(SEL_COMMAND, dxirc_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("switchTab")), this, FXSEL(SEL_COMMAND, dxirc_NEXTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("moveTabUp")), this, FXSEL(SEL_COMMAND, dxirc_MOVETAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("moveTabDown")), this, FXSEL(SEL_COMMAND, dxirc_MOVETAB));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("unreadTab")), this, FXSEL(SEL_COMMAND, dxirc_NEXTUNREAD));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("forceFocus")), this, FXSEL(SEL_COMMAND, dxirc_FORCEFOCUS));
    getAccelTable()->addAccel(dxutils::parseAccel(m_preferences.getBinding("listCommand")), this, FXSEL(SEL_COMMAND, dxirc_LIST));
}

dxirc::~dxirc()
{
    m_app->removeTimeout(this, dxirc_STIMEOUT);
    m_app->removeTimeout(this, dxirc_AWAYTIMEOUT);
    delete ICO_BIG;
    delete ICO_SMALL;
    delete ICO_IRCADMIN;
    delete ICO_IRCAWAYADMIN;
    delete ICO_IRCAWAYHALFOP;
    delete ICO_IRCAWAYNORMAL;
    delete ICO_IRCAWAYOP;
    delete ICO_IRCAWAYOWNER;
    delete ICO_IRCAWAYVOICE;
    delete ICO_IRCHALFOP;
    delete ICO_IRCNORMAL;
    delete ICO_IRCOP;
    delete ICO_IRCOWNER;
    delete ICO_IRCVOICE;
    delete ICO_SERVERLIST;
    delete ICO_CONNECT;
    delete ICO_DISCONNECT;
    delete ICO_QUIT;
    delete ICO_CLOSE;
    delete ICO_OPTION;
    delete ICO_HELP;
    delete ICO_SERVER;
    delete ICO_CHANNEL;
    delete ICO_QUERY;
    delete ICO_CLEAR;
    delete ICO_FLAG;
    delete ICO_TRAY;
    delete ICO_NEWMSG;
    delete ICO_NEWFILE;
    delete ICO_QUERYNEWMSG;
    delete ICO_CHANNELNEWMSG;
    delete ICO_CLOSEFOLDER;
    delete ICO_OPENFOLDER;
    delete ICO_FILE;
    delete ICO_CANCEL;
    delete ICO_FINISH;
    delete ICO_DOWN;
    delete ICO_UP;
    delete ICO_DCC;
    delete ICO_DCCNEWMSG;
    delete ICO_LOGS;
    delete ICO_SCRIPT;
    delete ICO_TRANSFER;
    delete ICO_PLAY;
    delete ICO_COPY;
    delete ICO_SELECTALL;
    delete ICO_FIND;
    delete ICO_REFRESH;
    delete ICO_KEYBINDINGS;
    while(m_smileys.no())
    {
        delete m_smileys[0].icon;
        m_smileys.erase(0);
    }
    delete m_servermenu;
    delete m_editmenu;
    delete m_helpmenu;
#ifdef HAVE_TRAY
    if(m_traymenu)
    {
        delete m_traymenu;
    }
#endif //HAVE_TRAY
    delete m_ircFont;
    if(m_versionSocket) delete m_versionSocket;
    _pThis = NULL;
}

dxirc* dxirc::instance()
{
    return _pThis;
}

void dxirc::create()
{
    FXMainWindow::create();
    if(m_preferences.m_maximized) maximize(TRUE);
    //Checking for screen resolution and correction size, position
    else
    {
        FXint maxWidth = getRoot()->getWidth();
        FXint maxHeight = getRoot()->getHeight();
        if(getX()+getWidth() > maxWidth || getX()<0)
        {
            setX(0);
            setWidth(maxWidth);
        }
        if(getY()+getHeight() > maxHeight || getY()<0)
        {
            setY(0);
            setHeight(maxHeight);
        }
    }
    show();
    _pThis = this;
    createIrcTab("(server)", ICO_SERVER, SERVER, m_ircengines[0]);
    for(FXint i=0; i<m_preferences.m_serverList.no(); i++)
    {
        if(m_preferences.m_serverList[i].autoConnect)
        {
            connectServer(m_preferences.m_serverList[i].hostname, m_preferences.m_serverList[i].port, m_preferences.m_serverList[i].passwd, m_preferences.m_serverList[i].nick, m_preferences.m_serverList[i].realname, m_preferences.m_serverList[i].channels, m_preferences.m_serverList[i].commands, m_preferences.m_serverList[i].useSsl);
        }
    }
    autoloadScripts();
    m_app->addTimeout(this, dxirc_AWAYTIMEOUT, 180000);
}

// Flash the window to get user's attention
// Taken from fox development version 1.7
void dxirc::flash(FXbool yes)
{
    if(xid)
    {
#ifdef WIN32
        FlashWindow((HWND)xid, true);
#else
    #ifdef HAVE_X11
        XEvent se;
        se.xclient.type=ClientMessage;
        se.xclient.display=DISPLAY(m_app);
        se.xclient.message_type=XInternAtom(DISPLAY(m_app), "_NET_WM_STATE", 0);;
        se.xclient.format=32;
        se.xclient.window=xid;
        se.xclient.data.l[0]=yes;   // 0=_NET_WM_STATE_REMOVE, 1=_NET_WM_STATE_ADD, 2=_NET_WM_STATE_TOGGLE
        se.xclient.data.l[1]=XInternAtom(DISPLAY(m_app), "_NET_WM_STATE_DEMANDS_ATTENTION", 0);
        se.xclient.data.l[2]=0;
        se.xclient.data.l[3]=0;
        se.xclient.data.l[4]=0;
        XSendEvent(DISPLAY(m_app),XDefaultRootWindow(DISPLAY(m_app)),False,SubstructureRedirectMask|SubstructureNotifyMask,&se);
    #endif //HAVE_X11
#endif //WIN32
    }
}

void dxirc::setTempKeybindings(dxStringMap bindings)
{
    m_tempKeybindings = bindings;
}

long dxirc::onCmdQuit(FXObject*, FXSelector, void*)
{
    m_preferences.m_x = getX();
    m_preferences.m_y = getY();
    m_preferences.m_w = getWidth();
    m_preferences.m_h = getHeight();
    m_preferences.m_maximized = isMaximized();
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass
                && static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType()==CHANNEL)
        {
            m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
            m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
        }
    }
    m_preferences.save();
    while(m_ircengines.no())
    {
        m_ircengines[0]->clearTarget();
        if(m_ircengines[0]->getConnected()) m_ircengines[0]->disconnect();
        m_ircengines.erase(0);
    }
#ifdef HAVE_LUA
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("quit", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        if (lua_pcall(m_scripts[j].L, 0, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
    while(m_scripts.no())
    {
        if(m_scripts[0].L != NULL) lua_close(m_scripts[0].L);
        m_scripts.erase(0);
    }
    while(m_scriptEvents.no())
    {
        m_scriptEvents.erase(0);
    }
#endif //HAVE_LUA
    m_app->exit(0);
    return 1;
}

long dxirc::onCmdRestart(FXObject*, FXSelector, void*)
{
    m_preferences.m_x = getX();
    m_preferences.m_y = getY();
    m_preferences.m_w = getWidth();
    m_preferences.m_h = getHeight();
    m_preferences.m_maximized = isMaximized();
    m_preferences.m_keybindings = m_tempKeybindings;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass
                && static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType()==CHANNEL)
        {
            m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
            m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
        }
    }
    m_preferences.save();
    while(m_ircengines.no())
    {
        m_ircengines[0]->clearTarget();
        if(m_ircengines[0]->getConnected()) m_ircengines[0]->disconnect();
        m_ircengines.erase(0);
    }
#ifdef HAVE_LUA
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("quit", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        if (lua_pcall(m_scripts[j].L, 0, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
    while(m_scripts.no())
    {
        if(m_scripts[0].L != NULL) lua_close(m_scripts[0].L);
        m_scripts.erase(0);
    }
    while(m_scriptEvents.no())
    {
        m_scriptEvents.erase(0);
    }
#endif //HAVE_LUA
#ifdef WIN32
    getApp()->exit();
    FXString str ;
    for(FXint i=0; i < argn; i++)
    {
        str += args[i];
        str += (i != argn - 1? " " : "");
    }
    ShellExecuteA(NULL, "open", args[0], argn>1?str.text():NULL, NULL, SW_SHOWNORMAL);
#else
    if(fork() == 0)
        execvp(args[0], args);
    else
        _exit(0);
#endif //WIN32
    return 1;
}

long dxirc::onCmdClose(FXObject*, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(m_trayIcon && m_preferences.m_useTray && m_preferences.m_closeToTray)
        hide();
    else
        onCmdQuit(NULL, 0, NULL);
#else
    onCmdQuit(NULL, 0, NULL);
#endif //HAVE_TRAY
    return 1;
}

long dxirc::onCmdHelp(FXObject*, FXSelector, void*)
{
    HelpDialog help(this);
    help.execute(PLACEMENT_OWNER);
    return 1;
}

long dxirc::onCmdUsers(FXObject*, FXSelector, void*)
{
    m_preferences.m_usersShown = !m_preferences.m_usersShown;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass)
        {
            IrcTabItem *tab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i));
            if(m_preferences.m_usersShown) tab->showUsers();
            else tab->hideUsers();
        }
    }
    return 1;
}

long dxirc::onCmdSpell(FXObject*, FXSelector, void*)
{
    m_preferences.m_showSpellCombo = !m_preferences.m_showSpellCombo;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
        {
            dxTabItem *tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
            tab->showSpellComboUpdated();
        }
    }
    return 1;
}

long dxirc::onCmdForceFocus(FXObject*, FXSelector, void*)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()!=&TetrisTabItem::metaClass)
            static_cast<dxTabItem*>(m_tabbook->childAtIndex(index))->setCommandFocus();
    }
    return 1;
}

//check away on channels
long dxirc::onAwayTimeout(FXObject*, FXSelector, void*)
{
    m_app->addTimeout(this, dxirc_AWAYTIMEOUT, 180000);
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass)
            static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->checkAway();
    }
    return 1;
}

//check dxirc version on dxirc.org
long dxirc::onCmdCheckVersion(FXObject*, FXSelector, void*)
{
    if(!m_versionSocket)
        m_versionSocket = new dxSocket(m_app, this, dxirc_VERSIONSOCKET);
    if(m_versionSocket->isConnecting())
    {
        dxEXMessageBox::information(this, MBOX_OK, _("Information"), "%s", _("Checking version in progress"));
        return 1;
    }
    if(m_versionSocket->isConnected())
    {
        dxEXMessageBox::information(this, MBOX_OK, _("Information"), "%s", _("Checking version in progress"));
        return 1;
    }
    updateStatus(_("Checking version in progress"));
    m_versionSocket->setTarget(this);
    m_versionSocket->setHost("dxirc.org", 80);
    m_versionSocket->connectTo();
    return 1;
}

//versionsocket connected
long dxirc::onVersionSocketConnected(FXObject*, FXSelector, void*)
{
    if(m_versionSocket)
        m_versionSocket->writeLine("GET http://www.dxirc.org/version HTTP/1.0\nUser-Agent: dxirc version\n\n", TRUE);
    return 1;
}

//versionsocket data available
long dxirc::onVersionSocketCanRead(FXObject*, FXSelector, void*)
{
    onStatusTimeout(NULL, 0, NULL);
    if(m_versionSocket)
    {
        FXint read = m_versionSocket->getBytesAvailable();
        if(!read) return 1;
        FXchar *buffer = new FXchar[read];
        read = m_versionSocket->read(buffer, read);
        FXString serverVersion = buffer;
        serverVersion = serverVersion.after('\n', serverVersion.contains('\n')-(buffer[read-1]=='\n'?1:0)).before('\n');
        if(serverVersion.empty())
        {
            return 1;
        }
        else
        {
            if(serverVersion[0] != '1') return 1;
        }
        FXString currentVersion = VERSION;
        FXint current = FXIntVal(currentVersion.before('.'))*1000+FXIntVal(currentVersion.after('.').before('.'))*10+FXIntVal(currentVersion.after('.',2));
        FXint server = FXIntVal(serverVersion.before('.'))*1000+FXIntVal(serverVersion.after('.').before('.'))*10+FXIntVal(serverVersion.after('.',2));
        if(current==server)
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("You have last version %s"), VERSION);
        else if(current<server)
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("New version %s available"), serverVersion.text());
        else
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("You have svn version %s"), VERSION);
        delete []buffer;
        m_versionSocket->disconnect();
    }
    return 1;
}

//versionsocket error
long dxirc::onVersionSocketError(FXObject*, FXSelector, void *ptr)
{
    onStatusTimeout(NULL, 0, NULL);
    dxEXMessageBox::error(m_app, MBOX_OK, _("Error"), "%s", _("Error occurred during version check"));
    if(ptr)
    {
        SocketError *err = *((SocketError**)ptr);
        if(err) delete err;
    }
    return 1;
}

long dxirc::onCmdStatus(FXObject*, FXSelector, void*)
{
    m_preferences.m_statusShown = !m_preferences.m_statusShown;
    m_statusbar->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_TOGGLESHOWN), NULL);
    return 1;
}

long dxirc::onCmdOptions(FXObject*, FXSelector, void*)
{
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass
                && static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType()==CHANNEL)
        {
            m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
            m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
        }
    }
    dxStringMap oldsmileysMap = m_preferences.m_smileysMap;
    FXbool oldUseSmileys = m_preferences.m_useSmileys;
    FXString oldThemePath = m_preferences.m_themePath;
    FXbool oldOwnServerWindow = m_preferences.m_ownServerWindow;
    FXColor oldIrcTextBackColor = m_preferences.m_colors.back;
    FXColor oldUnreadColor = m_preferences.m_unreadColor;
    FXColor oldHighlightColor = m_preferences.m_highlightColor;
    ConfigDialog dialog(this);
    if(dialog.execute(PLACEMENT_CURSOR))
    {
        FXbool recreateSmileys = oldUseSmileys!=m_preferences.m_useSmileys ? TRUE : FALSE;
        if(m_preferences.m_logging) m_logviewer->enable();
        else m_logviewer->disable();
        if(m_viewer) m_viewer->setLogPath(m_preferences.m_logPath);
#ifdef HAVE_ENCHANT
        if(m_preferences.m_useSpell && dxUtils.getAvailableSpellLangsNum()) m_spellCombo->enable();
        else m_spellCombo->disable();
#endif
        if(m_ircFont->getFont()!=m_preferences.m_ircFont)
        {
            recreateSmileys = TRUE;
            if(!m_preferences.m_ircFont.empty())
            {
                m_ircFont = new FXFont(m_app, m_preferences.m_ircFont);
                m_ircFont->create();
            }
            else
            {
                m_app->getNormalFont()->create();
                FXFontDesc fontdescription;
                m_app->getNormalFont()->getFontDesc(fontdescription);
                m_ircFont = new FXFont(m_app,fontdescription);
                m_ircFont->create();
            }
        }
        if(oldsmileysMap!=m_preferences.m_smileysMap) recreateSmileys = TRUE;
        updateTheme(oldUnreadColor, oldHighlightColor);
        updateFont();
        updateTabs(recreateSmileys);
        updateTabPosition();
        if(oldThemePath!=m_preferences.m_themePath || oldIrcTextBackColor!=m_preferences.m_colors.back) updateNickIcons();
        if(oldOwnServerWindow!=m_preferences.m_ownServerWindow) updateServerWindow();
        updateTrayColor();
        recalc();
    }
    return 1;
}

long dxirc::onCmdAlias(FXObject*, FXSelector, void*)
{
    AliasDialog dialog(this);
    dialog.execute(PLACEMENT_CURSOR);
    return 1;
}

long dxirc::onCmdLog(FXObject*, FXSelector, void*)
{
    if(m_viewer == NULL)
    {
        m_viewer = new LogViewer(m_app, m_preferences.m_logPath, m_ircFont);
        m_viewer->create();
    }
    m_viewer->show(PLACEMENT_CURSOR);
    return 1;
}

long dxirc::onCmdTransfers(FXObject*, FXSelector, void*)
{
    if(m_transfers == NULL)
        m_transfers = new DccDialog(m_app);
    m_transfers->create();
    return 1;
}

void dxirc::updateTheme(FXColor oldUnread, FXColor oldHighlight)
{
    register FXWindow *w = FXApp::instance()->getRootWindow();

    FX7Segment * sevensegment;
    FXTextField * textfield;
    FXIconList * iconlist;
    FXList * list;
    FXListBox * listbox;
    FXTreeList * treelist;
    FXComboBox * combobox;
    FXArrowButton * arrowbuton;
    dxEXButton * button;
    FXFrame * frame;
    FXLabel * label;
    FXPopup * popup;
    FXMenuTitle * menutitle;
    FXMenuCheck * menucheck;
    FXMenuRadio * menuradio;
    FXMenuCaption * menucaption;
    FXMenuSeparator * menuseparator;
    FXText * text;
    dxText * dtext;
    FXFoldingList * foldinglist;
    FXMDIChild * mdichild;
    FXTable * table;
    FXDockTitle * docktitle;
    FXPacker * packer;
    FXHeader * header;
    FXGroupBox * groupbox;
    FXScrollBar * scrollbar;
    FXSlider * slider;
    FXStatusLine * statusline;
    FXDragCorner * dragcorner;
    FXRadioButton * radiobutton;
    FXCheckButton * checkbutton;
    FXToolTip * tooltip;
    FXImageFrame * imageframe;

    FXbool update = FALSE;
    FXColor oldForeColor = m_app->getForeColor();
    if(m_app->getBaseColor() != m_preferences.m_appTheme.base)
    {
        update = TRUE;
        m_app->setBaseColor(m_preferences.m_appTheme.base);
    }
    if(m_app->getBackColor() != m_preferences.m_appTheme.back)
    {
        update = TRUE;
        m_app->setBackColor(m_preferences.m_appTheme.back);
    }
    if(m_app->getBorderColor() != m_preferences.m_appTheme.border)
    {
        update = TRUE;
        m_app->setBorderColor(m_preferences.m_appTheme.border);
    }
    if(m_app->getForeColor() != m_preferences.m_appTheme.fore)
    {
        update = TRUE;
        m_app->setForeColor(m_preferences.m_appTheme.fore);
    }
    if(m_app->getSelMenuBackColor() != m_preferences.m_appTheme.menuback)
    {
        update = TRUE;
        m_app->setSelMenuBackColor(m_preferences.m_appTheme.menuback);
    }
    if(m_app->getSelMenuTextColor() != m_preferences.m_appTheme.menufore)
    {
        update = TRUE;
        m_app->setSelMenuTextColor(m_preferences.m_appTheme.menufore);
    }
    if(m_app->getSelbackColor() != m_preferences.m_appTheme.selback)
    {
        update = TRUE;
        m_app->setSelbackColor(m_preferences.m_appTheme.selback);
    }
    if(m_app->getSelforeColor() != m_preferences.m_appTheme.selfore)
    {
        update = TRUE;
        m_app->setSelforeColor(m_preferences.m_appTheme.selfore);
    }
    if(m_app->getTipbackColor() != m_preferences.m_appTheme.tipback)
    {
        update = TRUE;
        m_app->setTipbackColor(m_preferences.m_appTheme.tipback);
    }
    if(m_app->getTipforeColor() != m_preferences.m_appTheme.tipfore)
    {
        update = TRUE;
        m_app->setTipforeColor(m_preferences.m_appTheme.tipfore);
    }
    if(m_app->getHiliteColor() != m_preferences.m_appTheme.hilite)
    {
        update = TRUE;
        m_app->setHiliteColor(m_preferences.m_appTheme.hilite);
    }
    if(m_app->getShadowColor() != m_preferences.m_appTheme.shadow)
    {
        update = TRUE;
        m_app->setShadowColor(m_preferences.m_appTheme.shadow);
    }
    if(oldUnread != m_preferences.m_unreadColor)
    {
        update = TRUE;
    }
    if(oldHighlight != m_preferences.m_highlightColor)
    {
        update = TRUE;
    }
    if(!update)
        return;

    while (w)
    {
        w->setBackColor(m_preferences.m_appTheme.base);
        if ((frame = dynamic_cast<FXFrame*> (w)))
        {
            frame->setBaseColor(m_preferences.m_appTheme.base);
            frame->setBackColor(m_preferences.m_appTheme.base);
            frame->setShadowColor(m_preferences.m_appTheme.shadow);
            frame->setHiliteColor(m_preferences.m_appTheme.hilite);
            frame->setBorderColor(m_preferences.m_appTheme.border);
            if ((label = dynamic_cast<FXLabel*> (w)))
            {
                if(label->getTextColor() == oldForeColor) label->setTextColor(m_preferences.m_appTheme.fore);
                else if(label->getTextColor() == oldUnread) label->setTextColor(m_preferences.m_unreadColor);
                else if(label->getTextColor() == oldHighlight) label->setTextColor(m_preferences.m_highlightColor);
                if ((button = dynamic_cast<dxEXButton*> (w)))
                {
                    if (dynamic_cast<FXListBox*> (button->getParent()))
                    {
                        w->setBackColor(m_preferences.m_appTheme.back);
                    }
                    else
                    {
                        w->setBackColor(m_preferences.m_appTheme.base);
                    }
                }
                else if ((checkbutton = dynamic_cast<FXCheckButton*> (w)))
                {
                    checkbutton->setCheckColor(m_preferences.m_appTheme.fore);
                    checkbutton->setBoxColor(m_preferences.m_appTheme.back);
                }
                else if ((radiobutton = dynamic_cast<FXRadioButton*> (w)))
                {
                    radiobutton->setRadioColor(m_preferences.m_appTheme.fore);
                    radiobutton->setDiskColor(m_preferences.m_appTheme.back);
                }
            }
            else if ((arrowbuton = dynamic_cast<FXArrowButton*> (w)))
            {
                arrowbuton->setArrowColor(m_preferences.m_appTheme.fore);
            }
            else if ((textfield = dynamic_cast<FXTextField*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                textfield->setTextColor(m_preferences.m_appTheme.fore);
                textfield->setSelTextColor(m_preferences.m_appTheme.selfore);
                textfield->setSelBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((docktitle = dynamic_cast<FXDockTitle*> (w)))
            {
                docktitle->setCaptionColor(m_preferences.m_appTheme.selfore);
                docktitle->setBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((header = dynamic_cast<FXHeader*> (w)))
            {
                header->setTextColor(m_preferences.m_appTheme.fore);
            }
            else if ((statusline = dynamic_cast<FXStatusLine*> (w)))
            {
                statusline->setTextColor(m_preferences.m_appTheme.fore);
            }
            else if ((sevensegment = dynamic_cast<FX7Segment*> (w)))
            {
                sevensegment->setTextColor(m_preferences.m_appTheme.fore);
            }
            else if ((slider = dynamic_cast<FXSlider*> (w)))
            {
                slider->setSlotColor(m_preferences.m_appTheme.back);
            }
            else if ((imageframe = dynamic_cast<FXImageFrame*> (w)))
            {
                imageframe->setBackColor(m_preferences.m_appTheme.back); /// fixme, only for coverframe in mainwindow
            }
        }
        else if ((packer = dynamic_cast<FXPacker*> (w)))
        {
            packer->setBaseColor(m_preferences.m_appTheme.base);
            packer->setBackColor(m_preferences.m_appTheme.base);
            packer->setShadowColor(m_preferences.m_appTheme.shadow);
            packer->setHiliteColor(m_preferences.m_appTheme.hilite);
            packer->setBorderColor(m_preferences.m_appTheme.border);
            if ((combobox = dynamic_cast<FXComboBox*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
            }
            else if ((listbox = dynamic_cast<FXListBox*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
            }
            else if ((groupbox = dynamic_cast<FXGroupBox*> (w)))
            {
                groupbox->setTextColor(m_preferences.m_appTheme.fore);
            }
        }
        else if ((popup = dynamic_cast<FXPopup*> (w)))
        {
            popup->setBaseColor(m_preferences.m_appTheme.base);
            popup->setShadowColor(m_preferences.m_appTheme.shadow);
            popup->setHiliteColor(m_preferences.m_appTheme.hilite);
            popup->setBorderColor(m_preferences.m_appTheme.border);
        }
        else if ((menucaption = dynamic_cast<FXMenuCaption*> (w)))
        {
            w->setBackColor(m_preferences.m_appTheme.base);
            menucaption->setTextColor(m_preferences.m_appTheme.fore);
            menucaption->setSelTextColor(m_preferences.m_appTheme.menufore);
            menucaption->setSelBackColor(m_preferences.m_appTheme.menuback);
            menucaption->setShadowColor(m_preferences.m_appTheme.shadow);
            menucaption->setHiliteColor(m_preferences.m_appTheme.hilite);

            if ((menucheck = dynamic_cast<FXMenuCheck*> (w)))
            {
                menucheck->setBoxColor(m_preferences.m_appTheme.back);
            }
            else if ((menuradio = dynamic_cast<FXMenuRadio*> (w)))
            {
                menuradio->setRadioColor(m_preferences.m_appTheme.back);
            }
            else if ((menutitle = dynamic_cast<FXMenuTitle*> (w)))
            {
                menutitle->setTextColor(m_preferences.m_appTheme.fore);
                menutitle->setSelTextColor(m_preferences.m_appTheme.fore);
                menutitle->setSelBackColor(m_preferences.m_appTheme.base);
            }
        }
        else if ((menuseparator = dynamic_cast<FXMenuSeparator*> (w)))
        {
            menuseparator->setShadowColor(m_preferences.m_appTheme.shadow);
            menuseparator->setHiliteColor(m_preferences.m_appTheme.hilite);
        }
        else if ((scrollbar = dynamic_cast<FXScrollBar*> (w)))
        {
            scrollbar->setShadowColor(m_preferences.m_appTheme.shadow);
            scrollbar->setHiliteColor(m_preferences.m_appTheme.hilite);
            scrollbar->setBorderColor(m_preferences.m_appTheme.border);
            scrollbar->setArrowColor(m_preferences.m_appTheme.fore);
        }
        else if ((dragcorner = dynamic_cast<FXDragCorner*> (w)))
        {
            dragcorner->setShadowColor(m_preferences.m_appTheme.shadow);
            dragcorner->setHiliteColor(m_preferences.m_appTheme.hilite);
        }
        else if (dynamic_cast<FXScrollArea*> (w))
        {
            if ((text = dynamic_cast<FXText*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                text->setTextColor(m_preferences.m_appTheme.fore);
                text->setSelTextColor(m_preferences.m_appTheme.selfore);
                text->setSelBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((dtext = dynamic_cast<dxText*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                dtext->setTextColor(m_preferences.m_appTheme.fore);
                dtext->setSelTextColor(m_preferences.m_appTheme.selfore);
                dtext->setSelBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((list = dynamic_cast<FXList*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                list->setTextColor(m_preferences.m_appTheme.fore);
                list->setSelTextColor(m_preferences.m_appTheme.selfore);
                list->setSelBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((treelist = dynamic_cast<FXTreeList*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                treelist->setTextColor(m_preferences.m_appTheme.fore);
                treelist->setLineColor(m_preferences.m_appTheme.shadow);
                treelist->setSelTextColor(m_preferences.m_appTheme.selfore);
                treelist->setSelBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((iconlist = dynamic_cast<FXIconList*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                iconlist->setTextColor(m_preferences.m_appTheme.fore);
                iconlist->setSelTextColor(m_preferences.m_appTheme.selfore);
                iconlist->setSelBackColor(m_preferences.m_appTheme.selback);
            }
            else if ((foldinglist = dynamic_cast<FXFoldingList*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                foldinglist->setTextColor(m_preferences.m_appTheme.fore);
                foldinglist->setSelTextColor(m_preferences.m_appTheme.selfore);
                foldinglist->setSelBackColor(m_preferences.m_appTheme.selback);
                foldinglist->setLineColor(m_preferences.m_appTheme.shadow);
            }
            else if ((table = dynamic_cast<FXTable*> (w)))
            {
                w->setBackColor(m_preferences.m_appTheme.back);
                table->setTextColor(m_preferences.m_appTheme.fore);
                table->setSelTextColor(m_preferences.m_appTheme.selfore);
                table->setSelBackColor(m_preferences.m_appTheme.selback);
            }
        }
        else if ((mdichild = dynamic_cast<FXMDIChild*> (w)))
        {
            mdichild->setBackColor(m_preferences.m_appTheme.base);
            mdichild->setBaseColor(m_preferences.m_appTheme.base);
            mdichild->setShadowColor(m_preferences.m_appTheme.shadow);
            mdichild->setHiliteColor(m_preferences.m_appTheme.hilite);
            mdichild->setBorderColor(m_preferences.m_appTheme.border);
            mdichild->setTitleColor(m_preferences.m_appTheme.selfore);
            mdichild->setTitleBackColor(m_preferences.m_appTheme.selback);
        }
        else if ((tooltip = dynamic_cast<FXToolTip*> (w)))
        {
            tooltip->setTextColor(m_preferences.m_appTheme.tipfore);
            tooltip->setBackColor(m_preferences.m_appTheme.tipback);
        }

        w->update();
        if (w->getFirst())
        {
            w = w->getFirst();
            continue;
        }
        while (!w->getNext() && w->getParent())
        {
            w = w->getParent();
        }
        w = w->getNext();
    }
}

//update tray backround color for X11
void dxirc::updateTrayColor()
{
#ifndef WIN32
#ifdef HAVE_TRAY
    if(m_preferences.m_useTray && m_trayIcon)
        m_trayIcon->setColor(m_preferences.m_trayColor);
#endif //HAVE_TRAY
#endif //WIN32
}

void dxirc::updateFont()
{
    m_app->getNormalFont()->destroy();
    m_app->getNormalFont()->setFont(m_preferences.m_font);
    m_app->getNormalFont()->create();
    register FXWindow *w = this;
    while(w)
    {
        w->recalc();
        if(w->getFirst())
        {
            w = w->getFirst();
            continue;
        }
        while(!w->getNext() && w->getParent())
        {
            w = w->getParent();
        }
        w = w->getNext();
    }
}

void dxirc::updateTabs(FXbool recreateSmileys)
{
    if(recreateSmileys)
    {
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
            {
                static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->removeSmileys();
            }
        }
        createSmileys();
    }
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass)
        {
            IrcTabItem *irctab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i));
            irctab->preferencesUpdated();
            irctab->setIrcFont(m_ircFont);
            irctab->setSmileys(m_preferences.m_useSmileys, m_smileys);
        }
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&DccTabItem::metaClass)
        {
            DccTabItem *irctab = static_cast<DccTabItem*>(m_tabbook->childAtIndex(i));
            irctab->preferencesUpdated();
            irctab->setIrcFont(m_ircFont);
            irctab->setSmileys(m_preferences.m_useSmileys, m_smileys);
        }
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&TetrisTabItem::metaClass)
        {
            TetrisTabItem *tetristab = static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(i));
            tetristab->setColor(m_preferences.m_colors);
        }
    }
    //update font in LogViewer too. Both must be same
    if(m_viewer) m_viewer->setFont(m_ircFont);
}

void dxirc::updateTabPosition()
{
    switch(m_preferences.m_tabPosition) {
        case 0: //bottom
            {
                m_tabbook->setTabStyle(TABBOOK_BOTTOMTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_BOTTOM);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        case 1: //left
            {
                m_tabbook->setTabStyle(TABBOOK_LEFTTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_LEFT);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing |= PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        case 2: //top
            {
                m_tabbook->setTabStyle(TABBOOK_TOPTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_TOP);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        case 3: //right
            {
                m_tabbook->setTabStyle(TABBOOK_RIGHTTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_RIGHT);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing |= PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        default:
            {
                m_tabbook->setTabStyle(TABBOOK_BOTTOMTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_BOTTOM);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }
    }
}

void dxirc::updateNickIcons()
{
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass)
        {
            static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->clearNickIcons();
        }
    }
    recreateNickIcons(m_app, m_preferences.m_themePath, m_preferences.m_colors.back);
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass)
        {
            static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->recreateNickIcons(m_preferences.m_themePath!="internal");
        }
    }
}

void dxirc::updateServerWindow()
{
    if(m_preferences.m_ownServerWindow)
    {
        for(FXint i=0; i<m_ircengines.no(); i++)
            createIrcTab(m_ircengines[i]->getNetworkName(), ICO_SERVER, SERVER, m_ircengines[i]);
    }
    else
    {
        for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass
                    && static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType()==SERVER)
            {
                IrcEngine *currentserver = NULL;
                for(FXint j=0; j < m_ircengines.no(); j++)
                {
                    if(m_ircengines[j]->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))))
                    {
                        currentserver = m_ircengines[j];
                        break;
                    }
                }
                if(currentserver) currentserver->removeTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i)));
                delete m_tabbook->childAtIndex(i);
                delete m_tabbook->childAtIndex(i);
            }
        }
        m_tabbook->recalc();
        if(m_tabbook->numChildren())
        {
            m_tabbook->setCurrent(0, TRUE);
        }
        sortTabs();
        updateMenus();
    }
}

void dxirc::createSmileys()
{
    while(m_smileys.no())
    {
        delete m_smileys[0].icon;
        m_smileys[0].icon = NULL;
        m_smileys.erase(0);
    }
    if((FXint)m_preferences.m_smileysMap.size() && m_ircFont)
    {
        StringIt it;
        for(it=m_preferences.m_smileysMap.begin(); it!=m_preferences.m_smileysMap.end(); it++)
        {
            dxSmiley smiley;
            smiley.text = (*it).first;
            smiley.path = (*it).second;
            smiley.icon = makeIcon(m_app, smiley.path, m_ircFont->getFontHeight(), m_preferences.m_colors.back);
            m_smileys.append(smiley);
        }
    }
}

long dxirc::onCmdAbout(FXObject*, FXSelector, void*)
{
    AboutDialog about(this);
    about.execute(PLACEMENT_OWNER);
    return 1;
}

long dxirc::onCmdDccCancel(FXObject*, FXSelector, void *ptr)
{
    FXint index = (FXint)(FXival)ptr;
    if(index >= m_dccfilesList.no())
        return 1;
    for(FXint i=0; i<m_dccengines.no(); i++)
    {
        if(m_dccengines[i]->hasDccFile(m_dccfilesList[index]))
        {
            m_dccengines[i]->disconnect();
            return 1;
        }
    }
    //dccfile hasn't server
    m_dccfilesList[index].canceled = TRUE;
    m_dccfilesList[index].token = -1;
    return 1;
}

long dxirc::onCmdServers(FXObject*, FXSelector, void*)
{
    dxUtils.debugLine("OnCommandServers");
    ServerDialog *dialog = new ServerDialog(this, m_preferences.m_serverList);
    if(dialog->execute(PLACEMENT_OWNER))
    {
        FXint indexJoin = -1;
        m_preferences.m_serverList = dialog->getServers();
        indexJoin = dialog->getIndexJoin();
        if(indexJoin != -1 && !serverExist(m_preferences.m_serverList[indexJoin].hostname, m_preferences.m_serverList[indexJoin].port, m_preferences.m_serverList[indexJoin].nick))
        {
            connectServer(m_preferences.m_serverList[indexJoin].hostname, m_preferences.m_serverList[indexJoin].port, m_preferences.m_serverList[indexJoin].passwd, m_preferences.m_serverList[indexJoin].nick, m_preferences.m_serverList[indexJoin].realname, m_preferences.m_serverList[indexJoin].channels, m_preferences.m_serverList[indexJoin].commands, m_preferences.m_serverList[indexJoin].useSsl);
        }
    }
    return 1;
}

long dxirc::onCmdConnect(FXObject*, FXSelector, void*)
{
    FXDialogBox serverEdit(this, _("Quick connect"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    serverEdit.getAccelTable()->addAccel(KEY_Return, &serverEdit, FXDialogBox::ID_ACCEPT);
    FXVerticalFrame *contents = new FXVerticalFrame(&serverEdit, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    new FXLabel(matrix, _("Hostname:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *hostname = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
#ifdef DEBUG
    hostname->setText("localhost");
#else
    hostname->setText("irc.freenode.net");
#endif

    new FXLabel(matrix, _("Port:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXSpinner *port = new FXSpinner(matrix, 23,NULL, 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    port->setRange(0, 65536);
    port->setValue(6667);

    new FXLabel(matrix, _("Password:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *passwd = new FXTextField(matrix, 25, NULL, 0, TEXTFIELD_PASSWD|TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);

    new FXLabel(matrix, _("Nickname:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *nick = new FXTextField(matrix, 25, NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    nick->setText(FXSystem::currentUserName());

    new FXLabel(matrix, _("Realname:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField* realname = new FXTextField(matrix, 25, NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);

    new FXLabel(matrix, _("Channel(s):\tChannels need to be comma separated"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *channel = new FXTextField(matrix, 25, NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    channel->setText("#");

#ifdef HAVE_OPENSSL
    new FXLabel(matrix, _("Use SSL:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXCheckButton *ussl = new FXCheckButton(matrix, "", NULL, 0);
#endif //HAVE_OPENSSL

    new FXLabel(matrix, _("Commands on connection:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXHorizontalFrame *commandsbox=new FXHorizontalFrame(matrix, LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0);
    FXText *command = new FXText(commandsbox, NULL, 0, TEXT_WORDWRAP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    command->setVisibleRows(4);
    command->setVisibleColumns(25);

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &serverEdit, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &serverEdit, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    if (serverEdit.execute(PLACEMENT_OWNER))
    {
#ifdef HAVE_OPENSSL
        connectServer(hostname->getText(), port->getValue(), passwd->getText(), nick->getText(), realname->getText(), channel->getText(), command->getText(), ussl->getCheck());
#else
        connectServer(hostname->getText(), port->getValue(), passwd->getText(), nick->getText(), realname->getText(), channel->getText(), command->getText(), FALSE);
#endif //HAVE_OPENSSL
    }
    return 1;
}

long dxirc::onTabConnect(FXObject*, FXSelector, void *data)
{
    ServerInfo *srv = (ServerInfo*)data;
    connectServer(srv->hostname, srv->port, srv->passwd, srv->nick, srv->realname, srv->channels, "", FALSE);
    return 1;
}

void dxirc::connectServer(FXString hostname, FXint port, FXString pass, FXString nick, FXString rname, FXString channels, FXString commands, FXbool ssl)
{
    dxUtils.debugLine("ConnectServer");
    if(m_ircengines.no() == 1 && !m_ircengines[0]->getConnected() && !m_ircengines[0]->getConnecting())
    {
        m_ircengines[0]->setServerName(hostname);
        m_ircengines[0]->setServerPort(port);
        m_ircengines[0]->setServerPassword(pass);
        nick.length() ? m_ircengines[0]->setNickName(nick) : m_ircengines[0]->setNickName("_xxx_");
        nick.length() ? m_ircengines[0]->setUserName(nick) : m_ircengines[0]->setUserName("_xxx_");
        rname.length() ? m_ircengines[0]->setRealName(rname) : m_ircengines[0]->setRealName(nick.length() ? nick : "_xxx_");
        if(channels.length()>1) m_ircengines[0]->setStartChannels(channels);
        if(commands.length()) m_ircengines[0]->setStartCommands(commands);
#ifndef HAVE_OPENSSL
        ssl = FALSE;
#endif //HAVE_OPENSSL
        m_ircengines[0]->setUseSsl(ssl);
        if(!m_tabbook->numChildren())
        {
            createIrcTab(hostname, ICO_SERVER, SERVER, m_ircengines[0]);
            m_tabbook->recalc();
        }
        else
        {
            FXbool needNewTab = TRUE;
            for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
            {
                if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass
                        && static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType()==SERVER)
                {
                    static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->setText(hostname);
                    sortTabs();
                    needNewTab = FALSE;
                    break;
                }
            }
            if(needNewTab) createIrcTab(hostname, ICO_SERVER, SERVER, m_ircengines[0]);
        }
        m_ircengines[0]->clearAttempts();
        m_ircengines[0]->startConnection();
    }
    else if(!serverExist(hostname, port, nick))
    {
        IrcEngine *server = new IrcEngine(m_app, this, channels.length()>1 ? channels : "", commands.length() ? commands : "");
        m_ircengines.prepend(server);
        m_ircengines[0]->setServerName(hostname);
        m_ircengines[0]->setServerPort(port);
        m_ircengines[0]->setServerPassword(pass);
        nick.length() ? m_ircengines[0]->setNickName(nick) : m_ircengines[0]->setNickName("_xxx_");
        nick.length() ? m_ircengines[0]->setUserName(nick) : m_ircengines[0]->setUserName("_xxx_");
        rname.length() ? m_ircengines[0]->setRealName(rname) : m_ircengines[0]->setRealName(nick.length() ? nick : "_xxx_");
#ifndef HAVE_OPENSSL
        ssl = FALSE;
#endif //HAVE_OPENSSL
        m_ircengines[0]->setUseSsl(ssl);
        createIrcTab(hostname, ICO_SERVER, SERVER, m_ircengines[0]);
        m_ircengines[0]->clearAttempts();
        m_ircengines[0]->startConnection();
    }
    updateMenus();
}

long dxirc::onCmdDisconnect(FXObject*, FXSelector, void*)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&TetrisTabItem::metaClass) return 0;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&IrcTabItem::metaClass)
        {
            IrcTabItem *currenttab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index));
            IrcEngine *currentserver = NULL;
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(currenttab))
                {
                    currentserver = m_ircengines[i];
                    break;
                }
            }
            if(currentserver == NULL) return 0;
            if(currentserver->getConnected())
            {
                FXDialogBox confirmDialog(this, _("Confirm disconnect"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
                FXVerticalFrame *contents = new FXVerticalFrame(&confirmDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
                new FXLabel(contents, FXStringFormat(_("Disconnect server: %s\nPort: %d\nNick: %s"), currentserver->getNetworkName().text(), currentserver->getServerPort(), currentserver->getNickName().text()), NULL, JUSTIFY_LEFT|ICON_BEFORE_TEXT);
                FXHorizontalFrame* buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
                new dxEXButton(buttonframe, _("&No"), NULL, &confirmDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
                new dxEXButton(buttonframe, _("&Yes"), NULL, &confirmDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
                if (confirmDialog.execute(PLACEMENT_OWNER))
                {
                    currentserver->disconnect();
                    for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
                    {
                        if(currentserver->findTarget(m_tabbook->childAtIndex(i)))
                        {
                            if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == CHANNEL)
                            {
                                m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
                                m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
                            }
                            currentserver->removeTarget(m_tabbook->childAtIndex(i));
                            delete m_tabbook->childAtIndex(i);
                            delete m_tabbook->childAtIndex(i);
                        }
                    }
                    m_tabbook->recalc();
                    if(m_tabbook->numChildren())
                    {
                        sortTabs();
                        m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
                    }
                }
            }
            else
            {
                currentserver->disconnect();
                for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
                {
                    if(currentserver->findTarget(m_tabbook->childAtIndex(i)))
                    {
                        if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == CHANNEL)
                        {
                            m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
                            m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
                        }
                        currentserver->removeTarget(m_tabbook->childAtIndex(i));
                        delete m_tabbook->childAtIndex(i);
                        delete m_tabbook->childAtIndex(i);
                    }
                }
                m_tabbook->recalc();
                if(m_tabbook->numChildren())
                {
                    sortTabs();
                    m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
                }
            }
        }
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&DccTabItem::metaClass)
        {
            DccTabItem *currenttab = static_cast<DccTabItem*>(m_tabbook->childAtIndex(index));
            if(currenttab->getConnected())
            {
                FXDialogBox confirmDialog(this, _("Confirm disconnect"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
                FXVerticalFrame *contents = new FXVerticalFrame(&confirmDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
                new FXLabel(contents, FXStringFormat(_("Disconnect dcc connection: %s\nPort: %d\nWith: %s"), currenttab->getServerName().text(), currenttab->getServerPort(), currenttab->getText().text()), NULL, JUSTIFY_LEFT|ICON_BEFORE_TEXT);
                FXHorizontalFrame* buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
                new dxEXButton(buttonframe, _("&No"), NULL, &confirmDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
                new dxEXButton(buttonframe, _("&Yes"), NULL, &confirmDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
                if (confirmDialog.execute(PLACEMENT_OWNER))
                {
                    currenttab->disconnect();
                    delete m_tabbook->childAtIndex(index);
                    delete m_tabbook->childAtIndex(index);
                    m_tabbook->recalc();
                    if(m_tabbook->numChildren())
                    {
                        sortTabs();
                        m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
                    }
                }
            }
            else
            {
                delete m_tabbook->childAtIndex(index);
                delete m_tabbook->childAtIndex(index);
                m_tabbook->recalc();
                if(m_tabbook->numChildren())
                {
                    sortTabs();
                    m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
                }
            }
        }
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&BoatsTabItem::metaClass)
        {
            BoatsTabItem *currenttab = static_cast<BoatsTabItem*>(m_tabbook->childAtIndex(index));
            IrcEngine *currentserver = NULL;
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(currenttab))
                {
                    currentserver = m_ircengines[i];
                    break;
                }
            }
            if(currentserver == NULL) return 0;
            if(currentserver->getConnected())
            {
                FXDialogBox confirmDialog(this, _("Confirm disconnect"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
                FXVerticalFrame *contents = new FXVerticalFrame(&confirmDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
                new FXLabel(contents, FXStringFormat(_("Disconnect server: %s\nPort: %d\nNick: %s"), currentserver->getNetworkName().text(), currentserver->getServerPort(), currentserver->getNickName().text()), NULL, JUSTIFY_LEFT|ICON_BEFORE_TEXT);
                FXHorizontalFrame* buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
                new dxEXButton(buttonframe, _("&No"), NULL, &confirmDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
                new dxEXButton(buttonframe, _("&Yes"), NULL, &confirmDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
                if (confirmDialog.execute(PLACEMENT_OWNER))
                {
                    currentserver->disconnect();
                    for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
                    {
                        if(currentserver->findTarget(m_tabbook->childAtIndex(i)))
                        {
                            if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == CHANNEL)
                            {
                                m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
                                m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
                            }
                            currentserver->removeTarget(m_tabbook->childAtIndex(i));
                            delete m_tabbook->childAtIndex(i);
                            delete m_tabbook->childAtIndex(i);
                        }
                    }
                    m_tabbook->recalc();
                    if(m_tabbook->numChildren())
                    {
                        sortTabs();
                        m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
                    }
                }
            }
            else
            {
                currentserver->disconnect();
                for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
                {
                    if(currentserver->findTarget(m_tabbook->childAtIndex(i)))
                    {
                        if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == CHANNEL)
                        {
                            m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
                            m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
                        }
                        currentserver->removeTarget(m_tabbook->childAtIndex(i));
                        delete m_tabbook->childAtIndex(i);
                        delete m_tabbook->childAtIndex(i);
                    }
                }
                m_tabbook->recalc();
                if(m_tabbook->numChildren())
                {
                    sortTabs();
                    m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
                }
            }
        }
    }
    updateMenus();
    return 1;
}

//handle dxirc_RECONNECT
long dxirc::onCmdReconnect(FXObject*, FXSelector, void*)
{
    for(FXint i=0; i<m_ircengines.no(); i++)
        m_ircengines[i]->reconnect();
    return 1;
}

long dxirc::onIrcEvent(FXObject *obj, FXSelector, void *data)
{
    IrcEvent *ev = (IrcEvent *)data;
    IrcEngine *server = (IrcEngine *)obj;
    if(ev->eventType == IRC_NEWCHANNEL)
    {
        onIrcNewchannel(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_QUERY)
    {
        onIrcQuery(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_PART)
    {
        onIrcPart(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_KICK)
    {
        onIrcKick(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DISCONNECT)
    {
        onIrcDisconnect(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_RECONNECT || ev->eventType == IRC_RECONNECTALL || ev->eventType == IRC_CONNECT)
    {
        onIrcConnectAndReconnect(ev);
        return 1;
    }
    if(ev->eventType == IRC_ENDMOTD)
    {
        onIrcEndmotd();
        return 1;
    }
    if(ev->eventType == IRC_PRIVMSG || ev->eventType == IRC_ACTION)
    {
        onIrcPrivmsgAndAction(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_JOIN)
    {
        onIrcJoin(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_QUIT)
    {
        onIrcQuit(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_INVITE)
    {
        onIrcInvite(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCCHAT)
    {
        onIrcDccChat(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCSERVER)
    {
        onIrcDccServer(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCIN)
    {
        onIrcDccIn(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCOUT)
    {
        onIrcDccOut(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCPOUT)
    {
        onIrcDccPout(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCMYTOKEN)
    {
        onIrcDccMyToken(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCTOKEN)
    {
        onIrcDccToken(server, ev);
        return 1;
    }
    //IrcEngine couldn't send IRC_DCCPOSITION :)
    if(ev->eventType == IRC_DCCPOSITION)
    {
        return 1;
    }
    if(ev->eventType == IRC_DCCRESUME)
    {
        onIrcDccResume(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCPRESUME)
    {
        onIrcDccPresume(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCACCEPT)
    {
        onIrcDccAccept(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_DCCPACCEPT)
    {
        onIrcDccPaccept(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_BOATSINVITE)
    {
        onIrcBoatsInvite(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_BOATSACCEPT)
    {
        onIrcBoatsAccept(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_RAW)
    {
        onIrcRaw(server, ev);
        return 1;
    }
    if(ev->eventType == IRC_323)
    {
        onIrc323(server);
        return 1;
    }
    return 1;
}

long dxirc::onDccEvent(FXObject *obj, FXSelector, void *data)
{
    IrcEvent *ev = (IrcEvent *)data;
    DccEngine *dcc = (DccEngine *)obj;
    if(ev->eventType == IRC_DISCONNECT)
    {
        appendIrcStyledText(FXStringFormat("\"%s\": %s", FXPath::name(ev->dccFile.path).text(), ev->param1.text()), 4, FALSE);
        return 1;
    }
    if(ev->eventType == IRC_ERROR)
    {
        appendIrcStyledText(FXStringFormat("\"%s\": %s", FXPath::name(ev->dccFile.path).text(), ev->param1.text()), 4, FALSE);
        return 1;
    }
    if(ev->eventType == IRC_DCCPOSITION)
    {
        onIrcDccPosition(dcc, ev);
        return 1;
    }
    return 1;
}

//handle IrcEvent IRC_NEWCHANNEL
void dxirc::onIrcNewchannel(IrcEngine *server, IrcEvent *ev)
{
    FXint serverTabIndex = getServerTab(server);
    if(tabExist(server, ev->param1, CHANNEL))
    {
        return;
    }
    if(serverTabIndex != -1 && !m_preferences.m_ownServerWindow)
    {
        static_cast<IrcTabItem*>(m_tabbook->childAtIndex(serverTabIndex))->setType(CHANNEL, ev->param1);
        static_cast<IrcTabItem*>(m_tabbook->childAtIndex(serverTabIndex))->setTextColor(m_preferences.m_appTheme.fore);
        m_tabbook->setCurrent(FXMAX(0,serverTabIndex/2-1), TRUE);
        sortTabs();
    }
    else
    {
        createIrcTab(ev->param1, ICO_CHANNEL, CHANNEL, server);
    }
    updateMenus();
}

//handle IrcEvent IRC_QUERY
void dxirc::onIrcQuery(IrcEngine *server, IrcEvent *ev)
{
    if(tabExist(server, ev->param1, QUERY))
        return;
    FXint serverTabIndex = getServerTab(server);
    if(serverTabIndex != -1 && !m_preferences.m_ownServerWindow)
    {
        static_cast<IrcTabItem*>(m_tabbook->childAtIndex(serverTabIndex))->setType(QUERY, ev->param1);
        m_tabbook->setCurrent(FXMAX(0,serverTabIndex/2-1), TRUE);
        sortTabs();
    }
    else
    {
        setCurrentTabById(createIrcTab(ev->param1, ICO_QUERY, QUERY, server));
    }
    updateMenus();
}

//handle IrcEvent IRC_PART
void dxirc::onIrcPart(IrcEngine *server, IrcEvent *ev)
{
#ifdef HAVE_LUA
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("part", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
//                    lua_pushstring(m_scripts[j].L, m_scriptEvents[i].funcname.text());
//                    lua_gettable(m_scripts[j].L, LUA_GLOBALSINDEX);
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, ev->param1.text());
                        lua_pushinteger(m_scripts[j].L, getTabId(server, ev->param2));
                        if (lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
    if(isFriend(ev->param1, ev->param2, server->getNetworkName()))
    {
        if(m_preferences.m_sounds && m_preferences.m_soundDisconnect)
            dxutils::playFile(m_preferences.m_pathDisconnect);
        if(m_preferences.m_notify && m_preferences.m_notifyDisconnect)
            showNotify(FXStringFormat(_("%s has parted %s"), ev->param1.text(), ev->param2.text()));
    }
    if(tabExist(server, ev->param2, CHANNEL))
    {
        if(ev->param1 == server->getNickName())
        {
            if(server->getConnected() && isLastTab(server))
            {
                for(FXint j = 0; j < m_tabbook->numChildren(); j+=2)
                {
                    if(server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))))
                    {
                        m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))->getText());
                        m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))->getSpellLang()));
                        static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))->setType(SERVER, server->getNetworkName());
                        m_tabbook->setCurrent(j/2-1, TRUE);
                        break;
                    }
                }
            }
            else
            {
                FXint index = -1;
                for(FXint j = 0; j < m_tabbook->numChildren(); j+=2)
                {
                    if((comparecase(static_cast<FXTabItem*>(m_tabbook->childAtIndex(j))->getText(), ev->param2) == 0) && server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j)))) index = j;
                }
                if(index == -1) return;
                m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index))->getText());
                m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index))->getSpellLang()));
                server->removeTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index)));
                delete m_tabbook->childAtIndex(index);
                delete m_tabbook->childAtIndex(index);
                m_tabbook->recalc();
                if(m_tabbook->numChildren())
                {
                    m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
                }
            }
            sortTabs();
            updateMenus();
        }
    }
}

//handle IrcEvent IRC_KICK
void dxirc::onIrcKick(IrcEngine *server, IrcEvent *ev)
{
    if(ev->param2 == server->getNickName())
    {
        if(server->getConnected() && isLastTab(server))
        {
            for(FXint j = 0; j < m_tabbook->numChildren(); j+=2)
            {
                if(server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))))
                {
                    static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j))->setType(SERVER, server->getNetworkName());
                    m_tabbook->setCurrent(j/2-1, TRUE);
                    break;
                }
            }
        }
        else
        {
            FXint index = -1;
            for(FXint j = 0; j < m_tabbook->numChildren(); j+=2)
            {
                if((comparecase(static_cast<FXTabItem*>(m_tabbook->childAtIndex(j))->getText(), ev->param3) == 0) && server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(j)))) index = j;
            }
            if(index == -1) return;
            server->removeTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index)));
            delete m_tabbook->childAtIndex(index);
            delete m_tabbook->childAtIndex(index);
            m_tabbook->recalc();
            if(m_tabbook->numChildren())
            {
                m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
            }
        }
        sortTabs();
        updateMenus();
    }
}

//handle IrcEvent IRC_DISCONNECT
void dxirc::onIrcDisconnect(IrcEngine *server, IrcEvent *ev)
{
    for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
    {
        if(server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))))
        {
            if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == CHANNEL)
            {
                m_preferences.m_langs.erase(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText());
                m_preferences.m_langs.insert(StringPair(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getText(), static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getSpellLang()));
            }
            server->removeTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i)));
            delete m_tabbook->childAtIndex(i);
            delete m_tabbook->childAtIndex(i);
        }
    }
    m_tabbook->recalc();
    if(m_tabbook->numChildren())
    {
        sortTabs();
        m_tabbook->setCurrent(0, TRUE);
    }
    updateMenus();
    updateStatus(ev->param1);
}

//handle IrcEvent IRC_CONNECT and IRC_RECONNECT
void dxirc::onIrcConnectAndReconnect(IrcEvent *ev)
{
    updateStatus(ev->param1);
}

//handle IrcEvent IRC_ENDMOTD
void dxirc::onIrcEndmotd()
{
    updateMenus();
}

//handle IrcEvent IRC_PRIVMSG and IRC_ACTION
void dxirc::onIrcPrivmsgAndAction(IrcEngine *server, IrcEvent *ev)
{

#ifdef HAVE_TRAY
    if(m_trayIcon && m_preferences.m_notifyTray && m_trayIcon->getIcon() == ICO_TRAY && !shown())
        m_trayIcon->setIcon(ICO_NEWMSG);
#endif //HAVE_TRAY
#ifdef HAVE_LUA
    if(!m_scripts.no() || !m_scriptEvents.no()) return;
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("privmsg", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, ev->param1.text());
                        lua_pushstring(m_scripts[j].L, ev->param3.text());
                        lua_pushinteger(m_scripts[j].L, getTabId(server, ev->param2 == server->getNickName() ? ev->param1 : ev->param2));
                        if (lua_pcall(m_scripts[j].L, 3, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

//handle IrcEvent IRC_INVITE
void dxirc::onIrcInvite(IrcEngine *server, IrcEvent *ev)
{
    if(m_preferences.m_ignoreCommandsList.contains("invite"))
        return;
    if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("%s invites you on %s on %s \n Do you want connect?"), ev->param1.text(), ev->param2.text(), server->getNetworkName().text()) == 1)
        server->sendJoin(ev->param2);
}

//handle IrcEvent IRC_JOIN
void dxirc::onIrcJoin(IrcEngine *server, IrcEvent *ev)
{
    if(isFriend(ev->param1, ev->param2, server->getNetworkName()))
    {
        if(m_preferences.m_sounds && m_preferences.m_soundConnect)
            dxutils::playFile(m_preferences.m_pathConnect);
        if(m_preferences.m_notify && m_preferences.m_notifyConnect)
            showNotify(FXStringFormat(_("%s has joined to %s"), ev->param1.text(), ev->param2.text()));
    }
#ifdef HAVE_LUA
    if(server->isUserIgnored(ev->param1, ev->param2)) return;
    if(!m_scripts.no() || !m_scriptEvents.no()) return;
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("join", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, ev->param1.text());
                        lua_pushinteger(m_scripts[j].L, getTabId(server, ev->param2));
                        if (lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

//handle IrcEvent IRC_QUIT
void dxirc::onIrcQuit(IrcEngine *server, IrcEvent *ev)
{
    if(isFriend(ev->param1, "all", server->getNetworkName()))
    {
        if(m_preferences.m_sounds && m_preferences.m_soundDisconnect)
            dxutils::playFile(m_preferences.m_pathDisconnect);
        if(m_preferences.m_notify && m_preferences.m_notifyDisconnect)
            showNotify(FXStringFormat(_("%s has quit"), ev->param1.text()));
    }
}

//handle IrcEvent IRC_DCCCHAT
void dxirc::onIrcDccChat(IrcEngine *server, IrcEvent *ev)
{
    if(m_preferences.m_autoDccChat)
        createDccTab(server->getNickName(), ev->param1, ev->param2, FXIntVal(ev->param3), FXIntVal(ev->param3), FALSE, server);
    if(!m_preferences.m_autoDccChat && dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("%s offers DCC Chat on %s(%s) port %s.\n Do you want connect?"), ev->param1.text(), ev->param2.text(), server->getHostname(ev->param2).text(), ev->param3.text()) == 1)
        createDccTab(server->getNickName(), ev->param1, ev->param2, FXIntVal(ev->param3), FXIntVal(ev->param3), FALSE, server);
}

//handle IrcEvent IRC_DCCSERVER
void dxirc::onIrcDccServer(IrcEngine *server, IrcEvent *ev)
{
    createDccTab(server->getNickName(), ev->param1, server->getLocalIP(), m_preferences.m_dccPortD, m_preferences.m_dccPortH, TRUE, server);
}

//handle IrcEvent IRC_DCCIN
void dxirc::onIrcDccIn(IrcEngine *server, IrcEvent *ev)
{
    if(m_preferences.m_autoDccFile)
    {
        DccFile dcc;
        dcc.path = getUniqueName(m_preferences.m_dccPath, FXPath::stripExtension(ev->param3), FXPath::extension(ev->param3));
        dcc.speed = 0;
        dcc.currentPosition = 0;
        dcc.size = FXLongVal(ev->param4);
        dcc.type = DCC_IN;
        dcc.started = FALSE;
        dcc.canceled = FALSE;
        dcc.finished = FALSE;
        dcc.finishedPosition = 0;
        dcc.token = -1;
        dcc.ip = ev->param2.before('@');
        dcc.port = FXIntVal(ev->param2.after('@'));
        dcc.nick = ev->param1;
        for(FXint i=0; i<m_dccfilesList.no(); i++)
        {
            if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
            {
                m_dccfilesList.erase(i);
                break;
            }
        }
        m_dccfilesList.append(dcc);
        if(isForResume(FXPath::name(dcc.path)))
        {
            server->sendCtcp(ev->param1, "DCC RESUME "+FXPath::name(dcc.path)+" "+FXStringVal(dcc.port)+" "+FXStringVal(FXStat::size(dcc.path+".part")));
        }
        else
        {
            DccEngine *engine = new DccEngine(m_app, this, dcc, server);
            m_dccengines.append(engine);
            engine->startConnection();
            appendIrcStyledText(FXStringFormat(_("Receiving \"%s\" from %s"), ev->param3.text(), ev->param1.text()),8,FALSE);
        }
    }
    else
    {
        if(isForResume(ev->param3)
            &&dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("%s offers file %s with size %s over DCC .\nFile is already partially downloaded.\nDo you want continue in download?"), ev->param1.text(), ev->param3.text(), dxutils::getFileSize(ev->param4).text()) == 1)
        {
            DccFile dcc;
            dcc.path = m_preferences.m_dccPath+PATHSEPSTRING+ev->param3;
            dcc.speed = 0;
            dcc.currentPosition = 0;
            dcc.size = FXLongVal(ev->param4);
            dcc.type = DCC_IN;
            dcc.started = FALSE;
            dcc.canceled = FALSE;
            dcc.finished = FALSE;
            dcc.finishedPosition = 0;
            dcc.token = -1;
            dcc.ip = ev->param2.before('@');
            dcc.port = FXIntVal(ev->param2.after('@'));
            dcc.nick = ev->param1;
            for(FXint i=0; i<m_dccfilesList.no(); i++)
            {
                if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                {
                    m_dccfilesList.erase(i);
                    break;
                }
            }
            m_dccfilesList.append(dcc);
            server->sendCtcp(ev->param1, "DCC RESUME "+FXPath::name(dcc.path)+" "+FXStringVal(dcc.port)+" "+FXStringVal(FXStat::size(dcc.path+".part")));
            return;
        }
        if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("%s offers file %s with size %s over DCC .\nDo you want connect?"), ev->param1.text(), ev->param3.text(), dxutils::getFileSize(ev->param4).text()) == 1)
        {
            dxEXFileDialog dialog(this, _("Save file"));
            dialog.setFilename(m_preferences.m_dccPath+PATHSEPSTRING+ev->param3);
            if(dialog.execute())
            {
                DccFile dcc;
                dcc.path = getUniqueName(FXPath::directory(dialog.getFilename()), FXPath::title(dialog.getFilename()), FXPath::extension(dialog.getFilename())); // dirty (owerwrite user opinion) but absolutely needed :)
                dcc.speed = 0;
                dcc.currentPosition = 0;
                dcc.size = FXLongVal(ev->param4);
                dcc.type = DCC_IN;
                dcc.started = FALSE;
                dcc.canceled = FALSE;
                dcc.finished = FALSE;
                dcc.finishedPosition = 0;
                dcc.token = -1;
                dcc.ip = ev->param2.before('@');
                dcc.port = FXIntVal(ev->param2.after('@'));
                dcc.nick = ev->param1;
                for(FXint i=0; i<m_dccfilesList.no(); i++)
                {
                    if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                    {
                        m_dccfilesList.erase(i);
                        break;
                    }
                }
                //old .part file have to be deleted
                if(FXStat::exists(dcc.path+".part"))
                    FXFile::remove(dcc.path+".part");
                m_dccfilesList.append(dcc);
                DccEngine *engine = new DccEngine(m_app, this, dcc, server);
                m_dccengines.append(engine);
                engine->startConnection();
                appendIrcStyledText(FXStringFormat(_("Receiving \"%s\" from %s"), ev->param3.text(), ev->param1.text()),8,FALSE);
                onCmdTransfers(NULL, 0, NULL);
            }
        }
    }
}

//handle IrcEvent IRC_DCCOUT
void dxirc::onIrcDccOut(IrcEngine *server, IrcEvent *ev)
{
    DccFile dcc;
    dcc.path = ev->param2;
    dcc.speed = 0;
    dcc.currentPosition = 0;
    dcc.size = FXStat::size(ev->param2);
    dcc.type = DCC_OUT;
    dcc.started = FALSE;
    dcc.canceled = FALSE;
    dcc.finished = FALSE;
    dcc.finishedPosition = 0;
    dcc.ip = server->getLocalIP();
    dcc.port = 0;
    dcc.token = -1;
    dcc.nick = ev->param1;
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(dcc.path == m_dccfilesList[i].path && dcc.type == m_dccfilesList[i].type)
        {
            m_dccfilesList.erase(i);
            break;
        }
    }
    m_dccfilesList.append(dcc);
    DccEngine *engine = new DccEngine(m_app, this, dcc, server);
    m_dccengines.append(engine);
    engine->startConnection();
    onCmdTransfers(NULL, 0, NULL);
}

//handle IrcEvent IRC_DCCPOUT
void dxirc::onIrcDccPout(IrcEngine *server, IrcEvent *ev)
{
    DccFile dcc;
    dcc.path = ev->param2;
    dcc.speed = 0;
    dcc.currentPosition = 0;
    dcc.size = FXStat::size(ev->param2);
    dcc.type = DCC_POUT;
    dcc.started = FALSE;
    dcc.canceled = FALSE;
    dcc.finished = FALSE;
    dcc.finishedPosition = 0;
    if(m_lastToken==32767) m_lastToken=0;
    dcc.token = ++m_lastToken;
    dcc.nick = ev->param1;
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(dcc.path == m_dccfilesList[i].path && dcc.type == m_dccfilesList[i].type)
        {
            m_dccfilesList.erase(i);
            break;
        }
    }
    m_dccfilesList.append(dcc);
    server->sendCtcp(ev->param1, "DCC SEND "+dxutils::removeSpaces(dcc.path.rafter(PATHSEP))+" "+server->getLocalIPBinary()+" 0 "+FXStringVal(dcc.size)+" "+FXStringVal(dcc.token));
}

//handle IrcEvent IRC_DCCMYTOKEN
void dxirc::onIrcDccMyToken(IrcEngine *server, IrcEvent *ev)
{
    FXint token = FXIntVal(ev->param3);
    FXint index = -1;
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(m_dccfilesList[i].token == token && m_dccfilesList[i].type == DCC_POUT)
        {
            index = i;
            break;
        }
    }
    if(index == -1)
        return;
    m_dccfilesList[index].ip = ev->param1;
    m_dccfilesList[index].port = FXIntVal(ev->param2);
    DccEngine *engine = new DccEngine(m_app, this, m_dccfilesList[index], server);
    m_dccengines.append(engine);
    engine->startConnection();
    onCmdTransfers(NULL, 0, NULL);
}

//handle IrcEvent IRC_DCCTOKEN
void dxirc::onIrcDccToken(IrcEngine *server, IrcEvent *ev)
{
    if(m_preferences.m_autoDccFile)
    {
        DccFile dcc;
        dcc.path = getUniqueName(m_preferences.m_dccPath, FXPath::stripExtension(ev->param2), FXPath::extension(ev->param2));
        dcc.speed = 0;
        dcc.currentPosition = 0;
        dcc.size = FXLongVal(ev->param3);
        dcc.type = DCC_PIN;
        dcc.started = FALSE;
        dcc.canceled = FALSE;
        dcc.finished = FALSE;
        dcc.finishedPosition = 0;
        dcc.token = FXIntVal(ev->param4);
        dcc.nick = ev->param1;
        dcc.ip = server->getLocalIP();
        dcc.port = 0;
        for(FXint i=0; i<m_dccfilesList.no(); i++)
        {
            if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
            {
                m_dccfilesList.erase(i);
                break;
            }
        }
        m_dccfilesList.append(dcc);
        if(isForResume(FXPath::name(dcc.path)))
        {
            server->sendCtcp(ev->param1, "DCC RESUME "+FXPath::name(dcc.path)+" 0 "+FXStringVal(FXStat::size(dcc.path+".part"))+" "+FXStringVal(dcc.token));
        }
        else
        {
            DccEngine *engine = new DccEngine(m_app, this, dcc, server);
            m_dccengines.append(engine);
            engine->startConnection();
            appendIrcStyledText(FXStringFormat(_("Receiving \"%s\" from %s"), ev->param2.text(), ev->param1.text()),8,FALSE);
        }
    }
    else
    {
        if(isForResume(ev->param2)
                && dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("%s offers file %s with size %s over DCC passive.\nFile is already partially downloaded.\nDo you want continue in download?"), ev->param1.text(), ev->param2.text(), dxutils::getFileSize(ev->param3).text()) == 1)
        {
            DccFile dcc;
            dcc.path = m_preferences.m_dccPath+PATHSEPSTRING+ev->param2;
            dcc.speed = 0;
            dcc.currentPosition = 0;
            dcc.size = FXLongVal(ev->param3);
            dcc.type = DCC_PIN;
            dcc.started = FALSE;
            dcc.canceled = FALSE;
            dcc.finished = FALSE;
            dcc.finishedPosition = 0;
            dcc.token = FXIntVal(ev->param4);
            dcc.nick = ev->param1;
            for(FXint i=0; i<m_dccfilesList.no(); i++)
            {
                if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                {
                    m_dccfilesList.erase(i);
                    break;
                }
            }
            m_dccfilesList.append(dcc);
            server->sendCtcp(ev->param1, "DCC RESUME "+ev->param2+" 0 "+FXStringVal(FXStat::size(dcc.path+".part"))+" "+FXStringVal(dcc.token));
            return;
        }
        if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("%s offers file %s with size %s over DCC passive.\n Do you want accept?"), ev->param1.text(), ev->param2.text(), dxutils::getFileSize(ev->param3).text()) == 1)
        {
            dxEXFileDialog dialog(this, _("Save file"));
            dialog.setFilename(m_preferences.m_dccPath+PATHSEPSTRING+ev->param2);
            if(dialog.execute())
            {
                DccFile dcc;
                dcc.path = getUniqueName(FXPath::directory(dialog.getFilename()), FXPath::title(dialog.getFilename()), FXPath::extension(dialog.getFilename())); // dirty (owerwrite user opinion) but absolutely needed :)
                dcc.speed = 0;
                dcc.currentPosition = 0;
                dcc.size = FXLongVal(ev->param3);
                dcc.type = DCC_PIN;
                dcc.started = FALSE;
                dcc.canceled = FALSE;
                dcc.finished = FALSE;
                dcc.finishedPosition = 0;
                dcc.token = FXIntVal(ev->param4);
                dcc.nick = ev->param1;
                dcc.ip = server->getLocalIP();
                dcc.port = 0;
                for(FXint i=0; i<m_dccfilesList.no(); i++)
                {
                    if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                    {
                        m_dccfilesList.erase(i);
                        break;
                    }
                }
                //old .part file have to be deleted
                if(FXStat::exists(dcc.path+".part"))
                    FXFile::remove(dcc.path+".part");
                m_dccfilesList.append(dcc);
                DccEngine *engine = new DccEngine(m_app, this, dcc, server);
                m_dccengines.append(engine);
                engine->startConnection();
                appendIrcStyledText(FXStringFormat(_("Receiving \"%s\" from %s"), ev->param2.text(), ev->param1.text()),8,FALSE);
                onCmdTransfers(NULL, 0, NULL);
            }
        }
    }
}

//handle IrcEvent IRC_DCCPOSITION
void dxirc::onIrcDccPosition(DccEngine*, IrcEvent *ev)
{
    FXint index = -1;
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(m_dccfilesList[i].path == ev->dccFile.path)
        {
            index = i;
            break;
        }
    }
    if(index == -1) return;
    m_dccfilesList[index].speed = ev->dccFile.speed;
    m_dccfilesList[index].currentPosition = ev->dccFile.currentPosition;
    m_dccfilesList[index].finishedPosition = ev->dccFile.finishedPosition;
    m_dccfilesList[index].canceled = ev->dccFile.canceled;
    m_dccfilesList[index].ip = ev->dccFile.ip;
    m_dccfilesList[index].port = ev->dccFile.port;
    m_dccfilesList[index].finished = ev->dccFile.finished;
    if(m_dccfilesList[index].type == DCC_IN  || m_dccfilesList[index].type == DCC_PIN)
    {
        if(m_dccfilesList[index].currentPosition >= m_dccfilesList[index].size)
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(FXStringFormat(_("Download of \"%s\" from %s canceled"), FXPath::name(m_dccfilesList[index].path).text(), m_dccfilesList[index].nick.text()), 4, FALSE);
            }
            else
            {
                appendIrcStyledText(FXStringFormat(_("Download of \"%s\" from %s finished"), FXPath::name(m_dccfilesList[index].path).text(), m_dccfilesList[index].nick.text()), 8, FALSE);
#ifdef HAVE_TRAY
                if(m_trayIcon && m_trayIcon->getIcon() != ICO_NEWFILE && !shown())
                    m_trayIcon->setIcon(ICO_NEWFILE);
#endif //HAVE_TRAY
            }
        }
        else
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(FXStringFormat(_("Download of \"%s\" from %s canceled"), FXPath::name(m_dccfilesList[index].path).text(), m_dccfilesList[index].nick.text()), 4, FALSE);
            }
        }
    }
    else
    {
        if(m_dccfilesList[index].finishedPosition >= m_dccfilesList[index].size)
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(FXStringFormat(_("Upload of \"%s\" to %s canceled"), FXPath::name(m_dccfilesList[index].path).text(), m_dccfilesList[index].nick.text()), 4, FALSE);
            }
            else
            {
                appendIrcStyledText(FXStringFormat(_("Upload of \"%s\" to %s finished"), FXPath::name(m_dccfilesList[index].path).text(), m_dccfilesList[index].nick.text()), 8, FALSE);
#ifdef HAVE_TRAY
                if(m_trayIcon && m_trayIcon->getIcon() != ICO_NEWFILE && !shown())
                    m_trayIcon->setIcon(ICO_NEWFILE);
#endif //HAVE_TRAY
            }
        }
        else
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(FXStringFormat(_("Upload of \"%s\" to %s canceled"), FXPath::name(m_dccfilesList[index].path).text(), m_dccfilesList[index].nick.text()), 4, FALSE);
            }
        }
    }
}

//handle IrcEvent IRC_DCCRESUME
void dxirc::onIrcDccResume(IrcEngine *server, IrcEvent *ev)
{
    FXint engineIndex = -1;
    for(FXint i=0; i<m_dccengines.no(); i++)
    {
        if(m_dccengines[i]->isForResume(ev->param1, ev->param2, FXIntVal(ev->param3)))
        {
            engineIndex = i;
            break;
        }
    }
    if(engineIndex == -1) return;
    FXint dccIndex = -1;
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(m_dccfilesList[i].path == m_dccengines[engineIndex]->getDccFile().path
                && m_dccfilesList[i].size == m_dccengines[engineIndex]->getDccFile().size
                && m_dccfilesList[i].nick == m_dccengines[engineIndex]->getDccFile().nick)
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    FXulong position = FXULongVal(ev->param4);
    if(position >= m_dccfilesList[dccIndex].size) return;
    m_dccengines[engineIndex]->setDccPosition(position);
    m_dccfilesList[dccIndex].ip = m_dccengines[engineIndex]->getDccFile().ip;
    m_dccfilesList[dccIndex].port = m_dccengines[engineIndex]->getDccFile().port;
    m_dccfilesList[dccIndex].currentPosition = position;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = position;
    m_dccfilesList[dccIndex].canceled = FALSE;
    server->sendCtcp(ev->param1, "DCC ACCEPT "+dxutils::removeSpaces(m_dccengines[engineIndex]->getDccFile().path.rafter(PATHSEP))+" "+FXStringVal(m_dccengines[engineIndex]->getDccFile().port)+" "+ev->param4);
}

//handle IrcEvent IRC_DCCPRESUME
void dxirc::onIrcDccPresume(IrcEngine *server, IrcEvent *ev)
{
    FXint dccIndex = -1;
    FXint token = FXIntVal(ev->param1);
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(m_dccfilesList[i].token == token)
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    FXulong position = FXULongVal(ev->param2);
    if(position >= m_dccfilesList[dccIndex].size) return;
    m_dccfilesList[dccIndex].currentPosition = position;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = position;
    m_dccfilesList[dccIndex].canceled = FALSE;
    server->sendCtcp(m_dccfilesList[dccIndex].nick, "DCC ACCEPT "+dxutils::removeSpaces(m_dccfilesList[dccIndex].path.rafter(PATHSEP))+" 0 "+ev->param2+" "+FXStringVal(token));
}

//handle IrcEvent IRC_DCCACCEPT
void dxirc::onIrcDccAccept(IrcEngine *server, IrcEvent *ev)
{
    FXint dccIndex = -1;
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(m_dccfilesList[i].nick == ev->param1 && FXPath::name(m_dccfilesList[i].path) == ev->param2
                && m_dccfilesList[i].port == FXIntVal(ev->param3))
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    FXulong position = FXULongVal(ev->param4);
    if(position >= m_dccfilesList[dccIndex].size) return;
    m_dccfilesList[dccIndex].currentPosition = position;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = position;
    m_dccfilesList[dccIndex].canceled = FALSE;
    DccEngine *engine = new DccEngine(m_app, this, m_dccfilesList[dccIndex], server);
    m_dccengines.append(engine);
    engine->startConnection();
    appendIrcStyledText(FXStringFormat(_("Receiving \"%s\" from %s"), FXPath::name(m_dccfilesList[dccIndex].path).text(), m_dccfilesList[dccIndex].nick.text()),8,false);
    if(!m_preferences.m_autoDccFile) onCmdTransfers(NULL, 0, NULL);
}

//handle IrcEvent IRC_DCCPACCEPT
void dxirc::onIrcDccPaccept(IrcEngine *server, IrcEvent *ev)
{
    FXint dccIndex = -1;
    FXint token = FXIntVal(ev->param1);
    for(FXint i=0; i<m_dccfilesList.no(); i++)
    {
        if(m_dccfilesList[i].token == token)
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    FXulong position = FXULongVal(ev->param2);
    if(position >= m_dccfilesList[dccIndex].size) return;
    m_dccfilesList[dccIndex].currentPosition = position;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = position;
    m_dccfilesList[dccIndex].canceled = FALSE;
    m_dccfilesList[dccIndex].ip = server->getLocalIP();
    m_dccfilesList[dccIndex].port = 0;
    DccEngine *engine = new DccEngine(m_app, this, m_dccfilesList[dccIndex], server);
    m_dccengines.append(engine);
    engine->startConnection();
    appendIrcStyledText(FXStringFormat(_("Receiving \"%s\" from %s"), FXPath::name(m_dccfilesList[dccIndex].path).text(), m_dccfilesList[dccIndex].nick.text()),8,FALSE);
    if(!m_preferences.m_autoDccFile) onCmdTransfers(NULL, 0, NULL);
}

//handle IrcEvent IRC_BOATSINVITE
void dxirc::onIrcBoatsInvite(IrcEngine *server, IrcEvent *ev)
{
    if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"),
                             _("%s invite you into Boats game.\n Do you want play?"), ev->param1.text())
            == 1)
    {
        createBoatsTab(ev->param1, 0, server, FALSE);
        server->sendCtcp(ev->param1, "BOATS ACCEPT");
    }
    else
    {
        server->sendCtcp(ev->param1, "BOATS DECLINE");
    }
}

//handle IrcEvent IRC_BOATSACCEPT
void dxirc::onIrcBoatsAccept(IrcEngine *server, IrcEvent *ev)
{
    createBoatsTab(ev->param1, 0, server, TRUE);
}

//handle IrcEvent IRC_RAW
void dxirc::onIrcRaw(IrcEngine *server, IrcEvent *ev)
{
#ifdef HAVE_LUA
    if(!m_scripts.no() || !m_scriptEvents.no()) return;
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("raw", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if(lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, ev->param1.text());
                        lua_newtable(m_scripts[j].L);
                        lua_pushstring(m_scripts[j].L, "server");
                        lua_pushstring(m_scripts[j].L, server->getServerName().text());
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "port");
                        lua_pushnumber(m_scripts[j].L, server->getServerPort());
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "nick");
                        lua_pushstring(m_scripts[j].L, server->getNickName().text());
                        lua_settable(m_scripts[j].L, -3);
                        if(lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

//handle IrcEvent IRC_323
void dxirc::onIrc323(IrcEngine *server)
{
    ListDialog dialog(this, server);
    dialog.execute(PLACEMENT_CURSOR);
}

long dxirc::onTabBook(FXObject *, FXSelector, void *ptr)
{
    FXint index = (FXint)(FXival)ptr*2;
    if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&IrcTabItem::metaClass)
    {
        IrcTabItem *currenttab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index));
        dxUtils.debugLine(FXStringFormat("OnTabBook(%d), Class: %s, id: %d", index, currenttab->getClassName(), currenttab->getID()));
        if(m_preferences.m_appTheme.fore != currenttab->getTextColor()) currenttab->setTextColor(m_preferences.m_appTheme.fore);
        if(currenttab->getType() == CHANNEL && currenttab->getIcon() == ICO_CHANNELNEWMSG)
        {
            currenttab->setIcon(ICO_CHANNEL);
#ifdef HAVE_TRAY
            if(m_trayIcon && m_trayIcon->getIcon() == ICO_NEWMSG)
                m_trayIcon->setIcon(ICO_TRAY);
#endif //HAVE_TRAY
        }
        if(currenttab->getType() == QUERY && currenttab->getIcon() == ICO_QUERYNEWMSG)
        {
            currenttab->setIcon(ICO_QUERY);
#ifdef HAVE_TRAY
            if(m_trayIcon && m_trayIcon->getIcon() == ICO_NEWMSG)
                m_trayIcon->setIcon(ICO_TRAY);
#endif //HAVE_TRAY
        }
        currenttab->setFocus();
        currenttab->setCommandFocus();
        currenttab->makeLastRowVisible();
        if(hasTetrisTab())
        {
            TetrisTabItem *tetristab = static_cast<TetrisTabItem*>(getTabItemById(getTabId("tetris")));
            if(tetristab && tetristab->isPauseEnable() && !tetristab->isPaused()) tetristab->pauseResumeGame();
        }
        if(currenttab->getType() == SERVER)
            updateStatus(currenttab->getRealServerName()+"-"+currenttab->getNickName());
        else if(currenttab->getType() == OTHER)
            updateStatus(currenttab->getText());
        else
            updateStatus((currenttab->getText()[0]=='&' ? "&"+currenttab->getText(): currenttab->getText())+"-"+currenttab->getRealServerName()+"-"+currenttab->getNickName());
    }
    if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&DccTabItem::metaClass)
    {
        DccTabItem *currenttab = static_cast<DccTabItem*>(m_tabbook->childAtIndex(index));
        dxUtils.debugLine(FXStringFormat("OnTabBook(%d), Class: %s, id: %d", index, currenttab->getClassName(), currenttab->getID()));
        if(m_preferences.m_appTheme.fore != currenttab->getTextColor()) currenttab->setTextColor(m_preferences.m_appTheme.fore);
        if(currenttab->getIcon() == ICO_DCCNEWMSG)
            currenttab->setIcon(ICO_DCC);
#ifdef HAVE_TRAY
        if(m_trayIcon && m_trayIcon->getIcon() == ICO_NEWMSG)
            m_trayIcon->setIcon(ICO_TRAY);
#endif //HAVE_TRAY
        currenttab->setFocus();
        currenttab->setCommandFocus();
        currenttab->makeLastRowVisible();
        if(hasTetrisTab())
        {
            TetrisTabItem *tetristab = static_cast<TetrisTabItem*>(getTabItemById(getTabId("tetris")));
            if(tetristab && tetristab->isPauseEnable() && !tetristab->isPaused()) tetristab->pauseResumeGame();
        }
        updateStatus((currenttab->getText()[0]=='&' ? "&"+currenttab->getText(): currenttab->getText())+"-"+currenttab->getServerName()+"-"+currenttab->getNickName());
    }
    if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&BoatsTabItem::metaClass)
    {
        BoatsTabItem *currenttab = static_cast<BoatsTabItem*>(m_tabbook->childAtIndex(index));
        dxUtils.debugLine(FXStringFormat("OnTabBook(%d), Class: %s, id: %d", index, currenttab->getClassName(), currenttab->getID()));
        currenttab->setFocus();
        currenttab->setCommandFocus();
        if(hasTetrisTab())
        {
            TetrisTabItem *tetristab = static_cast<TetrisTabItem*>(getTabItemById(getTabId("tetris")));
            if(tetristab && tetristab->isPauseEnable() && !tetristab->isPaused()) tetristab->pauseResumeGame();
        }
        updateStatus(currenttab->getText()+"-"+currenttab->getRealServerName()+"-"+currenttab->getNickName());
    }
    if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&TetrisTabItem::metaClass)
    {
        TetrisTabItem *tetristab = static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(index));
        dxUtils.debugLine(FXStringFormat("OnTabBook(%d), Class: %s, id: %d", index, tetristab->getClassName(), tetristab->getID()));
        tetristab->setFocus();
        tetristab->setGameFocus();
    }
    return 1;
}

long dxirc::onCmdNextTab(FXObject *, FXSelector, void *)
{
    FXint index = m_tabbook->getCurrent();
    if(m_tabbook->numChildren())
    {
        if((index+1)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index+1, m_tabbook->numChildren() > index*2 ? TRUE : FALSE);
        else m_tabbook->setCurrent(0, TRUE);
    }
    return 1;
}

//handle move tab
long dxirc::onCmdMoveTab(FXObject*, FXSelector, void *ptr)
{
    if(!m_preferences.m_disableSortTabs)
        return 1;
    FXString key = dxutils::getKeybinding((FXEvent*)ptr);
    if(comparecase(key, m_preferences.getBinding("moveTabUp"))==0)
    {
        moveTab(true);
    }
    else if(comparecase(key, m_preferences.getBinding("moveTabDown"))==0)
    {
        moveTab(false);
    }
    return 1;
}

//Handle mousewheel for change currenttab
long dxirc::onMouseWheel(FXObject *, FXSelector, void *ptr)
{
    FXEvent *event = (FXEvent*)ptr;
    FXint index = m_tabbook->getCurrent();
    if(event->code > 0) //positive movement
    {
        if(m_tabbook->numChildren())
        {
            if((index-1) >= 0) m_tabbook->setCurrent(index-1, TRUE);
            else m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
        }
    }
    else
    {
        if(m_tabbook->numChildren())
        {
            if((index+1)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index+1, m_tabbook->numChildren() > index*2 ? TRUE : FALSE);
            else m_tabbook->setCurrent(0, TRUE);
        }
    }
    return 1;
}

long dxirc::onCmdNextUnread(FXObject *, FXSelector, void*)
{
    if(m_tabbook->numChildren())
    {
        for(FXint i = m_tabbook->getCurrent()*2; i<m_tabbook->numChildren(); i+=2)
        {
            if (m_preferences.m_appTheme.fore != static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getTextColor())
            {
                m_tabbook->setCurrent(i/2, TRUE);
                return 1;
            }
        }
        for(FXint i = m_tabbook->getCurrent()*2; i>-1; i-=2)
        {
            if (m_preferences.m_appTheme.fore != static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getTextColor())
            {
                m_tabbook->setCurrent(i/2, TRUE);
                return 1;
            }
        }
    }
    return 1;
}

long dxirc::onCmdClear(FXObject *, FXSelector, void *)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()!=&TetrisTabItem::metaClass
                && m_tabbook->childAtIndex(index)->getMetaClass()!=&BoatsTabItem::metaClass)
        {
            dxTabItem *currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(index));
            currenttab->clearChat();
        }
    }
    return 1;
}

long dxirc::onCmdClearAll(FXObject *, FXSelector, void *)
{
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass
                && m_tabbook->childAtIndex(i)->getMetaClass()!=&BoatsTabItem::metaClass)
        {
            static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->clearChat();
            if (m_preferences.m_appTheme.fore != static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getTextColor()) static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->setTextColor(m_preferences.m_appTheme.fore);
            if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == CHANNEL) static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->setIcon(ICO_CHANNEL);
            if(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getType() == QUERY) static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->setIcon(ICO_QUERY);
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&DccTabItem::metaClass) static_cast<DccTabItem*>(m_tabbook->childAtIndex(i))->setIcon(ICO_DCC);
        }
    }
#ifdef HAVE_TRAY
    if(m_trayIcon && m_trayIcon->getIcon() == ICO_NEWMSG)
        m_trayIcon->setIcon(ICO_TRAY);
#endif //HAVE_TRAY
    return 1;
}

long dxirc::onCmdCloseTab(FXObject *, FXSelector, void *)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(!m_tabbook->childAtIndex(index)) return 1;
        if(m_tabbook->childAtIndex(index)->getMetaClass()==&IrcTabItem::metaClass)
        {
            IrcTabItem *currenttab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index));
            if(currenttab->getType() == OTHER)
            {
                delete m_tabbook->childAtIndex(index);
                delete m_tabbook->childAtIndex(index);
                m_tabbook->recalc();
                sortTabs();
                if(m_tabbook->numChildren())
                {
                    m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
                }
                updateMenus();
                return 1;
            }
            IrcEngine *currentserver = NULL;
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(currenttab))
                {
                    currentserver = m_ircengines[i];
                    break;
                }
            }
            if(currentserver == NULL) return 0;
            if(currenttab->getType() == QUERY)
            {
                if(currentserver->getConnected() && isLastTab(currentserver))
                {
                    currenttab->setType(SERVER, currentserver->getNetworkName());
                }
                else
                {
                    if(!currentserver->getConnected()) currentserver->disconnect();
                    currentserver->removeTarget(currenttab);
                    delete m_tabbook->childAtIndex(index);
                    delete m_tabbook->childAtIndex(index);
                    m_tabbook->recalc();

                }
                sortTabs();
                if(m_tabbook->numChildren())
                {
                    m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
                }
                updateMenus();
                return 1;
            }
            if(currenttab->getType() == CHANNEL)
            {
                if(currentserver->getConnected()) currentserver->sendPart(currenttab->getText());
                if(currentserver->getConnected() && isLastTab(currentserver))
                {
                    currenttab->setType(SERVER, currentserver->getNetworkName());
                }
                else
                {
                    if(!currentserver->getConnected()) currentserver->disconnect();
                    currentserver->removeTarget(currenttab);
                    delete m_tabbook->childAtIndex(index);
                    delete m_tabbook->childAtIndex(index);
                    m_tabbook->recalc();
                }
                sortTabs();
                if(m_tabbook->numChildren())
                {
                    m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
                }
                updateMenus();
                return 1;
            }
            if(currenttab->getType() == SERVER)
            {
                currentserver->disconnect();
                for(FXint i = m_tabbook->numChildren()-2; i > -1; i-=2)
                {
                    if(currentserver->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))))
                    {
                        currentserver->removeTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i)));
                        delete m_tabbook->childAtIndex(i);
                        delete m_tabbook->childAtIndex(i);
                    }
                }
                m_tabbook->recalc();
                sortTabs();
                if(m_tabbook->numChildren())
                {
                    m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
                }
                updateMenus();
                return 1;
            }
        }
        else if(m_tabbook->childAtIndex(index)->getMetaClass()==&DccTabItem::metaClass)
        {
            DccTabItem *dcctab = static_cast<DccTabItem*>(m_tabbook->childAtIndex(index));
            dcctab->disconnect();
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(dcctab))
                {
                    m_ircengines[i]->removeTarget(dcctab);
                    break;
                }
            }
            delete m_tabbook->childAtIndex(index);
            delete m_tabbook->childAtIndex(index);
            m_tabbook->recalc();
            sortTabs();
            if(m_tabbook->numChildren())
            {
                m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
            }
            updateMenus();
            return 1;
        }
        else if(m_tabbook->childAtIndex(index)->getMetaClass()==&BoatsTabItem::metaClass)
        {
            BoatsTabItem *tab = static_cast<BoatsTabItem*>(m_tabbook->childAtIndex(index));
            tab->leftGame();
            IrcEngine *currentserver = NULL;
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(tab))
                {
                    currentserver = m_ircengines[i];
                    break;
                }
            }
            if(currentserver != NULL) currentserver->removeTarget(tab);
            delete m_tabbook->childAtIndex(index);
            delete m_tabbook->childAtIndex(index);
            m_tabbook->recalc();
            sortTabs();
            if(m_tabbook->numChildren())
            {
                m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
            }
            updateMenus();
            return 1;
        }
        else
        {
            TetrisTabItem *tetristab = static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(index));
            tetristab->stopGame();
            delete m_tabbook->childAtIndex(index);
            delete m_tabbook->childAtIndex(index);
            m_tabbook->recalc();
            sortTabs();
            if(m_tabbook->numChildren())
            {
                m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
            }
            updateMenus();
            return 1;
        }
    }
    return 1;
}

long dxirc::onCmdSelectTab(FXObject*, FXSelector, void *ptr)
{
    FXint index = 0;
    FXString key = dxutils::getKeybinding((FXEvent*)ptr);
    if(comparecase(key, m_preferences.getBinding("firstTab"))==0)
        index = 0;
    else if(comparecase(key, m_preferences.getBinding("secondTab"))==0)
        index = 1;
    else if(comparecase(key, m_preferences.getBinding("thirdTab"))==0)
        index = 2;
    else if(comparecase(key, m_preferences.getBinding("fourthTab"))==0)
        index = 3;
    else if(comparecase(key, m_preferences.getBinding("fifthTab"))==0)
        index = 4;
    else if(comparecase(key, m_preferences.getBinding("sixthTab"))==0)
        index = 5;
    else if(comparecase(key, m_preferences.getBinding("seventhTab"))==0)
        index = 6;
    else if(comparecase(key, m_preferences.getBinding("eighthTab"))==0)
        index = 7;
    else if(comparecase(key, m_preferences.getBinding("ninethTab"))==0)
        index = 8;
    if((index)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index, TRUE);
    return 1;
}

long dxirc::onTrayClicked(FXObject*, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(shown())
        hide();
    else
    {
        show();
        if(m_trayIcon && m_trayIcon->getIcon() == ICO_NEWFILE) onCmdTransfers(NULL, 0, NULL);
    }
    if(m_trayIcon && m_trayIcon->getIcon() != ICO_TRAY)
        m_trayIcon->setIcon(ICO_TRAY);
#endif //HAVE_TRAY
    return 1;
}

//handle highlighted msg from IrcTabITem::ID_NEWMSG or DccTabItem::ID_NEWMSG
long dxirc::onNewMsg(FXObject *obj, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(m_trayIcon && m_trayIcon->getIcon() == ICO_TRAY && (!shown() || static_cast<dxTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2)) != static_cast<dxTabItem*>(obj)))
        m_trayIcon->setIcon(ICO_NEWMSG);
#endif //HAVE_TRAY
    if(m_preferences.m_sounds && m_preferences.m_soundMessage && (!shown() || isMinimized() || static_cast<dxTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2)) != static_cast<dxTabItem*>(obj)))
        dxutils::playFile(m_preferences.m_pathMessage);
    if(static_cast<IrcTabItem*>(obj)->getType() == CHANNEL)
    {
        updateStatus(FXStringFormat(_("New highlighted message on %s"), static_cast<IrcTabItem*>(obj)->getText().text()));
        if(m_preferences.m_notify && m_preferences.m_notifyMessage)
            showNotify(FXStringFormat(_("New highlighted message on %s"), static_cast<IrcTabItem*>(obj)->getText().text()));
    }
    else
    {
        updateStatus(FXStringFormat(_("New message on chat with %s"), static_cast<dxTabItem*>(obj)->getText().text()));
        if(m_preferences.m_notify && m_preferences.m_notifyMessage)
            showNotify(FXStringFormat(_("New message on chat with %s"), static_cast<dxTabItem*>(obj)->getText().text()));
    }
    flash(TRUE);
    return 1;
}

//handle for: /ignore addcmd
long dxirc::onAddIgnoreCommand(FXObject *sender, FXSelector, void *data)
{
    if(sender->getMetaClass()!=&IrcTabItem::metaClass) return 0;
    IrcTabItem *tab = static_cast<IrcTabItem*>(sender);
    FXString text = static_cast<FXString*>(data)->lower().text();
    FXString available[11] = { "away", "ban", "ctcp", "invite", "join", "me", "nick", "notice", "mode", "part", "quit"};
    FXbool canAdd = FALSE;
    for(FXint i=0; i<11; i++)
    {
        if(comparecase(text,available[i])==0)
        {
            canAdd = TRUE;
            break;
        }
    }
    if(!canAdd)
    {
        tab->appendStyledText(FXStringFormat(_("'%s' can't be added to ignored commands"), text.text()), 4, FALSE, FALSE, FALSE);
        return 1;
    }
    else
    {
        for(FXint i=0; i<m_preferences.m_ignoreCommandsList.contains(';'); i++)
        {
            if(comparecase(text,m_preferences.m_ignoreCommandsList.section(';', i))==0)
            {
                tab->appendStyledText(FXStringFormat(_("'%s' is already added in ignored commands"), text.text()), 4, FALSE, FALSE, FALSE);
                return 1;
            }
        }
        tab->appendStyledText(FXStringFormat(_("'%s' was added to ignored commands"), text.text()), 3, FALSE, FALSE, FALSE);
        m_preferences.m_ignoreCommandsList.append(text+";");
    }
    return 1;
}

//handle for: /ignore rmcmd
long dxirc::onRemoveIgnoreCommand(FXObject *sender, FXSelector, void *data)
{
    if(sender->getMetaClass()!=&IrcTabItem::metaClass) return 0;
    IrcTabItem *tab = static_cast<IrcTabItem*>(sender);
    FXString text = static_cast<FXString*>(data)->lower().text();
    FXString tempList = "";
    FXbool inCommands = FALSE;
    for(FXint i=0; i<m_preferences.m_ignoreCommandsList.contains(';'); i++)
    {
        if(comparecase(text,m_preferences.m_ignoreCommandsList.section(';', i))==0)
        {
            tab->appendStyledText(FXStringFormat(_("'%s' was removed from ignored commands"), text.text()), 3, FALSE, FALSE, FALSE);
            inCommands = TRUE;
        }
        else
            tempList.append(m_preferences.m_ignoreCommandsList.section(';', i)+";");
    }
    if(!inCommands) tab->appendStyledText(FXStringFormat(_("'%s' isn't in ignored commands"), text.text()), 4, FALSE, FALSE, FALSE);
    m_preferences.m_ignoreCommandsList = tempList;
    return 1;
}

//handle for: /ignore addusr
long dxirc::onAddIgnoreUser(FXObject *sender, FXSelector, void *data)
{
    if(sender->getMetaClass()!=&IrcTabItem::metaClass) return 0;
    IrcTabItem *tab = static_cast<IrcTabItem*>(sender);
    FXString text = static_cast<FXString*>(data)->text();
    FXString user = text.section(' ',0);
    FXString channel = text.section(' ',1);
    FXString network = dxutils::getParam(text, 3, TRUE);
    if(m_preferences.m_ignoreUsersList.no())
    {
        FXbool updated = FALSE;
        for(FXint i=0; i<m_preferences.m_ignoreUsersList.no(); i++)
        {
            if(compare(user, m_preferences.m_ignoreUsersList[i].nick)==0)
            {
                updated = TRUE;
                channel.empty() ? m_preferences.m_ignoreUsersList[i].channel = "all" : m_preferences.m_ignoreUsersList[i].channel = channel;
                network.empty() ? m_preferences.m_ignoreUsersList[i].network = "all" : m_preferences.m_ignoreUsersList[i].network = network;
                tab->appendStyledText(FXStringFormat(_("'%s' was updated in ignored users"), user.text()), 3, FALSE, FALSE, FALSE);
                break;
            }
        }
        if(!updated)
        {
            UserInfo iuser;
            iuser.nick = user;
            channel.empty() ? iuser.channel = "all" : iuser.channel = channel;
            network.empty() ? iuser.network = "all" : iuser.network = network;
            m_preferences.m_ignoreUsersList.append(iuser);
            tab->appendStyledText(FXStringFormat(_("'%s' was added to ignored users"), user.text()), 3, FALSE, FALSE, FALSE);
        }
    }
    else
    {
        UserInfo iuser;
        iuser.nick = user;
        channel.empty() ? iuser.channel = "all" : iuser.channel = channel;
        network.empty() ? iuser.network = "all" : iuser.network = network;
        m_preferences.m_ignoreUsersList.append(iuser);
        tab->appendStyledText(FXStringFormat(_("'%s' was added to ignored users"), user.text()), 3, FALSE, FALSE, FALSE);
    }
    return 1;
}

//handle for: /ignore rmusr
long dxirc::onRemoveIgnoreUser(FXObject *sender, FXSelector, void *data)
{
    if(sender->getMetaClass()!=&IrcTabItem::metaClass) return 0;
    IrcTabItem *tab = static_cast<IrcTabItem*>(sender);
    FXString text = static_cast<FXString*>(data)->text();
    FXbool updated = FALSE;
    if(m_preferences.m_ignoreUsersList.no())
    {
        for(FXint i=m_preferences.m_ignoreUsersList.no()-1; i>-1; i--)
        {
            if(compare(text, m_preferences.m_ignoreUsersList[i].nick)==0)
            {
                updated = TRUE;
                m_preferences.m_ignoreUsersList.erase(i);
                tab->appendStyledText(FXStringFormat(_("'%s' was removed from ignored users"), text.text()), 3, FALSE, FALSE, FALSE);
                break;
            }
        }
        if(!updated)
        {
            tab->appendStyledText(FXStringFormat(_("'%s' wasn't removed from ignored users"), text.text()), 3, FALSE, FALSE, FALSE);
        }
    }
    return 1;
}

long dxirc::onNewTetris(FXObject*, FXSelector, void*)
{
    if(hasTetrisTab()) return 1;
    TetrisTabItem *tab = new TetrisTabItem(m_tabbook, "tetris", 0, TAB_TOP, m_lastID);
    sendNewTab(NULL, "tetris", m_lastID, TRUE, FALSE, OTHER);
    m_lastID++;
    tab->create();
    tab->createGeom();
    tab->setColor(m_preferences.m_colors);
    updateTabPosition();
    sortTabs();
    updateMenus();
    m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
    return 1;
}

long dxirc::onKeyPress(FXObject *sender, FXSelector sel, void *ptr)
{
    FXEvent* event = (FXEvent*)ptr;
    if(m_tabbook->numTabs() && m_tabbook->childAtIndex(m_tabbook->getCurrent()*2)->getMetaClass()==&TetrisTabItem::metaClass)
    {
        switch(event->code){
            case KEY_N:
            case KEY_n:
            {
                static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2))->newGame();
                break;
            }
            case KEY_P:
            case KEY_p:
            {
                static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2))->pauseResumeGame();
                break;
            }
            case KEY_I:
            case KEY_i:
            {
                static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2))->rotate();
                break;
            }
            case KEY_L:
            case KEY_l:
            {
                static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2))->moveRight();
                break;
            }
            case KEY_K:
            case KEY_k:
            {
                static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2))->drop();
                break;
            }
            case KEY_J:
            case KEY_j:
            {
                static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(m_tabbook->getCurrent()*2))->moveLeft();
                break;
            }
        }
    }
    if(FXTopWindow::onKeyPress(sender, sel, ptr))
        return 1;
    return 0;
}

void dxirc::autoloadScripts()
{
#ifdef HAVE_LUA
    if(m_preferences.m_autoload && FXStat::exists(m_preferences.m_autoloadPath))
    {
        FXDir dir;
        FXString name, pathname;
        FXStat info;
        FXint islink;
        // Assume not a link
        islink = FALSE;
        // Managed to open directory
        if (dir.open(m_preferences.m_autoloadPath))
        {
            // Process directory entries
            while (dir.next())
            {
                // Get name of entry
                name = dir.name();
                // A dot special file?
                if (name[0] == '.' && (name[1] == 0 || (name[1] == '.' && name[2] == 0))) continue;
                // Hidden file or directory normally not shown
                if (name[0] == '.') continue;
                // Build full pathname of entry
                pathname = m_preferences.m_autoloadPath;
                if (!ISPATHSEP(pathname[pathname.length() - 1])) pathname += PATHSEPSTRING;
                pathname += name;
#ifndef WIN32
                // Get file/link info
                if (!FXStat::statLink(pathname, info)) continue;
                // If its a link, get the info on file itself
                islink = info.isLink();
                if (islink && !FXStat::statFile(pathname, info)) continue;
#else
                // Get file/link info
                if (!FXStat::statFile(pathname, info)) continue;
                // Hidden file or directory normally not shown
                if (info.isHidden()) continue;
#endif //WIN32
                // If it is a directory
                if(info.isDirectory()) continue;
                // If it is not matching pattern skip it
                if (!FXPath::match("*.lua", name))continue;
                loadLuaScript(pathname, FALSE);
            }
            // Close it
            dir.close();
        }
    }
#endif //HAVE_LUA
}

long dxirc::onCmdScripts(FXObject*, FXSelector, void*)
{
#ifdef HAVE_LUA
    ScriptDialog *dialog = new ScriptDialog(this);
    dialog->execute(PLACEMENT_OWNER);
#endif //HAVE_LUA
    return 1;
}

//fired from tab by command /lua
long dxirc::onLua(FXObject *obj, FXSelector, void *data)
{
#ifdef HAVE_LUA
    LuaRequest *lua = (LuaRequest*)data;
    if(lua->type == LUA_LOAD)
    {
        return loadLuaScript(lua->text);
    }
    if(lua->type == LUA_UNLOAD)
    {
        return unloadLuaScript(lua->text);
    }
    if(lua->type == LUA_LIST)
    {
        if(!m_scripts.no())
        {
            appendIrcStyledText(_("Scripts aren't loaded"), 4, FALSE);
            return 0;
        }
        else
        {
            appendIrcStyledText(_("Loaded scrips:"), 7, FALSE);
            for(FXint i=0; i<m_scripts.no(); i++)
            {
                appendIrcText(FXStringFormat(_("Name: %s"), m_scripts[i].name.text()), FALSE);
                appendIrcText(FXStringFormat(_("Description: %s"), m_scripts[i].description.text()), FALSE);
                appendIrcText(FXStringFormat(_("Version: %s"), m_scripts[i].version.text()), FALSE);
                appendIrcText(FXStringFormat(_("Path: %s"), m_scripts[i].path.text()), FALSE);
                appendIrcText(dxUtils.availableScriptCommands(m_scripts[i].name), FALSE);
                if(i+1<m_scripts.no()) appendIrcText("", FALSE);
            }
        }
        return 1;
    }
    if(lua->type == LUA_COMMAND)
    {
        dxTabItem *tab = static_cast<dxTabItem*>(obj);
        FXString command = lua->text.before(' ');
        FXString text = lua->text.after(' ');
        for(FXint i=0; i<m_scripts.no(); i++)
        {
            if(comparecase(dxUtils.getScriptName(command), m_scripts[i].name) == 0)
            {
                lua_getglobal(m_scripts[i].L, dxUtils.getFuncname(command).text());
                if(lua_isfunction(m_scripts[i].L, -1))
                {
                    lua_pushstring(m_scripts[i].L, text.text());
                    lua_pushnumber(m_scripts[i].L, tab->getID());
                    if(lua_pcall(m_scripts[i].L, 2, 0, 0)) appendIrcStyledText(FXStringFormat(_("Error: %s"), lua_tostring(m_scripts[i].L, -1)), 4, FALSE);
                }
                else lua_pop(m_scripts[i].L, 1);
                return 1;
            }
        }
    }
#else
    appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, FALSE);
#endif //HAVE_LUA
    return 1;
}

//fired from tab command /log show
long dxirc::onShowLog(FXObject *sender, FXSelector, void *data)
{
    dxTabItem *tab = static_cast<dxTabItem*>(sender);
    FXint id = getTabId(tab->getText());
    if(id==-1) id = createIrcTab(tab->getText(), ICO_LOGS, OTHER, NULL);
    IrcTabItem *logtab = static_cast<IrcTabItem*>(getTabItemById(id));
    logtab->showLog(static_cast<FXString*>(data)->text());
    setCurrentTabById(id);
    return 1;
}

//notify hides
long dxirc::onNotifyHide(FXObject*, FXSelector, void*)
{
    if(m_firstNotify)
        m_firstNotify = !m_firstNotify;
    return 1;
}

long dxirc::onCmdList(FXObject *, FXSelector, void *)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&TetrisTabItem::metaClass) return 0;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&DccTabItem::metaClass) return 0;
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&IrcTabItem::metaClass)
        {
            IrcTabItem *currenttab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(index));
            IrcEngine *currentserver = NULL;
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(currenttab))
                {
                    currentserver = m_ircengines[i];
                    break;
                }
            }
            if(currentserver == NULL) return 0;
            if(currentserver->getConnected())
            {
                FXString param = "";
                if(dxEXInputDialog::getString(param, this, _("Question"), _("Parameters of /LIST command")+FXString("\n")+currentserver->getNetworkName()))
                    currentserver->sendList(param);
            }
        }
        if(m_tabbook->childAtIndex(index) && m_tabbook->childAtIndex(index)->getMetaClass()==&BoatsTabItem::metaClass)
        {
            BoatsTabItem *currenttab = static_cast<BoatsTabItem*>(m_tabbook->childAtIndex(index));
            IrcEngine *currentserver = NULL;
            for(FXint i=0; i < m_ircengines.no(); i++)
            {
                if(m_ircengines[i]->findTarget(currenttab))
                {
                    currentserver = m_ircengines[i];
                    break;
                }
            }
            if(currentserver == NULL) return 0;
            if(currentserver->getConnected())
            {
                FXString param = "";
                if(dxEXInputDialog::getString(param, this, _("Question"), _("Parameters of /LIST command")+FXString("\n")+currentserver->getNetworkName()))
                    currentserver->sendList("");
            }
        }
    }
    return 1;
}

//Handle for entered text in IrcTab for ownmsg in lua scripting
long dxirc::onIrcOwnMsg(FXObject *sender, FXSelector, void *data)
{
#ifdef HAVE_LUA
    if(sender->getMetaClass()==&TetrisTabItem::metaClass) return 0;
    dxTabItem *tab = static_cast<dxTabItem*>(sender);
    FXString *text = static_cast<FXString*>(data);
    if(!m_scripts.no() || !m_scriptEvents.no())
    {
        tab->setHasOwnMsg(FALSE);
        return 0;
    }
    tab->setHasOwnMsg(hasOwnMsg());
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("ownmsg", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, text->text());
                        lua_pushinteger(m_scripts[j].L, tab->getID());
                        if (lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
    return 1;
}

//send event to script for new created tab
void dxirc::sendNewTab(IrcEngine *server, const FXString &name, FXint id, FXbool isTetris, FXbool isDccTab, TYPE type)
{
#ifdef HAVE_LUA
    if(!m_scripts.no() || !m_scriptEvents.no())
        return;
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("newtab", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_newtable(m_scripts[j].L);
                        lua_pushstring(m_scripts[j].L, "id");
                        lua_pushinteger(m_scripts[j].L, id);
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "name");
                        lua_pushstring(m_scripts[j].L, name.text());
                        lua_settable(m_scripts[j].L, -3);
                        if(isTetris)
                        {
                            lua_pushstring(m_scripts[j].L, "type");
                            lua_pushstring(m_scripts[j].L, "tetris");
                            lua_settable(m_scripts[j].L, -3);
                        }
                        else if(isDccTab)
                        {
                            lua_pushstring(m_scripts[j].L, "type");
                            lua_pushstring(m_scripts[j].L, "dccchat");
                            lua_settable(m_scripts[j].L, -3);
                        }
                        else
                        {
                            switch(type) {
                                case SERVER:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "server");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case CHANNEL:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "channel");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case QUERY:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "query");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case OTHER:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "other");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case DCC:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "dcc");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case BOATS:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "boats");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                            }
                        }
                        lua_pushstring(m_scripts[j].L, "servername");
                        lua_pushstring(m_scripts[j].L, server ? server->getServerName().text() : "");
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "port");
                        lua_pushinteger(m_scripts[j].L, server ? server->getServerPort() : 0);
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "nick");
                        lua_pushstring(m_scripts[j].L, server ? server->getNickName().text() : "");
                        lua_settable(m_scripts[j].L, -3);
                        if (lua_pcall(m_scripts[j].L, 1, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

//Handle for entered command in IrcTab for all in lua scripting
long dxirc::onIrcCommand(FXObject *sender, FXSelector, void *data)
{
#ifdef HAVE_LUA
    if(sender->getMetaClass()==&TetrisTabItem::metaClass) return 0;
    dxTabItem *tab = static_cast<dxTabItem*>(sender);
    FXString *commandtext = static_cast<FXString*>(data);
    if(!m_scripts.no() || !m_scriptEvents.no())
    {
        tab->setHasAllCommand(FALSE);
        return 0;
    }
    tab->setHasAllCommand(hasAllCommand());
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("all", m_scriptEvents[i].name) == 0)
        {
            for(FXint j=0; j<m_scripts.no(); j++)
            {
                if(comparecase(m_scriptEvents[i].script, m_scripts[j].name) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.text());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        if(commandtext->at(0) == '/')
                        {
                            lua_pushstring(m_scripts[j].L, commandtext->before(' ').after('/').text());
                            lua_pushstring(m_scripts[j].L, commandtext->after(' ').text());
                        }
                        else
                        {
                            lua_pushnil(m_scripts[j].L);
                            lua_pushstring(m_scripts[j].L, commandtext->text());
                        }
                        lua_pushinteger(m_scripts[j].L, tab->getID());
                        if (lua_pcall(m_scripts[j].L, 3, 0, 0))
                        {
                            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling %s %s"), m_scriptEvents[i].funcname.text(), lua_tostring(m_scripts[j].L, -1)), 4, FALSE);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
    return 1;
}

FXbool dxirc::tabExist(IrcEngine *server, FXString name, TYPE type)
{
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getType()==type && server
                && server->findTarget(m_tabbook->childAtIndex(i))
                && comparecase(static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getText(), name) == 0)
            return TRUE;
    }
    return FALSE;
}

FXbool dxirc::serverExist(FXString server, FXint port, FXString nick)
{
    for(FXint i = 0; i < m_ircengines.no(); i++)
    {
        if(m_ircengines[i]->getServerName().lower() == server.lower() && m_ircengines[i]->getServerPort() == port && m_ircengines[i]->getNickName().lower() == nick.lower() && m_ircengines[i]->getConnected()) return TRUE;
    }
    return FALSE;
}

FXint dxirc::getServerTab(IrcEngine *server)
{
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass)
        {
            IrcTabItem *tab = static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i));
            if(server->findTarget(tab) && tab->getType() == SERVER) return i;
        }
    }
    return -1;
}

FXint dxirc::getTabId(IrcEngine *server, FXString name)
{
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))) && comparecase(name, static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getText()) == 0)
            return static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i))->getID();
    }
    return -1;
}

//usefull only for tetristab, othertab
FXint dxirc::getTabId(FXString name)
{
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(comparecase(name, static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getText()) == 0)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&TetrisTabItem::metaClass)
                return static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(i))->getID();
            else
                if(static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getType()==OTHER) return static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getID();
        }
    }
    return -1;
}

FXint dxirc::getCurrentTabId()
{
    FXint index = m_tabbook->getCurrent()*2;
    if(!m_tabbook->childAtIndex(index)) return -1;
    if(m_tabbook->childAtIndex(index)->getMetaClass()==&TetrisTabItem::metaClass)
        return static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(index))->getID();
    else
        return static_cast<dxTabItem*>(m_tabbook->childAtIndex(index))->getID();
    return -1;
}

void dxirc::setCurrentTabById(FXint id)
{
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass
                && static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getID() == id)
        {
            m_tabbook->setCurrent(i/2, TRUE);
            return;
        }
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&TetrisTabItem::metaClass
                && static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(i))->getID() == id)
        {
            m_tabbook->setCurrent(i/2, TRUE);
            return;
        }
    }
}

FXWindow* dxirc::getTabItemById(FXint id)
{
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass
                && static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getID() == id)
        {
            return m_tabbook->childAtIndex(i);
        }
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&TetrisTabItem::metaClass
                && static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(i))->getID() == id)
        {
            return m_tabbook->childAtIndex(i);
        }
    }
    return NULL;
}

FXbool dxirc::isValidTabId(FXint id)
{
    if(id<0) return FALSE;
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&TetrisTabItem::metaClass)
        {
            if(static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(i))->getID()==id) return TRUE;
        }
        else
        {
            if(static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getID()==id) return TRUE;
        }
    }
    return FALSE;
}

FXbool dxirc::isIddxTabItem(FXint id)
{
    if(id<0) return FALSE;
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
        {
            if(static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->getID()==id) return TRUE;
        }
    }
    return FALSE;
}

FXbool dxirc::isFriend(FXString nick, FXString on, FXString network)
{
    FXbool bnick = FALSE;
    FXbool bchannel = FALSE;
    FXbool bnetwork = FALSE;
    for(FXint i=0; i<m_preferences.m_friendsList.no(); i++)
    {
        FXString fnick = m_preferences.m_friendsList[i].nick;
        if(FXRex(FXString("\\<"+fnick+"\\>").lower().substitute("*","\\w*")).match(nick.lower())) bnick = TRUE;
        if(bnick)
        {
            if(m_preferences.m_friendsList[i].channel == "all") bchannel = TRUE;
            if(m_preferences.m_friendsList[i].channel.contains(','))
            {
                for(FXint j=1; j<m_preferences.m_friendsList[i].channel.contains(',')+2; j++)
                {
                    if(FXRex(FXString(dxutils::getParam(m_preferences.m_friendsList[i].channel, j, FALSE, ',').lower()+"\\>").substitute("*","\\w*")).match(on.lower()))
                    {
                        bchannel = TRUE;
                        break;
                    }
                }
            }
            else
            {
                if(FXRex(FXString(m_preferences.m_friendsList[i].channel+"\\>").lower().substitute("*","\\w*")).match(on.lower())) bchannel = TRUE;
            }
            if(m_preferences.m_friendsList[i].network == "all") bnetwork = TRUE;
            if(FXRex(FXString("\\<"+m_preferences.m_friendsList[i].network+"\\>").lower().substitute("*","\\w*")).match(network.lower())) bnetwork = TRUE;
            break;
        }
    }
    return bnick && bchannel && bnetwork;
}

FXint dxirc::createIrcTab(const FXString& tabtext, FXIcon* icon, TYPE typ, IrcEngine* engine)
{
    FXuint style;
    switch(m_preferences.m_tabPosition) {
        case 0: //bottom
            {
                style = TAB_BOTTOM;
            }break;
        case 1: //left
            {
                style = TAB_LEFT;
            }break;
        case 2: //top
            {
                style = TAB_TOP;
            }break;
        case 3: //right
            {
                style = TAB_RIGHT;
            }break;
        default:
            {
                style = TAB_BOTTOM;
            }
    }
    IrcTabItem *tabitem = new IrcTabItem(m_tabbook, tabtext, icon, style, m_lastID, typ, engine, m_ircFont, m_preferences.m_themePath!="internal");
    sendNewTab(engine, tabtext, m_lastID, FALSE, FALSE, typ);
    m_lastID++;
    tabitem->create();
    tabitem->createGeom();
    tabitem->setSmileys(m_preferences.m_useSmileys, m_smileys);
    if(engine) engine->appendTarget(tabitem);
    sortTabs();
    return m_lastID-1;
}

void dxirc::createDccTab(const FXString &mynick, const FXString &nick, const FXString &address, FXint portD, FXint portH, FXbool listen, IrcEngine *engine)
{
    FXuint style;
    switch(m_preferences.m_tabPosition) {
        case 0: //bottom
            {
                style = TAB_BOTTOM;
            }break;
        case 1: //left
            {
                style = TAB_LEFT;
            }break;
        case 2: //top
            {
                style = TAB_TOP;
            }break;
        case 3: //right
            {
                style = TAB_RIGHT;
            }break;
        default:
            {
                style = TAB_BOTTOM;
            }
    }
    DccTabItem *tabitem = new DccTabItem(m_tabbook, mynick, nick, address, portD, portH, listen, ICO_DCC, style, m_lastID, m_ircFont, engine);
    sendNewTab(NULL, nick, m_lastID, FALSE, TRUE, DCC);
    m_lastID++;
    tabitem->create();
    tabitem->createGeom();
    tabitem->setSmileys(m_preferences.m_useSmileys, m_smileys);
    if(engine) engine->appendTarget(tabitem);
    sortTabs();
}

void dxirc::createBoatsTab(const FXString &tabtext, FXIcon *icon, IrcEngine *engine, FXbool first)
{
    FXuint style;
    switch(m_preferences.m_tabPosition) {
        case 0: //bottom
            {
                style = TAB_BOTTOM;
            }break;
        case 1: //left
            {
                style = TAB_LEFT;
            }break;
        case 2: //top
            {
                style = TAB_TOP;
            }break;
        case 3: //right
            {
                style = TAB_RIGHT;
            }break;
        default:
            {
                style = TAB_BOTTOM;
            }
    }
    BoatsTabItem *tabitem = new BoatsTabItem(m_tabbook, tabtext, first, icon, style, m_lastID, engine);
    sendNewTab(engine, tabtext, m_lastID, FALSE, FALSE, BOATS);
    m_lastID++;
    tabitem->create();
    tabitem->createGeom();
    if(engine) engine->appendTarget(tabitem);
    sortTabs();
    setCurrentTabById(m_lastID-1);
}

FXbool dxirc::isLastTab(IrcEngine *server)
{
    FXint numTabs = 0;
    for(FXint i = 0; i < m_tabbook->numChildren(); i+=2)
    {
        if(server->findTarget(static_cast<IrcTabItem*>(m_tabbook->childAtIndex(i)))) numTabs++;
    }
    if(numTabs > 1) return FALSE;
    else return TRUE;
}

FXbool dxirc::hasTetrisTab()
{
    if(m_tabbook->numChildren())
    {
        for(FXint i = 0; i < m_tabbook->numChildren()/2; i++)
        {
            if(m_tabbook->childAtIndex(i*2)->getMetaClass()==&TetrisTabItem::metaClass) return TRUE;
        }
    }
    return FALSE;
}

void dxirc::sortTabs()
{
    if(m_preferences.m_disableSortTabs)
        return;
    if(m_tabbook->numTabs() > 1)
    {
        FXint id = getCurrentTabId();
        if(hasTetrisTab())
        {
            FXint index = 0;
            TetrisTabItem *tetristab = NULL;
            dxTabItem* *tabpole = new dxTabItem*[m_tabbook->numTabs()-1];
            for(FXint i = 0; i < m_tabbook->numTabs(); i++)
            {
                if(m_tabbook->childAtIndex(i*2)->getMetaClass()!=&TetrisTabItem::metaClass)
                {
                    tabpole[index] = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i*2));
                    index++;
                }
                else tetristab = static_cast<TetrisTabItem*>(m_tabbook->childAtIndex(i*2));
            }
            qsort(tabpole, m_tabbook->numTabs()-1, sizeof(tabpole[0]), (int(*)(const void*, const void*))&CompareTabs);
            for(FXint i = 0; i < m_tabbook->numTabs()-1; i++)
            {
                tabpole[i]->reparentTab();
            }
            tetristab->reparentTab();
            m_tabbook->recalc();
            delete []tabpole;
        }
        else
        {
            dxTabItem* *tabpole = new dxTabItem*[m_tabbook->numTabs()];
            for(FXint i = 0; i < m_tabbook->numTabs(); i++)
            {
                tabpole[i] = (dxTabItem*)m_tabbook->childAtIndex(i*2);
            }
            qsort(tabpole, m_tabbook->numTabs(), sizeof(tabpole[0]), (int(*)(const void*, const void*))&CompareTabs);
            for(int i=0; i < m_tabbook->numTabs(); i++)
            {
                tabpole[i]->reparentTab();
            }
            m_tabbook->recalc();
            delete []tabpole;
        }
        setCurrentTabById(id);
    }
}

void dxirc::moveTab(bool up)
{
    if(m_tabbook->numTabs()>1)
    {
        FXint id = getCurrentTabId();
        FXint index = m_tabbook->getCurrent();
        dxEXTabItem* *tabpole = new dxEXTabItem*[m_tabbook->numTabs()];
        for(FXint i = 0; i < m_tabbook->numTabs(); i++)
        {
            tabpole[i] = (dxEXTabItem*)m_tabbook->childAtIndex(i*2);
        }
        if(up)
        {
            if(index)
            {
                dxEXTabItem *temp = tabpole[index-1];
                tabpole[index-1] = tabpole[index];
                tabpole[index] = temp;
            }
            else
            {
                dxEXTabItem *temp = tabpole[index];
                for(FXint i = 0; i < m_tabbook->numTabs()-1; i++)
                {
                    tabpole[i] = tabpole[i+1];
                }
                tabpole[m_tabbook->numTabs()-1] = temp;
            }
        }
        else
        {
            if(index==m_tabbook->numTabs()-1)
            {
                dxEXTabItem *temp = tabpole[index];
                for(FXint i = m_tabbook->numTabs()-1; i > 0; i--)
                {
                    tabpole[i] = tabpole[i-1];
                }
                tabpole[0] = temp;
            }
            else
            {
                dxEXTabItem *temp = tabpole[index+1];
                tabpole[index+1] = tabpole[index];
                tabpole[index] = temp;
            }
        }
        for(FXint i = 0; i < m_tabbook->numTabs(); i++)
        {
            if(tabpole[i]->getMetaClass()!=&TetrisTabItem::metaClass)
                static_cast<dxTabItem*>(tabpole[i])->reparentTab();
            else
                static_cast<TetrisTabItem*>(tabpole[i])->reparentTab();

        }
        m_tabbook->recalc();
        delete []tabpole;
        setCurrentTabById(id);
    }
}

void dxirc::updateMenus()
{
    if(m_tabbook->numChildren())
    {
        m_closeTab->enable();
        m_clearTab->enable();
        m_clearTabs->enable();
    }
    else
    {
        m_closeTab->disable();
        m_clearTab->disable();
        m_clearTabs->disable();
    }
    FXbool someConnected = FALSE;
    for(FXint i = 0; i < m_ircengines.no(); i++)
    {
        if(m_ircengines[i]->getConnected())
        {
            someConnected = TRUE;
            break;
        }
    }
    if(someConnected)
    {
        m_disconnect->enable();
        m_reconnect->enable();
    }
    else
    {
        m_disconnect->disable();
        m_reconnect->disable();
    }
}

void dxirc::updateStatus(FXString text)
{
    m_statuslabel->setText(text);
#ifdef HAVE_TRAY
    if(m_trayIcon && m_preferences.m_useTray)
    {
        m_trayIcon->setText("dxirc\n"+text);
    }
#endif //HAVE_TRAY
    m_app->addTimeout(this, dxirc_STIMEOUT, 5000);
}

long dxirc::onStatusTimeout(FXObject*, FXSelector, void*)
{
    m_statuslabel->setText(" ");
#ifdef HAVE_TRAY
    if(m_trayIcon && m_preferences.m_useTray)
    {
        m_trayIcon->setText("dxirc");
    }
#endif //HAVE_TRAY
    return 1;
}

void dxirc::appendIrcText(FXString text, FXbool logLine)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(!m_tabbook->childAtIndex(index)) return;
        if(m_tabbook->childAtIndex(index)->getMetaClass()==&TetrisTabItem::metaClass)
        {
            for(FXint i=0; i<m_tabbook->numChildren(); i+=2)
            {
                if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
                {
                    index = i;
                    break;
                }
            }
        }
        dxTabItem *currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(index));
        FXASSERT(currenttab != 0);
        currenttab->appendText(text, TRUE, logLine);
        currenttab->makeLastRowVisible();
    }
}

void dxirc::appendIrcStyledText(FXString text, FXint style, FXbool logLine)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(!m_tabbook->childAtIndex(index)) return;
        if(m_tabbook->childAtIndex(index)->getMetaClass()==&TetrisTabItem::metaClass)
        {
            for(FXint i=0; i<m_tabbook->numChildren(); i+=2)
            {
                if(m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
                {
                    index = i;
                    break;
                }
            }
        }
        dxTabItem *currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(index));
        FXASSERT(currenttab != 0);
        currenttab->appendStyledText(text, style, TRUE, FALSE, logLine);
        currenttab->makeLastRowVisible();
    }
}

FXint dxirc::loadLuaScript(FXString path, FXbool showMessage)
{
#ifdef HAVE_LUA
    if(m_scripts.no())
    {
       for(FXint i=0; i<m_scripts.no(); i++)
        {
            if(comparecase(path, m_scripts[i].path)==0)
            {
                appendIrcStyledText(FXStringFormat(_("Script %s is already loaded"), path.text()), 4, FALSE);
                return 0;
            }
        }
    }
    if(hasLuaAll(path))
    {
        if(showMessage)
        {
            if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Question"), _("Script %s contains dxirc.AddAll\nThis can BREAK dxirc funcionality.\nLoad it anyway?"), path.text()) == 2) return 0;
        }
        else appendIrcStyledText(FXStringFormat(_("Script %s contains dxirc.AddAll. This can BREAK dxirc funcionality."), path.text()), 4, FALSE);
    }
    lua_State *L = luaL_newstate();
    if(L == NULL)
    {
        appendIrcStyledText(_("Unable to initialize Lua."), 4, FALSE);
        return 0;
    }
    luaL_openlibs(L);
#if LUA_VERSION_NUM > 501
    lua_newtable(L);
    lua_pushvalue(L, -1);
    lua_setglobal(L, "dxirc");
    luaL_setfuncs(L, dxircFunctions, 0);
#else
    luaL_register(L, "dxirc", dxircFunctions);
#endif
    if(luaL_dofile(L, path.text()))
    {
        appendIrcStyledText(FXStringFormat(_("Unable to load/run the file %s"), lua_tostring(L, -1)), 4, FALSE);
        return 0;
    }
    lua_getglobal(L, "dxirc_Register");
    if(lua_pcall(L, 0, 3, 0))
    {
        appendIrcStyledText(FXStringFormat(_("Lua plugin: error registering script %s"), lua_tostring(L, -1)), 4, FALSE);
        return 0;
    }
    FXString name = lua_tostring(L, -3);
    FXString version = lua_tostring(L, -2);
    FXString description = lua_tostring(L, -1);
    lua_pop(L, 4);
    if(m_scripts.no())
    {
       for(FXint i=0; i<m_scripts.no(); i++)
        {
            if(comparecase(name, m_scripts[i].name)==0)
            {
                appendIrcStyledText(FXStringFormat(_("Script with name %s is already loaded"), name.lower().text()), 4, FALSE);
                return 0;
            }
        }
    }
    LuaScript script;
    script.L = L;
    script.name = name;
    script.version = version;
    script.description = description;
    script.path = path;
    m_scripts.append(script);
    appendIrcStyledText(FXStringFormat(_("Script %s was loaded"), path.text()), 3, FALSE);
    appendIrcStyledText(FXStringFormat("%s: %s", script.name.text(), script.description.text()), 3, FALSE);
    lua_getglobal(L, "dxirc_Init");
    if (lua_type(L, -1) != LUA_TFUNCTION) lua_pop(L, 1);
    else
    {
        if (lua_pcall(L, 0, 0, 0))
        {
            appendIrcStyledText(FXStringFormat(_("Lua plugin: error calling dxirc_Init() %s"), lua_tostring(L, -1)), 4, FALSE);
            lua_pop(L, 1);
        }
    }
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

FXint dxirc::unloadLuaScript(FXString name)
{
#ifdef HAVE_LUA
    FXbool success = FALSE;
    if(!m_scripts.no())
    {
        appendIrcStyledText(FXStringFormat(_("Script %s isn't loaded"), name.text()), 4, FALSE);
        return 0;
    }
    else
    {
        for(FXint i=m_scripts.no()-1; i>-1; i--)
        {
            if(comparecase(name, m_scripts[i].name)==0)
            {
                dxUtils.removeScriptCommands(m_scripts[i].name);
                lua_close(m_scripts[i].L);
                m_scripts.erase(i);
                success = TRUE;
            }
        }
    }
    for(FXint i=m_scriptEvents.no()-1; i>-1; i--)
    {
        if(comparecase(name, m_scriptEvents[i].script)==0)
        {
            m_scriptEvents.erase(i);
            success = TRUE;
        }
    }
    if(success) appendIrcStyledText(FXStringFormat(_("Script %s was unloaded"), name.text()), 4, FALSE);
    else appendIrcStyledText(FXStringFormat(_("Script %s isn't loaded"), name.text()), 4, FALSE);
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

FXbool dxirc::hasLuaAll(const FXString &file)
{
    std::ifstream fin(file.text());
    std::string line;
    while(fin.good())
    {
        std::getline(fin, line);
        if(line.find("dxirc.AddEvent") != std::string::npos)
        {
            FXString fxline = FXString(line.c_str()).lower();
            if(fxline.contains("\"all\""))
                return TRUE;
        }
    }
    return FALSE;
}

//check for mymsg in loaded script
FXbool dxirc::hasOwnMsg()
{
    if(!m_scripts.no() || !m_scriptEvents.no())
    {
        return FALSE;
    }
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("ownmsg", m_scriptEvents[i].name) == 0)
        {
            return TRUE;
        }
    }
    return FALSE;
}

//check for all in loaded script
FXbool dxirc::hasAllCommand()
{
    if(!m_scripts.no() || !m_scriptEvents.no())
    {
        return FALSE;
    }
    for(FXint i=0; i<m_scriptEvents.no(); i++)
    {
        if(comparecase("all", m_scriptEvents[i].name) == 0)
        {
            return TRUE;
        }
    }
    return FALSE;
}

/*return unique filename
usefull mainly for autoDccFile */
FXString dxirc::getUniqueName(const FXString &path, const FXString &name, const FXString &extension)
{
    if(extension.empty())
    {
        if(!FXStat::exists(path+PATHSEPSTRING+name)) return path+PATHSEPSTRING+name;
        else return path+PATHSEPSTRING+name+"-"+FXSystem::time("%Y%m%d%H%M%S", FXSystem::now());
    }
    else
    {
        if(!FXStat::exists(path+PATHSEPSTRING+name+"."+extension)) return path+PATHSEPSTRING+name+"."+extension;
        else return path+PATHSEPSTRING+name+"-"+FXSystem::time("%Y%m%d%H%M%S", FXSystem::now())+"."+extension;
    }
}

//check if exist .part file in dccpath
FXbool dxirc::isForResume(const FXString& name)
{
    return FXStat::exists(m_preferences.m_dccPath+PATHSEPSTRING+name+".part");
}

int dxirc::onLuaAddCommand(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString name, funcname, helptext, script;
    if(lua_isstring(lua, 1)) name = lua_tostring(lua,1);
    if(lua_isstring(lua, 2)) funcname = lua_tostring(lua,2);
    if(lua_isstring(lua, 3)) helptext = lua_tostring(lua,3);
    if(name.empty() || funcname.empty() || helptext.empty()) return 0;
    if(dxUtils.isCommand(name))
    {
        _pThis->appendIrcStyledText(FXStringFormat(_("Command %s already exists"), name.text()), 4, FALSE);
        return 0;
    }
    if(_pThis->m_scripts.no())
    {
        for(FXint i=0; i<_pThis->m_scripts.no(); i++)
        {
            if(lua == _pThis->m_scripts[i].L) script = _pThis->m_scripts[i].name;
        }
    }
    if(script.empty()) return 0;
    LuaScriptCommand command;
    command.name = name;
    command.funcname = funcname;
    command.helptext = helptext;
    command.script = script;
    dxUtils.addScriptCommand(command);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaAddEvent(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString name, funcname, script;
    if(lua_isstring(lua, 1)) name = lua_tostring(lua,1);
    if(lua_isstring(lua, 2)) funcname = lua_tostring(lua,2);
    if(name.empty() || funcname.empty()) return 0;
    if(_pThis->m_scripts.no())
    {
        for(FXint i=0; i<_pThis->m_scripts.no(); i++)
        {
            if(lua == _pThis->m_scripts[i].L) script = _pThis->m_scripts[i].name;
        }
    }
    if(script.empty()) return 0;
    if(_pThis->m_scriptEvents.no())
    {
        for(FXint i=0; i<_pThis->m_scriptEvents.no(); i++)
        {
            if(comparecase(name, _pThis->m_scriptEvents[i].name)==0 && comparecase(funcname, _pThis->m_scriptEvents[i].funcname)==0 && comparecase(script, _pThis->m_scriptEvents[i].script)==0)
            {
                _pThis->appendIrcStyledText(FXStringFormat(_("Function %s for event %s already exists"), funcname.text(), name.text()), 4, FALSE);
                return 0;
            }
        }
    }
    LuaScriptEvent event;
    event.name = name;
    event.funcname = funcname;
    event.script = script;
    _pThis->m_scriptEvents.append(event);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaRemoveName(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString command, script;
    if(lua_isstring(lua, 1)) command = lua_tostring(lua, 1);
    if(command.empty()) return 0;
    if(dxUtils.removeScriptCommand(command)) return 1;
    if(_pThis->m_scripts.no())
    {
        for(FXint i=0; i<_pThis->m_scripts.no(); i++)
        {
            if(lua == _pThis->m_scripts[i].L) script = _pThis->m_scripts[i].name;
        }
    }
    if(script.empty()) return 0;
    if(_pThis->m_scriptEvents.no())
    {
        for(FXint i=0; i<_pThis->m_scriptEvents.no(); i++)
        {
            if(comparecase(command, _pThis->m_scriptEvents[i].name) == 0 && comparecase(script, _pThis->m_scriptEvents[i].script) == 0)
            {
                _pThis->m_scriptEvents.erase(i);
                _pThis->appendIrcStyledText(FXStringFormat(_("Command/event %s in script %s was removed"), command.text(), script.text()), 3, FALSE);
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaCommand(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString command;
    if(lua_isstring(lua, 1)) command = lua_tostring(lua, 1);
    if(command.empty()) return 0;
    FXint id;
    if(lua_isnumber(lua, 2) && _pThis->isValidTabId(lua_tointeger(lua, 2))) id = lua_tointeger(lua, 2);
    else id = _pThis->getCurrentTabId();
    if(_pThis->m_tabbook->numChildren())
    {
        for(FXint i = 0; i<_pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
            {
                dxTabItem *tab = static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i));
                if(id==tab->getID())
                {
                    tab->processLine(command);
                    return 1;
                }
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaPrint(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString text;
    if(lua_isstring(lua, 1)) text = lua_tostring(lua, 1);
    if(text.empty()) return 0;
    FXint id, style;
    if(lua_isnumber(lua, 2) && _pThis->isIddxTabItem(lua_tointeger(lua, 2))) id = lua_tointeger(lua, 2);
    else id = _pThis->getCurrentTabId();
    if(lua_isnumber(lua, 3))
    {
        style = lua_tointeger(lua, 3);
        if(style<0 || style>8) style = 0;
    }
    else style = 0;
    if(_pThis->m_tabbook->numChildren())
    {
        for(FXint i = 0; i<_pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
            {
                dxTabItem *tab = static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i));
                if(id==tab->getID())
                {
                    tab->appendStyledText(text, style, TRUE, TRUE, TRUE);
                    tab->makeLastRowVisible();
                    return 1;
                }
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetServers(lua_State *lua)
{
#ifdef HAVE_LUA
    if(_pThis->m_ircengines.no())
    {
        lua_newtable(lua);
        for(FXint i=0; i<_pThis->m_ircengines.no(); i++)
        {
            lua_pushnumber(lua, i+1);
            lua_newtable(lua);
            lua_pushstring(lua, "server");
            lua_pushstring(lua, _pThis->m_ircengines[i]->getServerName().text());
            lua_settable(lua, -3);
            lua_pushstring(lua, "port");
            lua_pushnumber(lua, _pThis->m_ircengines[i]->getServerPort());
            lua_settable(lua, -3);
            lua_pushstring(lua, "nick");
            lua_pushstring(lua, _pThis->m_ircengines[i]->getNickName().text());
            lua_settable(lua, -3);
            lua_settable(lua, -3);
        }
    }
    else
    {
        lua_newtable(lua);
        lua_pushnumber(lua, 1);
        lua_newtable(lua);
        lua_pushstring(lua, "server");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "port");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "nick");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_settable(lua, -3);
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetTab(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString tab, server;
    if(lua_isstring(lua, 1)) tab = lua_tostring(lua, 1);
    if(lua_isstring(lua, 2)) server = lua_tostring(lua,2);
    if(_pThis->m_tabbook->numChildren())
    {
        for(FXint i = 0; i<_pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
            {
                if(comparecase(tab, static_cast<FXTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getText()) == 0 && comparecase(server, static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getServerName()) == 0)
                {
                    lua_pushnumber(lua, static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getID());
                    return 1;
                }
            }
        }
        lua_pushnumber(lua, _pThis->getCurrentTabId());
        return 1;
    }
    else lua_pushnumber(lua, -1);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetCurrentTab(lua_State *lua)
{
#ifdef HAVE_LUA
    lua_pushnumber(lua, _pThis->getCurrentTabId());
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetVersion(lua_State *lua)
{
#ifdef HAVE_LUA
    lua_pushstring(lua, VERSION);
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetTabInfo(lua_State *lua)
{
#ifdef HAVE_LUA
    FXint id;
    if(lua_isnumber(lua, 1))  id = lua_tointeger(lua, 1);
    else id = -1;
    if(_pThis->m_tabbook->numChildren() && _pThis->isIddxTabItem(id))
    {
        dxTabItem *tab = NULL;
        for(FXint i = 0; i<_pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
            {
                tab = static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i));
                if(id==tab->getID())
                {
                    break;
                }
            }
        }
        lua_newtable(lua);
        lua_pushstring(lua, "name");
        lua_pushstring(lua, tab->getText().text());
        lua_settable(lua, -3);
        switch(tab->getType()) {
            case SERVER:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "server");
                lua_settable(lua, -3);
            }break;
            case CHANNEL:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "channel");
                lua_settable(lua, -3);
            }break;
            case QUERY:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "query");
                lua_settable(lua, -3);
            }break;
            case OTHER:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "other");
                lua_settable(lua, -3);
            }break;
            case DCC:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "dcc");
                lua_settable(lua, -3);
            }break;
            case BOATS:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "boats");
                lua_settable(lua, -3);
            }break;
        }
        if(tab->getMetaClass()==&DccTabItem::metaClass)
        {
            lua_pushstring(lua, "type");
            lua_pushstring(lua, "dccchat");
            lua_settable(lua, -3);
        }
        lua_pushstring(lua, "servername");
        lua_pushstring(lua, tab->getServerName().text());
        lua_settable(lua, -3);
        lua_pushstring(lua, "port");
        lua_pushinteger(lua, tab->getServerPort());
        lua_settable(lua, -3);
        lua_pushstring(lua, "nick");
        lua_pushstring(lua, tab->getNickName().text());
        lua_settable(lua, -3);
    }
    else
    {
        lua_newtable(lua);
        lua_pushstring(lua, "name");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "type");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "servername");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "port");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "nick");
        lua_pushnil(lua);
        lua_settable(lua, -3);
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaSetTab(lua_State *lua)
{
#ifdef HAVE_LUA
    _pThis->setCurrentTabById(lua_tointeger(lua, 1));
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaCreateTab(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString name;
    if(lua_isstring(lua, 1)) name = lua_tostring(lua, 1);
    else
    {
        lua_pushnil(lua);
        return 0;
    }
    if(_pThis->m_tabbook->numChildren())
    {
        for(FXint i = 0; i < _pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass
                    && comparecase(static_cast<FXTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getText(), name) == 0
                    && static_cast<IrcTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getType() == OTHER)
            {
                lua_pushnil(lua);
                return 0;
            }
        }
    }
    lua_pushnumber(lua, _pThis->createIrcTab(name, NULL, OTHER, NULL));
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetTabCount(lua_State *lua)
{
#ifdef HAVE_LUA
    lua_pushnumber(lua, _pThis->m_tabbook->numChildren()/2);
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaClear(lua_State *lua)
{
#ifdef HAVE_LUA
    FXint id = -1;
    if(lua_isnumber(lua, 1) && _pThis->isIddxTabItem(lua_tointeger(lua, 1)))  id = lua_tointeger(lua, 1);
    else return 0;
    dxutils::debugLine(FXStringFormat("onLuaClear id:%d",id));
    if(_pThis->m_tabbook->numChildren())
    {
        for(FXint i = 0; i < _pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass
                    && static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getID() == id)
            {
                static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i))->clearChat();
                return 1;
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetChannelNames(lua_State *lua)
{
#ifdef HAVE_LUA
    FXString tab, server;
    if(lua_isstring(lua, 1)) tab = lua_tostring(lua, 1);
    if(lua_isstring(lua, 2)) server = lua_tostring(lua,2);
    if(_pThis->m_tabbook->numChildren())
    {
        for(FXint i = 0; i<_pThis->m_tabbook->numChildren(); i+=2)
        {
            if(_pThis->m_tabbook->childAtIndex(i)->getMetaClass()!=&TetrisTabItem::metaClass)
            {
                if(comparecase(tab, static_cast<FXTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getText()) == 0
                    && comparecase(server, static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getServerName()) == 0
                    && static_cast<dxTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getType() == CHANNEL)
                {
                    lua_newtable(lua);
                    dxStringArray names = static_cast<IrcTabItem*>(_pThis->m_tabbook->childAtIndex(i))->getNicks();
                    for(FXint j=0; j<names.no(); j++)
                    {
                        lua_pushnumber(lua, j+1);
                        lua_pushstring(lua, names[j].text());
                        lua_settable(lua, -3);
                    }
                    return 1;
                }
            }
        }
        lua_pushnil(lua);
        return 1;
    }
    lua_pushnil(lua);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

//show notify
void dxirc::showNotify(const FXString& notify, FXint pos)
{
    if(dxUtils.haveNotify())
    {
        FXString exec;
        exec.format("notify-send -i %s/icons/big_dxirc.png dxirc '%s'", DXIRC_DATADIR, notify.text());
        system(exec.text());
    }
    else
    {
        if(!m_firstNotify)
        {
            m_firstNotify = true;
            m_notifiesHeight = 0;
        }
        if(pos<0 || pos>3) pos = m_preferences.m_notifyPosition;
        dxEXNotify *wnotify = new dxEXNotify(m_app, ICO_BIG, "dxirc", notify, m_notifiesHeight, 5000, m_preferences.m_appTheme.notifyfore, m_preferences.m_appTheme.notifyback);
        wnotify->notify(getX(), getY(), pos);
        getApp()->addTimeout(this, dxirc_NOTIFYHIDE, 5000);
        m_notifiesHeight += wnotify->getDefaultHeight()+4;
    }
}


#define USAGE_MSG _("\
\nUsage: dxirc [options] \n\
\n\
    [options] can be any of the following:\n\
\n\
        -h, --help         Print (this) help screen and exit.\n\
        -v, --version      Print version information and exit.\n\
        -l [FILE]          Load configuration from FILE.\n\
        -i [PATH]          Use PATH for icons.\n\
\n")

int main(int argc,char *argv[])
{
#ifndef WIN32
    signal(SIGPIPE, SIG_IGN);
#endif //WIN32

    argn = argc;
    args = argv;

    FXString datadir = DXIRC_DATADIR;

    for(FXint i=0; i<argc; ++i)
    {
        if(compare(argv[i],"-v")==0 || compare(argv[i],"--version")==0)
        {
            fprintf(stdout, "%s %s\n", PACKAGE, VERSION);
            exit(0);
        }
        if(compare(argv[i],"-h")==0 || compare(argv[i],"--help")==0)
        {
            fprintf(stdout, "%s", USAGE_MSG);
            exit(0);
        }
        if(compare(argv[i],"-l")==0)
        {
            dxUtils.setIniFile(argv[i+1]);
        }
        if(compare(argv[i],"-i")==0)
        {
            if(FXStat::exists(argv[i+1])) datadir = argv[i+1];
        }
    }

    if(dxUtils.getBoolIniEntry("SETTINGS", "oneinstance", FALSE))
        IsInstanceRunningAlready();

    dxUtils.loadAvailableSpellLangs();

#ifdef HAVE_TRAY
    FXTrayApp app(PACKAGE, FXString::null);
#else
    FXApp app(PACKAGE, FXString::null);
#endif //HAVE_TRAY
    app.reg().setAsciiMode(TRUE);
    app.init(argc,argv);
#ifdef ENABLE_NLS
    app.setTranslator(new dxTranslator(&app));
#endif //ENABLE_NLS
    makeAllIcons(&app, dxUtils.getIniFile(), datadir);
    new dxirc(&app);
    app.create();
    dxUtils.loadAlias();
    return app.run();
}
